/*
 * $Id: CollectionUtil.java,v 1.6 2005/10/25 18:18:24 suhrbier Exp $
 * Created on 08.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: CollectionUtil.java,v $
 * Revision 1.6  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.5  2005/10/24 09:38:39  suhrbier
 * Assignment test ok
 *
 * Revision 1.4  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/14 18:31:00  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/10 11:20:28  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.1  2005/10/08 18:29:38  suhrbier
 * Klasse CollecctionUtil eingefhrt.
 *
 */
package nbi.util;

import java.util.Collection;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * Utility functions about Collections
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class CollectionUtil {
    static Logger logger = Logger.getLogger(CollectionUtil.class);

    /**
     * Instantiates a collection if uninstantiated.
     * 
     * @param coll
     *            Collection to check for instantiation
     * @param type
     *            Class of the Collection to instanatiate.
     * @return if <code>coll</code> was <code>null</code>, the created
     *         Collection instance, otherwise the given <code>coll</code>
     *         instance.
     * @throws InstantiationException
     * @see Class#newInstance().
     * @throws IllegalAccessException
     * @see Class#newInstance().
     */
    static public Collection create(Collection coll, Class type)
            throws InstantiationException, IllegalAccessException {
        if (coll == null)
            coll = (Collection) type.newInstance();
        return coll;
    }

    /**
     * Adds an object to a collection. This function only adds non null objects
     * to instantiated collections. Uninstantiated collections provoke raising
     * of a NullPointerException.
     * 
     * @param coll
     *            Collection to add the object to.
     * @param obj
     *            The object to be added to the collection.
     * @return true, if <code>obj</code> was succesfully added to the
     *         collection, false otherwise.
     */
    static public boolean add(Collection coll, Object obj) {
        boolean ret = false;
        if (obj != null)
            ret = coll.add(obj);
        return ret;
    }

    /**
     * Adds a collection to a collection. This function only adds non null
     * collections to instantiated collections. Uninstantiated collections
     * provoke raising of a NullPointerException.
     * 
     * @param coll
     *            Collection to add the object to.
     * @param addColl
     *            The collection to be added to the collection.
     * @return true, if <code>obj</code> was succesfully added to the
     *         collection, false otherwise.
     */
    static public boolean addAll(Collection coll, Collection addColl) {
        boolean ret = false;
        if (addColl != null)
            ret = coll.addAll(addColl);
        return ret;
    }

    /**
     * Removes an object from a collection. This function only removes non null
     * objects from non null collections. Otherwise it returns false.
     * 
     * @param coll
     *            Collection to remove the object from.
     * @param obj
     *            The object to be removed the collection.
     * @return true, if <code>obj</code> was succesfully removed to the
     *         collection, false otherwise.
     */
    static public boolean remove(Collection coll, Object obj) {
        boolean ret = false;
        if (coll != null) {
            if (obj != null)
                ret = coll.remove(obj);
        }
        return ret;
    }

    /**
     * Removes a collection from a collection. This function only removes non
     * null collections from instantiated collections. Uninstantiated
     * collections provoke raising of a NullPointerException.
     * 
     * @param coll
     *            Collection to remove the collection from.
     * @param removeColl
     *            The collection to be removed from the collection.
     * @return true, if <code>obj</code> was succesfully removed from the
     *         collection, false otherwise.
     */
    static public boolean removeAll(Collection coll, Collection removeColl) {
        boolean ret = false;
        if (removeColl != null)
            ret = coll.removeAll(removeColl);
        return ret;
    }

    /**
     * Removes a elements from a collection. This function avoids
     * NullPointerExceptions by only clears non null collections.
     * 
     * @param coll
     *            Collection to clear.
     */
    static public void clear(Collection coll) {
        if (coll != null)
            coll.clear();
    }

    /**
     * Searches for an object in a collection which equals the given object.
     * Therefore, the function uses the object's
     * {@link Object#equals(Object) equals(Object)}method. Uninstantiated
     * collections provoke raising of a NullPointerException.
     * 
     * @param coll
     *            Collection to search the object for.
     * @param obj
     *            The object to be searched in the collection.
     * @return The first object in the collection equaling the given object,
     *         <code>null</code> otherwise.
     */
    static public Object search(Collection coll, Object obj) {
        Object result = null;
        if ((obj != null) && (coll.contains(obj))) {
            Iterator objIt = coll.iterator();
            while ((result==null)&&objIt.hasNext()) {
                result = objIt.next();
                result = obj.equals(result) ? result : null;
            }
        }
        return result;
    }

    /**
     * Compares collections for equality. Uses the
     * {@link Collection#equals(Object) Collection.equals(Object)}method to
     * compare. Returns <code>true</code> if both collection parameters are
     * null.
     * 
     * @param coll1
     *            test collection.
     * @param coll2
     *            test collection.
     * @return <code>true</code> if collections are equal or both null,
     *         <code>false</code> otherwise.
     */
    static public boolean equals(Collection coll1, Collection coll2) {
        boolean result = false;
        if ((coll1 == null) && (coll2 == null))
            result = true;
        else if ((coll1 != null)) {
            result = coll1.equals(coll2);
        }
        return result;

    }

}
