/*
 * $Id: MapUtil.java,v 1.1 2005/10/21 17:43:03 suhrbier Exp $
 * Created on 21.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: MapUtil.java,v $
 * Revision 1.1  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 */
package nbi.util;

import java.util.Map;

/**
 * Utility functions about Maps.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.1 $
 * @since
 */

public class MapUtil {
    /**
     * Instantiates a collection if uninstantiated.
     * 
     * @param map
     *            Map to check for instantiation
     * @param type
     *            Class of the Map to instantiate.
     * @return if <code>map</code> was <code>null</code>, the created
     *         Map instance, otherwise the given <code>map</code>
     *         instance.
     * @throws InstantiationException
     * @see Class#newInstance().
     * @throws IllegalAccessException
     * @see Class#newInstance().
     */
    static public Map create(Map map, Class type)
            throws InstantiationException, IllegalAccessException {
        if (map == null)
            map = (Map) type.newInstance();
        return map;
    }

    /**
     * Associates an object with a specified key in a map. This function only adds non null keys and objects
     * to instantiated maps. Uninstantiated maps provoke raising
     * of a NullPointerException.
     * 
     * @param map
     *            Map to put the object to.
     * @param key
     *            The key with which the object associated.
     * @param obj
     *            The object to be associated with the key in the map.
     * @return The previous object associated with the key or <code>null</code>, if there was no mapping for this key.
     */
    static public Object put(Map map, Object key, Object obj) {
        Object ret = null;
        if ((key != null) && (obj != null))
            ret = map.put(key,obj);
        return ret;
    }

    /**
     * Removes a mapping for this key from the map if present.
     * 
     * @param map
     *            Map to remove the mapping from.
     * @param key
     *            The key whose mapping is to be removed.
     * @return The previous object associated with the key or <code>null</code>, if there was no mapping for this key.
     */
    static public Object remove(Map map, Object key) {
        Object ret = null;
        if (map != null) {
            if (key != null)
                ret = map.remove(key);
        }
        return ret;
    }
    
}
