/*
 * $Id: ReflectionUtil.java,v 1.5 2006/07/25 14:59:04 suhrbier Exp $
 * Created on 05.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ReflectionUtil.java,v $
 * Revision 1.5  2006/07/25 14:59:04  suhrbier
 * update JavaDoc
 *
 * Revision 1.4  2006/07/12 09:42:41  suhrbier
 * Fehler behoben in PEPHandler bzgl. getRoles/getCertificates wenn kein clientCert-Parameter vorliegt.
 *
 * Revision 1.3  2006/05/15 14:24:50  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/10 11:20:28  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.1  2005/10/05 14:38:00  suhrbier
 * Klasse ReflectionUtil eingefhrt.
 *
 */
package nbi.util;

import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;

/**
 * ReflectionUtil is a utility container for functions around Java Reflection
 * Mechanism
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.5 $
 * @since
 */

public class ReflectionUtil {

    static Logger logger = Logger.getLogger(ReflectionUtil.class);

    /**
     * Creates a class instance with one parameter.
     * 
     * @param className
     *            Name of the class to instantiate (with packages).
     * @param parameterType
     *            The class type of the constructor argument.
     * @param parameter
     *            The constructor argument.
     * @return Instance of the class.
     * @throws ClassNotFoundException
     * @see Class#forName(java.lang.String).
     * @throws IllegalAccessException
     * @see Class#newInstance().
     * @throws SecurityException
     * @see Class#getConstructor(java.lang.Class[]).
     * @throws InstantiationException
     * @see Class#forName(java.lang.String) and
     * @see java.lang.reflect.Constructor#newInstance(java.lang.Object[])
     * @throws IllegalAccessException
     * @see java.lang.reflect.Constructor#newInstance(java.lang.Object[])
     * @throws InvocationTargetException
     * @see java.lang.reflect.Constructor#newInstance(java.lang.Object[])
     * @throws NoSuchMethodException
     * @see Class#getConstructor(java.lang.Class[]).
     */
    static public Object getInstance(String className, Class parameterType,
            Object parameter) throws IllegalArgumentException,
            SecurityException, ClassNotFoundException, InstantiationException,
            IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        Class[] paramTypes = new Class[] { parameterType };
        Object[] params = new Object[] { parameter };
        return getInstance(className, paramTypes, params);
    }

    /**
     * Creates a class instance with the given constructor parameters.
     * 
     * @param className
     *            Name of the class to instantiate(with packages).
     * @param parameterTypes
     *            Array with class types of the constructor arguments
     * @param parameters
     *            Array with constructor arguments
     * @return Instance of the class.
     * @throws ClassNotFoundException
     * @see Class#forName(java.lang.String).
     * @throws IllegalArgumentException
     * @see Class#newInstance().
     * @throws SecurityException
     * @see Class#getConstructor(java.lang.Class[]).
     * @throws InstantiationException
     * @see Class#forName(java.lang.String) and
     * @see Class#newInstance().
     * @throws IllegalAccessException
     * @see Class#newInstance().
     * @throws InvocationTargetException
     * @see Class#newInstance().
     * @throws NoSuchMethodException
     * @see Class#getConstructor(java.lang.Class[]).
     */
    static public Object getInstance(String className, Class[] parameterTypes,
            Object[] parameters) throws ClassNotFoundException,
            IllegalArgumentException, SecurityException,
            InstantiationException, IllegalAccessException,
            InvocationTargetException, NoSuchMethodException {
        Object result = null;
        if ((parameterTypes == null) || (parameterTypes.length == 0)
                || (parameters == null) || (parameters.length == 0))
            result = getInstance(className);
        else {
            Class instClass = Class.forName(className);
            result = instClass.getConstructor(parameterTypes).newInstance(
                    parameters);
        }
        return result;
    }

    /**
     * Create a class instance without constructor parameters.
     * 
     * @param className
     *            Name of the class to instantiate (with packages).
     * @return Instance of the class.
     * @throws ClassNotFoundException
     * @see Class#forName(java.lang.String).
     * @throws IllegalAccessException
     * @see Class#newInstance().
     * @throws InvocationTargetException
     * @see Class#newInstance().
     */
    static public Object getInstance(String className)
            throws ClassNotFoundException, InstantiationException,
            IllegalAccessException {
        return Class.forName(className).newInstance();
    }
}
