/*
 * $Id: SAXElement.java,v 1.3 2005/08/29 12:35:16 suhrbier Exp $
 * Created on 02.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: SAXElement.java,v $
 * Revision 1.3  2005/08/29 12:35:16  suhrbier
 * Neuer Provider fast lauffhig, XMLWriter schreibt jetzt ESC, Protocolfilter sendet Character Events nicht mehr aufgestaut weiter.
 *
 * Revision 1.2  2005/08/15 21:58:00  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.1  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.8  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.7  2005/08/07 16:15:32  suhrbier
 * Umstellung Request auf Filter
 *
 * Revision 1.6  2005/08/06 23:28:44  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.5  2005/08/05 15:09:43  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.4  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.3  2005/08/04 11:21:11  suhrbier
 * update protocolfilter
 *
 * Revision 1.2  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 * Revision 1.1  2005/08/02 14:26:56  suhrbier
 * First version
 *
 */
package nbi.xml.sax;

import java.net.URI;
import java.net.URISyntaxException;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * This class provides an Stack element to be used with a
 * {@link FilterStack FilterStack}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */
public class SAXElement {
	/** General empty attributes instance */
	final static public AttributesImpl EMPTY_ATTS = new AttributesImpl();
	
    /** Attributes of the element */
    private AttributesImpl attributes;

    /** Value of the element */
    private StringBuffer value;

    /** URI of the element */
    private URI uri;

    /** Local name of the element */
    private String localName;

    /** Qualified Name of the element */
    private String qName;

    /**
     * Inits an empty element instance with an empty Attributes instance.
     */
    public SAXElement() {
        super();
        attributes = new AttributesImpl();
    }

    /**
     * Inits the element instance.
     * 
     * @param uri
     *            element's URI or null
     * @param localName
     *            element's local name or null
     * @param qName
     *            element's qualified name or null
     * @param attributes
     *            element's attributes or null
     * @throws SAXException
     *             If the given URI string violates RFC 2396, see
     *             {@link URI#URI(String) URI(String)}.
     */
    public SAXElement(String uri, String localName, String qName,
            Attributes attributes) throws SAXException {
        super();
        setAttributes(attributes);
        setURI(uri);
        setLocalName(localName);
        setQName(qName);
    }

    public void clearAttributes()
    {
    	attributes.clear();
    }
    /**
     * Writes the element to a SAX {@link ContentHandler ContentHandler}.
     * @param out {@link ContentHandler ContentHandler} for XML output
     * @throws SAXException Any SAX exception, possibly wrapping another exception.
     */
    public void write(ContentHandler out) throws SAXException
    {
    	out.startElement(uri.toString(),localName,qName,attributes);
    	if( (value != null)&&(value.length()>0))
    		out.characters(value.toString().toCharArray(), 0, value.length());
    	out.endElement(uri.toString(), localName, qName);
    }
    
    /**
     * Appends value event to the element's value buffer. 
     * If the value buffer does not exist, it will be created.
     * @param ch An array of value.
     * @param start The starting position in the array.
     * @param length The number of value to use from the array.
     * 
     */
    public void appendCharacters(char[] ch, int start, int length) {
        if (value == null)
            value = new StringBuffer();
        value.append(ch, start, length);
    }

    public void clearValue() {
        if (value != null)
            value.delete(0,value.length());
    }

    /**
     * Gets the element attributes.
     * 
     * @return Returns the attributes.
     */
    public Attributes getAttributes() {
        return attributes;
    }

    /**
     * Sets new element attributes.
     * 
     * @param attributes
     *            The attributes to set.
     */
    public void setAttributes(Attributes attributes) {
        this.attributes = ( attributes == null ? new AttributesImpl() : new AttributesImpl(attributes));
    }

    /**
     * Adds a new Attribute. If no attributes exists, a new Attribute instance is created.
     * @see AttributesImpl#addAttribute(String, String, String, String, String)  
     */
    public void addAttribute(String uri, String localName, String qName, String type, String value )
    {
    	if(value!=null)
    	{
	    	if( attributes == null)
	    		attributes = new AttributesImpl();
	    	attributes.addAttribute(uri, localName, qName, type, value);
    	}
    }
    
    /**
     * Adds a new Attribute from the type "xsd:string".
     * @see #addAttribute(String, String, String, String, String)  
     */
    public void addAttribute(String uri, String localName, String qName, String value )
	{
    	if(value!=null)
    		addAttribute(uri,localName, qName, "xsd:string", value.toString());
	}

    /**
     * Adds a new Attribute from the type "xsd:integer.
     * @see #addAttribute(String, String, String, String, String)  
     */
    public void addAttribute(String uri, String localName, String qName, Integer value )
	{
    	if(value!=null)
    		addAttribute(uri,localName, qName, "xsd:integer", value.toString());
	}
    /**
     * Gets the element's value.
     * 
     * @return Returns the value.
     */
    public StringBuffer getValue() {
        return value;
    }

    /**
     * Sets the element's value.
     * 
     * @param value
     *            The value to set.
     */
    public void setValue(StringBuffer value) {
        this.value = value;
    }
    
    /**
     * Sets the element's value.
     * 
     * @param value
     *            The value to set.
     */
    public void setValue(String value)
    {
    	this.value = value==null?null:new StringBuffer(value);
    }

    /**
     * Gets the element's local name.
     * 
     * @return Returns the localName.
     */
    public String getLocalName() {
        return localName;
    }

    /**
     * Sets the element's local name.
     * 
     * @param localName
     *            The localName to set.
     */
    public void setLocalName(String localName) {
        this.localName = (localName==null?"":localName);
    }

    /**
     * Sets the element's local name and qualified name to the same value.
     * 
     * @param localQName
     *            The localName and qName to set.
     */
    public void setLocalQName(String localQName) {
    	setLocalName(localQName);
    	setQName(localQName);
    }

    /**
     * Gets the element's qualified name.
     * 
     * @return Returns the qName.
     */
    public String getQName() {
        return qName;
    }

    /**
     * Sets the element's qualified name.
     * 
     * @param name
     *            The qName to set.
     */
    public void setQName(String name) {
        qName = ( name == null ? "" : name);
    }

    /**
     * Gets the element's URI.
     * 
     * @return Returns the uri.
     */
    public URI getURI() {
        return uri;
    }

    /**
     * Sets the element's URI.
     * 
     * @param uri
     *            The uri to set.
     */
    public void setURI(URI uri) {
        this.uri = uri;
    }

    /**
     * Sets the element's URI.
     * If the given uri is null, then an empty string is used to init the URI.
     * @param uri
     *            The uri to set.
     * @throws SAXException
     *             If the given string violates RFC 2396, see
     *             {@link URI#URI(String) URI(String)}.
     */
    public void setURI(String uri) throws SAXException {
        try
        {
        if (uri == null)
            this.uri = new URI("");
        else
            this.uri = new URI(uri);}
        catch(URISyntaxException e)
        {
        	throw new SAXException(e);
        }
    }

    /**
     * Gets the value of an specific attribute.
     * 
     * @param qName
     *            Qualified name of the attribute
     * @return Value of the attribute, or null if no matching attribute.
     */
    public String getAttributeValue(String qName) {
        return attributes.getValue(qName);
    }

    /**
     * Gets the integer value of an specific attribute.
     * 
     * @param qName
     *            Qualified name of the attribute
     * @return Integer value of the attribute, or null if no matching attribute.
     */
    public Integer getAttributeValueAsInteger(String qName) {
        String ret = attributes.getValue(qName);
        return ret==null ? null : Integer.valueOf(ret);
    }
    
    /**
     * Gets the boolean value of an specific attribute.
     * 
     * @param qName
     *            Qualified name of the attribute
     * @return Boolean value of the attribute, or null if no matching attribute.
     */
    public Boolean getAttributeValueAsBoolean(String qName) {
        String ret = attributes.getValue(qName);
        return ret==null ? null : Boolean.valueOf(ret);
    }

    /**
     * Gets the type of an specific attribute.
     * 
     * @param qName
     *            Qualified name of the attribute
     * @return Type of the attribute, or null if no matching attribute.
     */
    public String getAttributeType(String qName) {
        return attributes.getType(qName);
    }

    /**
     * Gets the value of the element as String.
     * 
     * @return Value of the element, or null if no value available.
     */
    public String getValueAsString() {
        return value==null ? null : value.toString();
    }
    /**
     * Gets the value of the element as boolean.
     * 
     * @return Boolean value of the element, or null if no value available.
     */
    public Boolean getValueAsBoolean() {
        return value==null ? null : Boolean.valueOf(value.toString());
    }
    /**
     * Gets the value of the element as Integer.
     * 
     * @return Integer value of the element, or null if no value available.
     */
    public Integer getValueAsInteger() {
        return value==null ? null : Integer.valueOf(value.toString());
    }
}
