/*
 * $Id: XMLDebugger.java,v 1.4 2005/07/28 21:23:38 suhrbier Exp $
 * Created on 06.07.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: XMLDebugger.java,v $
 * Revision 1.4  2005/07/28 21:23:38  suhrbier
 * Dump characters added
 *
 * Revision 1.3  2005/07/21 19:00:33  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/07/15 19:55:30  suhrbier
 * Javadoc added/updated
 *
 * Revision 1.1  2005/07/06 12:14:10  suhrbier
 * beginn RBACFilter
 *
 */
package nbi.xml.sax;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.apache.commons.io.HexDump;
import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;

/**
 * This class prints all SAX-Events with parameters to an given output stream.
 * All events are delivered to the related ContentHandler.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 */

public class XMLDebugger extends XMLFilterImpl {
	static Logger logger = Logger.getLogger(XMLDebugger.class);

	/** Writer used to write SAX events on a stream */
	private PrintWriter writer;

	/**
	 * Initialises XMLDebugger instance.
	 * 
	 * @param out
	 *            Outputstream, where the events are written.
	 */
	public XMLDebugger(OutputStream out) {
		super();
		this.writer = new PrintWriter(out, true);
	}

	/**
	 * Initialises XMLDebugger instance with a given reader.
	 * 
	 * @param out
	 *            Outputstream, where the events are written.
	 * @param reader
	 *            SAX event source.
	 */
	public XMLDebugger(XMLReader reader, OutputStream out) {
		super(reader);
		this.writer = new PrintWriter(out, true);
	}

	/**
	 * @see org.xml.sax.ContentHandler#startDocument()
	 */
	public void startDocument() throws SAXException {
		writer.println("startDocument:");
		super.startDocument();
	}

	/**
	 * @see org.xml.sax.ContentHandler#endDocument()
	 */
	public void endDocument() throws SAXException {
		writer.println("endDocument:");
		super.endDocument();
	}

	/**
	 * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String,
	 *      java.lang.String)
	 */
	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
		writer.println("startPrefixMapping:" + " prefix=" + prefix + " uri="
				+ uri);
		super.startPrefixMapping(prefix, uri);
	}

	/**
	 * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
	 */
	public void endPrefixMapping(String prefix) throws SAXException {
		writer.println("endPrefixMapping:" + " prefix=" + prefix);
		super.endPrefixMapping(prefix);
	}

	/**
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
	 *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localName, String qName,
			Attributes atts) throws SAXException {
		writer.println("startElement:" + " uri=" + uri + " localName="
				+ localName + " qName=" + qName);
		if (atts.getLength() > 0) {
			for (int i = 0; i < atts.getLength(); i++) {
				writer.println("Attribute[" + i + "]: " + "uri="
						+ atts.getURI(i) + "localName=" + atts.getLocalName(i)
						+ " qName=" + atts.getQName(i));
				writer.println("Attribute[" + i + "] type=" + atts.getType(i)
						+ " value=" + atts.getValue(i));
			}
		}
		super.startElement(uri, localName, qName, atts);
	}

	/**
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		writer.println("endElement:" + " uri=" + uri + " localName="
				+ localName + " qName=" + qName);
		super.endElement(uri, localName, qName);
	}

	/**
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		writer.println("characters[" + length + "]:");
		if (length > 0) {
			ByteArrayOutputStream dump = new ByteArrayOutputStream();
			try {
				HexDump.dump(new String(ch, start, length).getBytes(), 0, dump,
						0);
			} catch (IOException e) {
				writer.write(e.getMessage());
			}
			writer.write(dump.toString());
		}
		super.characters(ch, start, length);
	}

	/**
	 * @see org.xml.sax.ErrorHandler#error(org.xml.sax.SAXParseException)
	 */
	public void error(SAXParseException e) throws SAXException {
		writer.println("error: " + e.getMessage());
		super.error(e);
	}

	/**
	 * @see org.xml.sax.ErrorHandler#fatalError(org.xml.sax.SAXParseException)
	 */
	public void fatalError(SAXParseException e) throws SAXException {
		writer.println("fatalError: " + e.getMessage());
		super.fatalError(e);
	}

	/**
	 * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int)
	 */
	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
		writer.print("ignorableWhitespace[" + length + "]:");
		writer.write(ch, start, length);
		writer.print("\n");
		super.ignorableWhitespace(ch, start, length);
	}

	/**
	 * @see org.xml.sax.DTDHandler#notationDecl(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public void notationDecl(String name, String publicId, String systemId)
			throws SAXException {
		writer.println("notationDecl: name=" + name + " publicId=" + publicId
				+ " systemId=" + systemId);
		super.notationDecl(name, publicId, systemId);
	}

	/**
	 * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String,
	 *      java.lang.String)
	 */
	public void processingInstruction(String target, String data)
			throws SAXException {
		writer.println("processingInstruction: target=" + target + " data="
				+ data);
		super.processingInstruction(target, data);
	}

	/**
	 * @see org.xml.sax.EntityResolver#resolveEntity(java.lang.String,
	 *      java.lang.String)
	 */
	public InputSource resolveEntity(String publicId, String systemId)
			throws SAXException, IOException {
		writer.println("resolveEntity: publicId=" + publicId + " systemId="
				+ systemId);
		return super.resolveEntity(publicId, systemId);
	}

	/**
	 * @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String)
	 */
	public void skippedEntity(String name) throws SAXException {
		writer.println("skippedEntity: name=" + name);
		super.skippedEntity(name);
	}

	/**
	 * @see org.xml.sax.DTDHandler#unparsedEntityDecl(java.lang.String,
	 *      java.lang.String, java.lang.String, java.lang.String)
	 */
	public void unparsedEntityDecl(String name, String publicId,
			String systemId, String notationName) throws SAXException {
		writer.println(" unparsedEntityDecl: name=" + name + " publicId="
				+ publicId + " systemId=" + systemId + "  notationName="
				+ notationName);
		super.unparsedEntityDecl(name, publicId, systemId, notationName);
	}

	/**
	 * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
	 */
	public void warning(SAXParseException e) throws SAXException {
		writer.println("warning: " + e.getMessage());
		super.warning(e);
	}

	/**
	 * @see org.xml.sax.XMLReader#getContentHandler()
	 */
	public ContentHandler getContentHandler() {
		writer.println("getContentHandler:");
		return super.getContentHandler();
	}

	/**
	 * @see org.xml.sax.XMLReader#getDTDHandler()
	 */
	public DTDHandler getDTDHandler() {
		writer.println("getDTDHandler:");
		return super.getDTDHandler();
	}

	/**
	 * @see org.xml.sax.XMLReader#getEntityResolver()
	 */
	public EntityResolver getEntityResolver() {
		writer.println("getEntityResolver:");
		return super.getEntityResolver();
	}

	/**
	 * @see org.xml.sax.XMLReader#getErrorHandler()
	 */
	public ErrorHandler getErrorHandler() {
		writer.println("getErrorHandler:");
		return super.getErrorHandler();
	}

	/**
	 * @see org.xml.sax.XMLReader#getFeature(java.lang.String)
	 */
	public boolean getFeature(String name) throws SAXNotRecognizedException,
			SAXNotSupportedException {
		writer.println("getFeature: name=" + name);
		return super.getFeature(name);
	}

	/**
	 * @see org.xml.sax.XMLFilter#getParent()
	 */
	public XMLReader getParent() {
		writer.println("getParent:");
		return super.getParent();
	}

	/**
	 * @see org.xml.sax.XMLReader#getProperty(java.lang.String)
	 */
	public Object getProperty(String name) throws SAXNotRecognizedException,
			SAXNotSupportedException {
		writer.println("getProperty: name=" + name);
		return super.getProperty(name);
	}

	/**
	 * @see org.xml.sax.XMLReader#parse(org.xml.sax.InputSource)
	 */
	public void parse(InputSource input) throws SAXException, IOException {
		writer.println("parse: input=" + input);
		super.parse(input);
	}

	/**
	 * @see org.xml.sax.XMLReader#parse(java.lang.String)
	 */
	public void parse(String systemId) throws SAXException, IOException {
		writer.println("parse: systemId=" + systemId);
		super.parse(systemId);
	}

	/**
	 * @see org.xml.sax.XMLReader#setContentHandler(org.xml.sax.ContentHandler)
	 */
	public void setContentHandler(ContentHandler handler) {
		writer.println("setContentHandler: handler=" + handler);
		super.setContentHandler(handler);
	}

	/**
	 * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
	 */
	public void setDocumentLocator(Locator locator) {
		writer.println("setDocumentLocator:");
		writer.println("Locator: line=" + locator.getLineNumber() + " column"
				+ locator.getColumnNumber() + " publicId="
				+ locator.getPublicId() + " systemId=" + locator.getSystemId());
		super.setDocumentLocator(locator);
	}

	/**
	 * @see org.xml.sax.XMLReader#setDTDHandler(org.xml.sax.DTDHandler)
	 */
	public void setDTDHandler(DTDHandler handler) {
		writer.println("setDTDHandler: handler=" + handler);
		super.setDTDHandler(handler);
	}

	/**
	 * @see org.xml.sax.XMLReader#setEntityResolver(org.xml.sax.EntityResolver)
	 */
	public void setEntityResolver(EntityResolver resolver) {
		writer.println("setEntityResolver: resolver=" + resolver);
		super.setEntityResolver(resolver);
	}

	/**
	 * @see org.xml.sax.XMLReader#setErrorHandler(org.xml.sax.ErrorHandler)
	 */
	public void setErrorHandler(ErrorHandler handler) {
		writer.println("setErrorHandler: handler=" + handler);
		super.setErrorHandler(handler);
	}

	/**
	 * @see org.xml.sax.XMLReader#setFeature(java.lang.String, boolean)
	 */
	public void setFeature(String name, boolean state)
			throws SAXNotRecognizedException, SAXNotSupportedException {
		writer.println("setFeature: name=" + name + " state=" + state);
		super.setFeature(name, state);
	}

	/**
	 * @see org.xml.sax.XMLFilter#setParent(org.xml.sax.XMLReader)
	 */
	public void setParent(XMLReader parent) {
		if (writer != null)
			writer.println("setParent:");
		super.setParent(parent);
	}

	/**
	 * @see org.xml.sax.XMLReader#setProperty(java.lang.String,
	 *      java.lang.Object)
	 */
	public void setProperty(String name, Object value)
			throws SAXNotRecognizedException, SAXNotSupportedException {
		writer.println("setProperty: name=" + name + " value=" + value);
		super.setProperty(name, value);
	}
}
