/*
 * $Id: XMLWriter.java,v 1.13 2005/12/19 19:11:06 suhrbier Exp $
 * Created on 04.03.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: XMLWriter.java,v $
 * Revision 1.13  2005/12/19 19:11:06  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.12  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.11  2005/08/31 11:23:03  suhrbier
 * Fehler mit verschachtelten LOP-Filtern behoben.
 * HttpRequestMethod um URIParameter erweitert.
 *
 * Revision 1.10  2005/08/29 12:35:16  suhrbier
 * Neuer Provider fast lauffhig, XMLWriter schreibt jetzt ESC, Protocolfilter sendet Character Events nicht mehr aufgestaut weiter.
 *
 * Revision 1.9  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.8  2005/07/28 20:50:36  suhrbier
 * Namespace Support added
 *
 * Revision 1.7  2005/07/27 15:02:07  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.6  2005/07/23 10:58:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/07/21 19:00:33  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/07/15 19:55:30  suhrbier
 * Javadoc added/updated
 *
 * Revision 1.3  2005/07/15 18:09:18  suhrbier
 * JavaDoc added
 *
 * Revision 1.2  2005/07/06 12:14:10  suhrbier
 * beginn RBACFilter
 *
 * Revision 1.1  2005/03/07 18:22:35  suhrbier
 * Initiale Version
 *
 */
package nbi.xml.sax;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;

/**
 * This class transforms Sax-events to an XML-document.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.13 $
 * @since
 */

public class XMLWriter extends XMLFilterImpl {
    /** Writer used to write SAX events on a stream */
    private Writer out;

    /** Buffer to capture values from character events */
    StringBuffer characters;

    /** Buffer to capture prefix values startPrefixMapping events */
    StringBuffer prefixes;

    /**
     * Initialises the XMLWriter. Encodes the output stream with UTF-8.
     * 
     * @param out
     *            Outputstream, where the document is written
     */
    public XMLWriter(OutputStream out) {
        super();
        try {
            this.out = new OutputStreamWriter(out, "UTF8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    /**
     * Initialises XMLWriter instance with a given reader.
     * 
     * @param reader
     *            SAX event source.
     * @param out
     *            Outputstream, where the events are written.
     */
    public XMLWriter(XMLReader reader, OutputStream out) {
        super(reader);
        try {
            this.out = new OutputStreamWriter(out, "UTF8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    /**
     * Writes the XML header line to the stream.
     * 
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException {
//        writeEsc("<?xml version='1.0' encoding='UTF-8'?>",true);
      write("<?xml version='1.0' encoding='UTF-8'?>");
        writeNewLine();
        super.startDocument();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String,
     *      java.lang.String)
     */
    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
        if (prefixes == null)
            prefixes = new StringBuffer();
        if (prefix.equals(""))
            prefixes.append(" xmlns='" + uri + "'");
        else
            prefixes.append(" xmlns:" + prefix + "='" + uri + "'");
        super.startPrefixMapping(prefix, uri);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
     */
    public void endPrefixMapping(String prefix) throws SAXException {
        super.endPrefixMapping(prefix);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String,
     *      java.lang.String)
     */
    public void processingInstruction(String target, String data)
            throws SAXException {
        super.processingInstruction(target, data);
    }

    /**
     * Finishes the document with a line separator.
     * 
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void endDocument() throws SAXException {
        super.endDocument();
        try {
            writeNewLine();
            out.flush();
        } catch (IOException e) {
            throw new SAXException("I/O error", e);
        }
    }

    /**
     * Writes the element with its attributes to the stream. Possibly
     * outstanding value characters are written first.
     * 
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
     *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String namespaceURI, String sName, // simple name
            String qName, // qualified name
            Attributes attrs) throws SAXException {
        // write outstanding value characters
        writeCharacters();

        // Evaluate element name
        String eName = sName; // element name
        if ("".equals(eName)) {
            eName = qName; // not namespaceAware
        }

        // Write element and attributes
//        writeEsc("<" + qName,false);
        write("<" + qName);
        if (prefixes != null) {
//            writeEsc(prefixes.toString());
            write(prefixes.toString());
            prefixes = null;
        }

        if (attrs != null) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String aName = attrs.getLocalName(i); // Attr name

                if ("".equals(aName)) {
                    aName = attrs.getQName(i);
                }

//                writeEsc(" ",true);
//                writeEsc(attrs.getQName(i) + "=\"" + attrs.getValue(i) + "\"", true);
                write(" ");
                write(attrs.getQName(i) + "=\"" + attrs.getValue(i) + "\"");
            }
        }
        // close the element name
        write(">");
//        writeEsc(">",false);
        super.startElement(namespaceURI, sName, qName, attrs);
    }

    /**
     * Writes the value characters of the element and closes the element.
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public void endElement(String namespaceURI, String sName, String qName)
            throws SAXException {
        // write value characters
        writeCharacters();

        // Evaluate element name
        String eName = sName; // element name
        if ("".equals(eName)) {
            eName = qName; // not namespaceAware
        }

        // close the element
        write("</" + eName + ">");
//        writeEsc("</" + eName + ">",false);
        super.endElement(namespaceURI, sName, qName);
    }

    /**
     * Adds this character event to the current element value.
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] buf, int offset, int len) throws SAXException {
        // create String
        String s = new String(buf, offset, len);

        // Create/Append string to buffer
        if (characters == null) {
            characters = new StringBuffer(s);
        } else {
            characters.append(s);
        }
        super.characters(buf, offset, len);
    }

    /**
     * Writes the buffered character values to the stream
     * 
     * @throws SAXException
     *             When an I/O-Error occurs.
     */
    private void writeCharacters() throws SAXException {
        if (characters == null) {
            return;
        }

        String s = "" + characters;
        writeEsc(s,true);
        characters = null;
    }

    private void writeEsc (String s, boolean isAttVal)
	throws SAXException
    {
        char[] ch = new char[s.length()];
        s.getChars(0,s.length(),ch,0);
	for (int i = 0; i < s.length(); i++) {
	    switch (ch[i]) {
	    case '&':
		write("&amp;");
		break;
	    case '<':
		write("&lt;");
		break;
	    case '>':
		write("&gt;");
		break;
	    case '\"':
		if (isAttVal) {
		    write("&quot;");
		} else {
		    write('\"');
		}
		break;
	    default:
		if (ch[i] > '\u007f') {
		    write("&#");
		    write(Integer.toString(ch[i]));
		    write(';');
		} else {
		    write(ch[i]);
		}
	    }
	}
    }

    /**
     * Writes a String to the output stream.
     * 
     * @param s
     *            String to write to the stream.
     * @throws SAXException
     *             When an I/O-Error occurs.
     */
    private void write(String s) throws SAXException {
        try {
            out.write(s);
            out.flush();
        } catch (IOException e) {
            throw new SAXException("I/O error", e);
        }
    }

    /**
     * Writes a Character to the output stream.
     * 
     * @param c
     *            character to write to the stream.
     * @throws SAXException
     *             When an I/O-Error occurs.
     */
    private void write(char c) throws SAXException {
        try {
            out.write(c);
            out.flush();
        } catch (IOException e) {
            throw new SAXException("I/O error", e);
        }
    }

    /**
     * Writes the system's line separator to the stream.
     * 
     * @throws SAXException
     *             When an I/O-Error occurs.
     */
    public void writeNewLine() throws SAXException {
        String lineEnd = System.getProperty("line.separator");

        try {
            out.write(lineEnd);
        } catch (IOException e) {
            throw new SAXException("I/O error", e);
        }
    }
}
