/*
 * $Id: PEPClient.java,v 1.15 2006/08/11 16:14:04 suhrbier Exp $
 * Created on 30.05.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PEPClient.java,v $
 * Revision 1.15  2006/08/11 16:14:04  suhrbier
 * *** empty log message ***
 *
 * Revision 1.14  2006/07/25 14:34:48  suhrbier
 * signed Elements with namespace-URI working.
 *
 * Revision 1.13  2006/07/24 08:21:10  suhrbier
 * + class SecurityDiagnostic
 *
 * Revision 1.12  2006/07/21 18:38:13  suhrbier
 * +XPATH2-References(but not really working)
 * +Signing of requested elements
 * +SECURITY-CODE
 *
 * Revision 1.11  2006/07/19 10:34:14  suhrbier
 * + Keyinfo support certificates (not completed)
 *
 * Revision 1.10  2006/07/18 14:58:11  suhrbier
 * Fehler behoben in ContentObjectFilter
 * + PEPClient send Request
 * Fehler in der Request.write()-Kette behoben.
 *
 * Revision 1.9  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.8  2006/07/14 21:17:58  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.7  2006/07/14 13:12:06  suhrbier
 * FilterMode integrieren
 *
 * Revision 1.6  2006/07/12 09:42:41  suhrbier
 * Fehler behoben in PEPHandler bzgl. getRoles/getCertificates wenn kein clientCert-Parameter vorliegt.
 *
 * Revision 1.5  2006/06/02 11:02:05  suhrbier
 * JavaDoc
 *
 * Revision 1.4  2006/06/01 18:15:34  suhrbier
 * 1st working API with TrustedClient.
 * XMLSigning only whole documents.
 *
 * Revision 1.3  2006/05/31 14:55:20  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2006/05/31 10:29:28  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2006/05/30 19:05:35  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.security.Provider;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Iterator;

import javax.net.ssl.HttpsURLConnection;
import javax.xml.crypto.MarshalException;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureException;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import nbi.xml.sax.XMLWriter;
import nbi.xmlsec.biocase.protocol.Diagnostic;
import nbi.xmlsec.biocase.protocol.Diagnostics;
import nbi.xmlsec.biocase.protocol.Request;
import nbi.xmlsec.biocase.protocol.Response;
import nbi.xmlsec.biocase.protocol.SecurityDiagnostic;
import nbi.xmlsec.biocase.protocol.filter.ProtocolFilter;
import nbi.xmlsec.biocase.protocol.filter.ResponseFilter;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.log4j.Logger;
import org.bouncycastle.util.encoders.UrlBase64;
import org.mortbay.util.IO;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import com.sun.org.apache.xml.security.utils.HexDump;

/**
 * The PEPClient manages communication with a BioCASE provider's
 * {@link nbi.xmlsec.PEP Policy Enforcement Point(PEP)}.
 * <p>
 * The PEP protects the provider combining SSL based client authentication and
 * role based access control on the provider resources. The PEPClient API may be
 * used from clients as well as web interface providers. The former
 * authenticates the client to the PEP directly using his private key. The PEP
 * evaluates the roles assigned to this client and handles the related request
 * according to the access control policies defined for these roles. The latter
 * must be recognised by the provider PEP as a trusted communication partner.
 * For this purpose, the PEP implements a fixed role named <b>trustedClient
 * </b>. Clients having assigned this role by the provider's access control
 * policies, may pass the identity of any client authenticated by themselves to
 * the PEP. Therefore, the PEPClient sends the X.509 certificate of such
 * authenticated clients to the PEP. This is done through the addition of an
 * additional parameter <b>clientCert </b> to the Http request. Following the
 * initial authentication of the <b>trusted client </b>, the PEP uses the X.509
 * certificate of the client certificates to determine the access rights of the
 * included BioCASE request. It is on the <b>trusted client </b> to present or
 * deliver the response to the <b>real client </b> respectively.
 * </p>
 * <p>
 * The PEPClient needs the following parameters to provide the functionality
 * described above:
 * <ul>
 * <li>URL of the provider's PEP (e.g.https://localhost/biocase/pywrapper.cgi)
 * </li>
 * <li>Database name (i.e. the dsa parameter of the Http request to the
 * PyWrapper)</li>
 * <li>Debug flag (i.e. the debug parameter of the Http request to the
 * PyWrapper)</li>
 * <li>KeyStore</li>
 * <li>TrustStore</li>
 * </ul>
 * <p>
 * The Keystore specifies a Java KeyStore file holding the private key and
 * certificate of the client sending requests to the PEP (e.g. PKCS12-file
 * (.p12) delivered from the CA). The Truststore also specifies a Java KeyStore
 * file including the certificates from all entities trusted by the client,
 * especially the certificates of the provider PEPs to be contacted.
 * </p>
 * <p>
 * The basic usage of this class consists of three steps:
 * <ul>
 * <li>Initialise the PEPClient</li>
 * <li>Send the BioCase request using one of the <code>send</code> methods
 * </li>
 * <li>Retrieve the BioCase response using one of the <code>getResponse</code>
 * methods</li>
 * </ul>
 * </p>
 * <p>
 * The initialisation of the PEPClient may also be read from a XML-Configuration
 * file, according to the Apache Jakarta Commons Configuration API.
 * </p>
 * <p>
 * If the BioCASE response returned from the PEP includes any XML-Signature, the
 * PEPClient validates these signature and adds the validation results to the
 * diagnostic part of the BioCASE response. Finally, the PEPClient removes the
 * corresponding XML Signature elements and delivers the core BioCASE response
 * only. Note: Currently, this feature is not implemented.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.15 $
 * @since
 */

public class PEPClient {

    static Logger logger = Logger.getLogger(PEPClient.class);

    /** The URL of the PEP protected PyWrapper */
    private URL pepURL;

    /** The connection to the PEP */
    private HttpURLConnection pepConnection;

    private Diagnostics validationDiagnostics;

    /**
     * Initialises the PEP client using a configuration.
     * 
     * @param config
     *            Configuration of the PEPClient class.
     * @throws MalformedURLException
     */
    public PEPClient(Configuration config) throws MalformedURLException {
        this(config.getString("PEPClient.PEP[@url]"), config
                .getString("PEPClient.KeyStore[@file]"), config
                .getString("PEPClient.KeyStore[@type]"), config
                .getString("PEPClient.KeyStore[@password]"), config
                .getString("PEPClient.TrustStore[@file]"), config
                .getString("PEPClient.TrustStore[@type]"), config
                .getString("PEPClient.TrustStore[@password]"));
    }

    /**
     * Initialises the PEPClient.
     * 
     * @param pepURL
     *            URL of the PyWrapper.
     * @param keystore
     *            Path to a JKS-Keystore including the client's private Key.
     * @param keystoreType
     *            Type of the keystore (e.g. PKCS12)
     * @param keystorePass
     *            Password of the keystore
     * @param truststore
     *            Path to a JKS-Keystore including certificates of all trusted
     *            (provider) entities.
     * @param truststoreType
     *            Type of the truststore (e.g. PKCS12)
     * @param truststorePass
     *            Password of the truststore.
     * @throws MalformedURLException
     */
    public PEPClient(String pepURL, String keystore, String keystoreType,
            String keystorePass, String truststore, String truststoreType,
            String truststorePass) throws MalformedURLException {
        setPepURL(pepURL);
        setKeyStoreProperties(keystore, keystoreType, keystorePass, false);
        setKeyStoreProperties(truststore, truststoreType, truststorePass, true);
    }

    /**
     * Store the Keystore parameters into the corresponding system properties.
     * The system properties are the only way to configure a
     * {@link HttpsURLConnection HttpsURLConnection}. The system properties for
     * keystores are:
     * <ul>
     * <li>javax.net.ssl.keyStore</li>
     * <li>javax.net.ssl.keyStoreType</li>
     * <li>javax.net.ssl.keyStorePassword</li>
     * </ul>
     * The system properties for truststores are:
     * 
     * <ul>
     * <li>javax.net.ssl.trustStore</li>
     * <li>javax.net.ssl.trustStoreType</li>
     * <li>javax.net.ssl.trustStorePassword</li>
     * </ul>
     * 
     * @param store
     *            Path to a JKS-Keystore.
     * @param type
     *            Type of the keystore.
     * @param password
     *            Password of the keystore.
     * @param truststore
     */
    private void setKeyStoreProperties(String store, String type,
            String password, boolean truststore) {
        if (!truststore) {
            if (store != null)
                System.setProperty("javax.net.ssl.keyStore", store);
            if (type != null)
                System.setProperty("javax.net.ssl.keyStoreType", type);
            if (password != null)
                System.setProperty("javax.net.ssl.keyStorePassword", password);
        } else {
            if (store != null)
                System.setProperty("javax.net.ssl.trustStore", store);
            if (type != null)
                System.setProperty("javax.net.ssl.trustStoreType", type);
            if (password != null)
                System
                        .setProperty("javax.net.ssl.trustStorePassword",
                                password);
        }
    }

    /**
     * Opens and preconfigures the Https-connection the PEP.
     * 
     * @throws IOException
     */
    private void openHostConnection() throws IOException {
        //        if (pepConnection == null) {
        pepConnection = (HttpURLConnection) pepURL.openConnection();
        pepConnection.setInstanceFollowRedirects(false);
        ((HttpsURLConnection) pepConnection)
                .setHostnameVerifier(new PermitAllHostNameVerifier());
        pepConnection.setDoOutput(true);
        pepConnection.setDoInput(true);
        //        }
        logDefaultStores();
    }

    /**
     * Logs the current System properties of keystore and truststore.
     * 
     *  
     */
    static public void logDefaultStores() {
        logger.debug("javax.net.ssl.trustStore="
                + System.getProperty("javax.net.ssl.trustStore"));
        logger.debug("javax.net.ssl.trustStoreType="
                + System.getProperty("javax.net.ssl.trustStoreType"));
        logger.debug("javax.net.ssl.trustStorePassword="
                + System.getProperty("javax.net.ssl.trustStorePassword"));
        logger.debug("javax.net.ssl.keyStore="
                + System.getProperty("javax.net.ssl.keyStore"));
        logger.debug("javax.net.ssl.keyStoreType="
                + System.getProperty("javax.net.ssl.keyStoreType"));
        logger.debug("javax.net.ssl.keyStorePassword="
                + System.getProperty("javax.net.ssl.keyStorePassword"));
    }

    /**
     * Logs the local and server certificates of the current connection.
     * 
     * @param conn
     *            The cuurent connection.
     */
    static protected void logCertificates(URLConnection conn) {
        if (conn instanceof HttpsURLConnection) {
            try {
                logger.debug("getLocalCertificates()=");
                for (int i = 0; i < ((HttpsURLConnection) conn)
                        .getLocalCertificates().length; i++) {
                    logger.debug(((X509Certificate) ((HttpsURLConnection) conn)
                            .getLocalCertificates()[i]).getSubjectDN()
                            .toString());
                }
                logger.debug("getServerCertificates()=");
                for (int i = 0; i < ((HttpsURLConnection) conn)
                        .getLocalCertificates().length; i++) {
                    logger.debug(((X509Certificate) ((HttpsURLConnection) conn)
                            .getServerCertificates()[i]).getSubjectDN()
                            .toString());
                }
            } catch (Exception e) {
                logger.debug("logCertificates() " + e.getMessage());
            }
        }
    }

    /**
     * Sends a BioCASE request to the provider PEP.
     * 
     * @param request
     *            The BioCASE request.
     * @param dataBase
     *            The database to be queried (dsa request parameter)
     * @param debug
     *            The debug flag (debug request parameter)
     * @param clientCert
     *            The client certificate (trusted client mode) or null.
     * @throws SAXException
     * @throws CertificateEncodingException
     * @throws IOException
     */
    public void send(Request request, String dataBase, boolean debug,
            X509Certificate clientCert) throws SAXException,
            CertificateEncodingException, IOException {
        ByteArrayOutputStream requestOut = new ByteArrayOutputStream();
        XMLWriter requestWriter = new XMLWriter(requestOut);
        request.write(requestWriter);
        send(new ByteArrayInputStream(requestOut.toByteArray()), dataBase,
                debug, clientCert);
    }

    /**
     * Sends a BioCASE request to the provider PEP.
     * 
     * @param requestIn
     *            The BioCASE request stream.
     * @param dataBase
     *            The database to be queried (dsa request parameter)
     * @param debug
     *            The debug flag (debug request parameter)
     * @param clientCert
     *            The client certificate (trusted client mode) or null.
     * @throws CertificateEncodingException
     * @throws IOException
     */
    public void send(InputStream requestIn, String dataBase, boolean debug,
            X509Certificate clientCert) throws CertificateEncodingException,
            IOException {
        openHostConnection();
        //        pepConnection.setRequestMethod("POST");
        //        pepConnection.setRequestProperty("Content-Type", "application/xml");
        pepConnection.getOutputStream();
        logCertificates(pepConnection);
        if (debug)
            pepConnection.getOutputStream().write("debug=1".getBytes());
        pepConnection.getOutputStream().write("&dsa=".getBytes());
        pepConnection.getOutputStream().write(
                URLEncoder.encode(dataBase, "UTF8").getBytes());
        logger.debug("clientCert=" + clientCert.getSubjectDN().toString());
        if (clientCert != null) {
            pepConnection.getOutputStream().write("&clientCert=".getBytes());
            UrlBase64.encode(clientCert.getEncoded(), pepConnection
                    .getOutputStream());
        }
        //TODO Read Client Cert from Config Keystore
        pepConnection.getOutputStream().write("&query=".getBytes());
        pepConnection.getOutputStream().write(
                URLEncoder.encode(IO.toString(requestIn), "UTF8").getBytes());
    }

    /**
     * Validates any signature of the incoming {@link InputStream InputStream}.
     * The validation result is reported in the {@link #validationDiagnostics validationDiagnostics}, which will be added later within {@link #getResponse() getResponse()} to the final response.
     * The key validation is done using the {@link PEPClientKeySelector PEPClientKeySelector}.
     * Unfortunately, the validation process needs a transformation of the XML-Document into the DOM-model.
     * 
     * @param in InputStream with the XML-document.
     * @return InputStream with the original XML-Document without signature element. Any validation error throws an exception.
     * @throws SAXException
     * @throws IOException
     * @throws ParserConfigurationException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws ClassNotFoundException
     * @throws MarshalException
     * @throws XMLSignatureException
     * @throws TransformerException
     */
    protected InputStream validateSignature(InputStream in)
            throws SAXException, IOException, ParserConfigurationException,
            InstantiationException, IllegalAccessException,
            ClassNotFoundException, MarshalException, XMLSignatureException,
            TransformerException {
        validationDiagnostics = new Diagnostics(null);
        // Instantiate the document to be validated
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        Document doc = dbf.newDocumentBuilder().parse(in);
        // Find Signature element
        NodeList nl = doc.getElementsByTagNameNS(XMLSignature.XMLNS,
                "Signature");
        if (nl.getLength() == 0) {
            validationDiagnostics.addDiagnostic(new SecurityDiagnostic(null,
                    Diagnostic.SEVERITY_INFO, "Document has no signature"));
        } else {
            // Create a DOM XMLSignatureFactory that will be used to unmarshal
            // the
            // document containing the XMLSignature
            String providerName = System.getProperty("jsr105Provider",
                    "org.jcp.xml.dsig.internal.dom.XMLDSigRI");
            XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM",
                    (Provider) Class.forName(providerName).newInstance());

            // Create a DOMValidateContext and specify a KeyValue KeySelector
            // and document context
            DOMValidateContext valContext = new DOMValidateContext(
                    new PEPClientKeySelector(), nl.item(0));
            valContext.setProperty("javax.xml.crypto.dsig.cacheReference",
                    Boolean.TRUE);

            // unmarshal the XMLSignature
            XMLSignature signature = fac.unmarshalXMLSignature(valContext);

            // Validate the XMLSignature (generated above)
            boolean coreValidity = signature.validate(valContext);

            // Check core validation status
            if (coreValidity == false) {
                validationDiagnostics.addDiagnostic(new SecurityDiagnostic(null,
                        Diagnostic.SEVERITY_WARN,
                        "Signature failed core validation"));
                logger.warn("Signature failed core validation");
                boolean sv = signature.getSignatureValue().validate(valContext);
                validationDiagnostics.addDiagnostic(new SecurityDiagnostic(null, 
                        Diagnostic.SEVERITY_WARN,
                        "Signature validation status: " + sv));
                logger.debug("signature validation status: " + sv);
                // check the validation status of each Reference
                Iterator i = signature.getSignedInfo().getReferences()
                        .iterator();
                for (int j = 0; i.hasNext(); j++) {
                    Reference ref = (Reference) i.next();
                    boolean refValid = ref.validate(valContext);
                    validationDiagnostics.addDiagnostic(new SecurityDiagnostic(null,
                            Diagnostic.SEVERITY_WARN, "ref[" + j
                                    + "] validity status: " + refValid));
                    logger.debug("ref[" + j + "] Type: " + ref.getType()
                            + "URI: " + ref.getURI());
                    logger.debug("ref[" + j + "] validity status: " + refValid);
                }
            } else {
                validationDiagnostics.addDiagnostic(new SecurityDiagnostic(null,
                        Diagnostic.SEVERITY_INFO,
                        "Signature passed core validation"));
                logger.info("Signature passed core validation");
                logger.debug("SigValue="
                        + HexDump.toHexString(signature.getSignatureValue()
                                .getValue()));
                Iterator refIt = signature.getSignedInfo().getReferences()
                        .iterator();
                while (refIt.hasNext()) {
                    Reference ref = (Reference) refIt.next();
                    logger.debug("DereferencedData:"
                            + ref.getDereferencedData());
                    logger.debug("DigestInputStream:"
                            + ref.getDigestInputStream());
                    logger.debug("DigestValue="
                            + HexDump.toHexString(ref.getDigestValue()));
                }

            }
            logger.debug("nl.item[0]: ns=" + nl.item(0).getNamespaceURI()
                    + " ln=" + nl.item(0).getLocalName());
            // Delete Signature element from the response
            nl.item(0).getParentNode().removeChild(nl.item(0));
        }
        ByteArrayOutputStream responseOut = new ByteArrayOutputStream();
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer trans = tf.newTransformer();
        trans.transform(new DOMSource(doc), new StreamResult(responseOut));

        return new ByteArrayInputStream(responseOut.toByteArray());
    }

    /**
     * Retrieves the BioCASE response stream from the PEP. Errors are documented
     * within the diagnostics part of the response. XML-Signature evaluation results are not added
     * to the diagnostics part of the response here!
     * 
     * @return The BioCASE response as InputStream without XML-Signature validation results in the diagnostics.
     * @throws IOException
     * @throws TransformerException
     * @throws XMLSignatureException
     * @throws MarshalException
     * @throws ClassNotFoundException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws ParserConfigurationException
     * @throws SAXException
     */
    protected InputStream getResponseAsInputStream() throws SAXException,
            IOException, ParserConfigurationException, InstantiationException,
            IllegalAccessException, ClassNotFoundException, MarshalException,
            XMLSignatureException, TransformerException {
        if (pepConnection.getResponseCode() == HttpURLConnection.HTTP_OK)
            return validateSignature(pepConnection.getInputStream());
        logger.error("HttpResponseCode not OK.");
        return pepConnection.getErrorStream();
    }

    /**
     * Retrieves the BioCASE response from the PEP. Errors are documented within
     * the diagnostics part of the response. XML-Signature evaluation are added
     * to the diagnostics part of the response.
     * 
     * @return The BioCASE response including signature evalution report.
     * @throws UnsupportedEncodingException
     * @throws IOException
     * @throws SAXException
     * @throws Exception
     */
    public Response getResponse() throws UnsupportedEncodingException,
            IOException, SAXException, Exception {
        Response ret = null;
        SAXParserFactory factory = SAXParserFactory.newInstance();
        SAXParser saxParser = null;
        factory.setNamespaceAware(true);
        saxParser = factory.newSAXParser();

        XMLReader reader = saxParser.getXMLReader();
        ResponseFilter responseFilter = new ResponseFilter(reader,
                ProtocolFilter.PROT_URI_13, ProtocolFilter.FILTERMODE_PARSER);
        responseFilter.setContentHandler(new DefaultHandler());
        reader.setContentHandler(responseFilter);

        logger.debug("start Response PARSING");
        reader.parse(new InputSource(new InputStreamReader(
                getResponseAsInputStream(), "UTF-8")));
        logger.debug("stop Response PARSING");
        ret = responseFilter.getResponse();
        ret.getDiagnostics().addDiagnostics(validationDiagnostics);
        return ret;
    }

    /**
     * Gets the pepURL
     * 
     * @return Returns the pepURL.
     */
    public URL getPepURL() {
        return pepURL;
    }

    /**
     * Sets pepURL
     * 
     * @param pepURL
     *            The pepURL to set.
     * @throws MalformedURLException
     *             {@link URL#URL(String) URL(String)}
     */
    protected void setPepURL(String pepURL) throws MalformedURLException {
        this.pepURL = new URL(pepURL);
    }
}
