/*
 * $Id: PEPHandler.java,v 1.34 2006/08/14 10:48:09 suhrbier Exp $
 * Created on 05.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PEPHandler.java,v $
 * Revision 1.34  2006/08/14 10:48:09  suhrbier
 * added Transformer output indention for signatues
 *
 * Revision 1.33  2006/07/25 14:34:48  suhrbier
 * signed Elements with namespace-URI working.
 *
 * Revision 1.32  2006/07/24 19:05:47  suhrbier
 * Im Umbau von signedElements.
 *
 * Revision 1.31  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.30  2006/07/21 18:38:13  suhrbier
 * +XPATH2-References(but not really working)
 * +Signing of requested elements
 * +SECURITY-CODE
 *
 * Revision 1.29  2006/07/19 10:34:14  suhrbier
 * + Keyinfo support certificates (not completed)
 *
 * Revision 1.28  2006/07/14 21:17:58  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.27  2006/07/14 13:12:06  suhrbier
 * FilterMode integrieren
 *
 * Revision 1.26  2006/07/12 09:42:41  suhrbier
 * Fehler behoben in PEPHandler bzgl. getRoles/getCertificates wenn kein clientCert-Parameter vorliegt.
 *
 * Revision 1.25  2006/06/01 18:15:34  suhrbier
 * 1st working API with TrustedClient.
 * XMLSigning only whole documents.
 *
 * Revision 1.24  2006/05/31 14:55:20  suhrbier
 * *** empty log message ***
 *
 * Revision 1.23  2006/05/30 19:05:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.22  2006/05/15 14:24:49  suhrbier
 * *** empty log message ***
 *
 * Revision 1.21  2006/05/12 09:30:09  suhrbier
 * XML-SigAll working.
 *
 * Revision 1.20  2006/05/11 19:03:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.19  2006/05/08 20:46:21  suhrbier
 * Sign-Schablone eingebaut (nicht funktional).
 *
 * Revision 1.18  2006/05/08 17:21:57  suhrbier
 * Transform Response into DOM Document.
 *
 * Revision 1.17  2005/12/19 19:11:06  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.16  2005/12/14 17:27:09  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.15  2005/12/14 15:34:30  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.14  2005/11/02 20:10:34  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.13  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.12  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 *
 */
package nbi.xmlsec;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLEncoder;
import java.security.InvalidAlgorithmParameterException;
import java.security.KeyStore;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.Transform;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
import javax.xml.crypto.dsig.keyinfo.X509Data;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.dsig.spec.XPathFilter2ParameterSpec;
import javax.xml.crypto.dsig.spec.XPathType;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import nbi.util.ReflectionUtil;
import nbi.xml.sax.XMLDebugger;
import nbi.xml.sax.XMLWriter;
import nbi.xmlsec.biocase.protocol.Diagnostic;
import nbi.xmlsec.biocase.protocol.Diagnostics;
import nbi.xmlsec.biocase.protocol.Header;
import nbi.xmlsec.biocase.protocol.Request;
import nbi.xmlsec.biocase.protocol.Response;
import nbi.xmlsec.biocase.protocol.filter.PDPFilterParameter;
import nbi.xmlsec.biocase.protocol.filter.ProtocolFilter;
import nbi.xmlsec.biocase.protocol.filter.RequestFilter;
import nbi.xmlsec.biocase.protocol.filter.ResponseFilter;
import nbi.xmlsec.biocase.protocol.filter.ResponseFilterParameter;
import nbi.xmlsec.xacml.profile.rbac.GuestRoleIdentifier;
import nbi.xmlsec.xacml.profile.rbac.RBACPDP;
import nbi.xmlsec.xacml.profile.rbac.RoleAssignmentPolicyIdentifier;
import nbi.xmlsec.xacml.profile.rbac.RoleEnablementAuthority;
import nbi.xmlsec.xacml.profile.rbac.TrustedClientRoleIdentifier;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.log4j.Logger;
import org.bouncycastle.util.encoders.UrlBase64;
import org.mortbay.http.HttpException;
import org.mortbay.http.HttpFields;
import org.mortbay.http.HttpRequest;
import org.mortbay.http.HttpResponse;
import org.mortbay.http.handler.AbstractHttpHandler;
import org.mortbay.util.IO;
import org.mortbay.util.URI;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import com.sun.xacml.ParsingException;

/**
 * This class realises the functionality of the PEP handling each incoming Http
 * request received via Jetty. Jetty calls the method
 * {@link #handle(String, String, HttpRequest, HttpResponse) handle(String, String, HttpRequest, HttpResponse)}.
 * This method filters each incoming request and the resulting response from the
 * PyWrapper calling the methods
 * {@link #filterRequest(HttpRequest, Set, HttpResponse) filterRequest(HttpRequest, Set, HttpResponse)}
 * and
 * {@link #filterResponse(HttpURLConnection, HttpResponse, Set) filterResponse(HttpURLConnection, HttpResponse, Set)}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.34 $
 * @since
 */
public class PEPHandler extends AbstractHttpHandler {
    static Logger logger = Logger.getLogger(PEPHandler.class);

    /** Initial status */
    final static int STATUS_REQUEST_NONE = 0;

    /** Status Request permitted */
    final static int STATUS_REQUEST_OK = 1;

    /** Status Request denied */
    final static int STATUS_REQUEST_ERROR = 2;

    /** Status Response processed */
    final static int STATUS_RESPONSE_OK = 4;

    /** Status Response processing error */
    final static int STATUS_RESPONSE_ERROR = 8;

    /**
     * The configuration.
     */
    private Configuration config;

    private KeyStore keystore;

    private PrivateKey privKey;

    private X509Certificate cert;

    /**
     * The RoleEnablementAuthority to evaluate roles for a given subject.
     */
    private RoleEnablementAuthority roleEnableAuth;

    /**
     * The RBACPDP to evaluate the access control policies for a given role set.
     */
    private RBACPDP pdp;

    /**
     * The working domain for this PEP.
     */
    private String domain;

    /**
     * The host to redirect permitted requests to.
     */
    private URI host;

    /**
     * The PyWrapperURI to redirect BioCASE requests to..
     */
    private URI wrapper;

    /**
     * Status of the filter
     */
    int status = STATUS_REQUEST_NONE;

    /**
     * BioCASE request to be filtered
     */
    Request request = null;

    static final String version = "alpha1";

    /**
     */
    public PEPHandler() {
        super();
        logger.debug("PEPHandler()");
    }

    /**
     * Creates a PEPHandler based on its configuration.
     * 
     * @param config
     *            The configuration.
     * @throws ClassNotFoundException
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    public PEPHandler(Configuration config) throws ClassNotFoundException,
            NoSuchMethodException, InvocationTargetException,
            InstantiationException, IllegalAccessException {
        super();
        domain = config.getString("[@domain]");
        roleEnableAuth = (RoleEnablementAuthority) ReflectionUtil.getInstance(
                config.getString("RoleEnablementAuthority[@class]"),
                Configuration.class, config.subset("RoleEnablementAuthority"));
        pdp = (RBACPDP) ReflectionUtil.getInstance(config
                .getString("PDP[@class]"), Configuration.class, config
                .subset("PDP"));
        host = new URI(config.getString("Proxy[@url]"));
        wrapper = new URI(config.getString("Wrapper[@url]"));

        try {
            keystore = KeyStore
                    .getInstance(config.getString("KeyStore[@type]"));
            keystore.load(new FileInputStream(config
                    .getString("KeyStore[@file]")), config.getString(
                    "KeyStore[@password]").toCharArray());

            logger.debug("KeyStore aliases\n");
            Enumeration keystoreEnum = keystore.aliases();
            while (keystoreEnum.hasMoreElements())
                logger.debug((String) keystoreEnum.nextElement() + "\n");
            privKey = (PrivateKey) keystore.getKey(config
                    .getString("Key[@alias]"), config.getString(
                    "Key[@password]").toCharArray());
            cert = (X509Certificate) keystore.getCertificate(config
                    .getString("Key[@alias]"));
        } catch (Exception e) {
            logger.fatal(e.getMessage());
        }
    }

    /**
     * Retrieves the client's certificate from a SSL ServerSocket.
     * 
     * @param socket
     *            The SSL server socket
     * @return The client's certificate or <code>null</code>.
     * @throws SSLPeerUnverifiedException
     */
    static X509Certificate getClientCertificate(SSLSocket socket)
            throws SSLPeerUnverifiedException {
        X509Certificate result = null;
        Certificate[] peerCertificates = null;

        peerCertificates = socket.getSession().getPeerCertificates();

        return (X509Certificate) peerCertificates[0];
    }

    /**
     * Retrieves the client's certificate from a HttpRequest parameter.
     * 
     * @param request
     *            The Http request.
     * @param parameter
     *            The Http request parameter name of the client certificate.
     * @return The client's certificate or <code>null</code>.
     * @throws NullPointerException
     *             if parameter could not be retrieved from the request.
     * @throws CertificateException
     */
    static X509Certificate getClientCertificate(HttpRequest request,
            String parameter) throws CertificateException {
        X509Certificate result = null;
        ByteArrayInputStream certIn = new ByteArrayInputStream(UrlBase64
                .decode(request.getParameter(parameter)));
        CertificateFactory certFact = CertificateFactory.getInstance("X.509");
        result = (X509Certificate) certFact.generateCertificate(certIn);
        logger.debug("Client cert retrieved from parameter " + parameter + ":"
                + result.getSubjectDN().toString());
        return result;
    }

    /**
     * Retrieve the enabled roles from the REA for a given Http-request. First,
     * the client's certificate is retrieved, then the REA is evaluated for the
     * client's roles enabled.
     * 
     * @param request
     *            The received Http request
     * @return Set of enabled roles for the client.
     */
    private Set getRoles(HttpRequest request) {
        Set roleIds = new HashSet();
        X509Certificate clientCert = null;

        try {
            if (request.getHttpConnection().getConnection() instanceof SSLSocket) {
                clientCert = (X509Certificate) getClientCertificate((SSLSocket) request
                        .getHttpConnection().getConnection());
                roleIds = roleEnableAuth.getRoles(domain, clientCert);
            }
            if (roleIds.contains(new TrustedClientRoleIdentifier(domain))) {
                logger.debug("TrustedClientRoleIdentifier found");
                try {
                    clientCert = getClientCertificate(request, "clientCert");
                    roleIds = roleEnableAuth.getRoles(domain, clientCert);
                } catch (NullPointerException e) {
                    logger.debug("Request parameter clientCert is empty.");
                }
            }
        } catch (SSLPeerUnverifiedException e) {
            roleIds.clear();
            roleIds.add(new GuestRoleIdentifier(domain));
            logger.warn("Subject Authentication failed" + "\nReason: "
                    + e.toString() + "\n-> role set to "
                    + GuestRoleIdentifier.ROLE_LABEL + "!!!");
        } catch (CertificateException e) {
            logger.warn("Unable to retrieve trusted client certificate"
                    + "\nReason: " + e.toString() + "\n-> role set to "
                    + GuestRoleIdentifier.ROLE_LABEL + "!!!");
        } catch (Exception e) {
            roleIds.clear();
            roleIds.add(new GuestRoleIdentifier(domain));
            logger.warn("Request not accepted for subject"
                    + clientCert.getSubjectDN().getName() + "\nReason: "
                    + e.toString() + "\n-> role set to "
                    + GuestRoleIdentifier.ROLE_LABEL + "!!!");
        }
        return roleIds;
    }

    /**
     * Opens an Http-connection to the provider's interface host and adds the
     * client roles to the given Http-request and redirects the request to the
     * provider's interface host.
     * 
     * @param request
     *            The received Http-Request
     * @param roleIds
     *            The roles to add to the Http-Request as parameters
     * @return The established URLConnection or null.
     * @throws IOException
     * @throws ParsingException
     * @throws URISyntaxException
     */
    private HttpURLConnection openHostConnection(HttpRequest request,
            Set roleIds) throws IOException, ParsingException,
            URISyntaxException {
        HttpURLConnection httpConnection = null;
        URL urlHost = null;
        URI uriHost = new URI(buildHostURL(request.getRootURL()
                + request.getURI().toString(), host.toString()));
        if (request.getMethod() == HttpRequest.__GET) {
            addRoleQuery(roleIds, uriHost);
            addURLQuery(uriHost);
        }
        logger.debug("uriHost=" + uriHost.toString());
        urlHost = new URL(uriHost.toString());
        httpConnection = (HttpURLConnection) urlHost.openConnection();
        httpConnection.setRequestMethod(request.getMethod());
        httpConnection.setInstanceFollowRedirects(false);

        // copy headers from the HttpRequest to the httpConnection
        boolean hasContent = false;
        Enumeration enm = request.getFieldNames();
        while (enm.hasMoreElements()) {
            String hdr = (String) enm.nextElement();

            if (HttpFields.__ContentType.equals(hdr))
                hasContent = true;

            Enumeration vals = request.getFieldValues(hdr);
            while (vals.hasMoreElements()) {
                String val = (String) vals.nextElement();
                if (val != null) {
                    httpConnection.addRequestProperty(hdr, val);
                }
            }
        }
        try {
            if (hasContent) {
                httpConnection.setDoOutput(true);
                httpConnection.setDoInput(true);
                IO.copy(request.getInputStream(), httpConnection
                        .getOutputStream());
                Set requestParameter = request.getParameterNames();
                if (requestParameter != null) {
                    Iterator requestParameterIt = requestParameter.iterator();
                    boolean firstParam = true;
                    OutputStream bodyOut = httpConnection.getOutputStream();
                    while (requestParameterIt.hasNext()) {
                        if (!firstParam)
                            bodyOut.write('&');
                        else
                            firstParam = false;
                        String paramLabel = (String) requestParameterIt.next();
                        String paramValue = request.getParameter(paramLabel);
                        logger.debug("paramLabel=" + paramLabel);
                        logger.debug("paramValue=" + paramValue);
                        bodyOut.write(paramLabel.getBytes());
                        bodyOut.write('=');
                        if (paramValue != null)
                            bodyOut.write(URLEncoder.encode(paramValue, "UTF8")
                                    .getBytes());
                    }
                } else {
                    httpConnection.setDoInput(true);
                    // do input thang!
                    InputStream in = request.getInputStream();
                    if (hasContent) {
                        httpConnection.setDoOutput(true);
                        IO.copy(in, httpConnection.getOutputStream());
                    }
                }
            }
            // Connect the host
            logger.debug("httpConnection vor connect=" + httpConnection);
            httpConnection.connect();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return httpConnection;
    }

    /**
     * Builds the wrapperURL to be set in the provider's interface. The URL is
     * created from the wrapper URL concatenating the paht of the request URL.
     * 
     * @param urlRequest
     *            The URL of the current Request.
     * @param urlWrapper
     *            The URL of the PyWrapper.
     * @return The new wrapperURL or <code>null</code>.
     */
    private String buildWrapperURL(String urlRequest, String urlWrapper) {
        String result = null;
        logger.debug("buildWrapperURL urlRequest=" + urlRequest
                + " urlWrapper=" + urlWrapper);
        if ((urlRequest != null) && (urlRequest.length() > 0)) {
            URI uriResult = new URI(urlRequest);
            URI uriWrapper = new URI(urlWrapper);
            uriResult.setScheme(uriWrapper.getScheme());
            uriResult.setHost(uriWrapper.getHost());
            uriResult.setPort(uriWrapper.getPort());
            logger.debug("buildWrapperURL uriWrapper.getPath()="
                    + uriWrapper.getPath());
            if (uriWrapper.getPath() != null)
                logger.debug("uriWrapper.getPath().length="
                        + uriWrapper.getPath().length());
            if ((uriWrapper.getPath() != null)
                    && (uriWrapper.getPath().length() > 1))
                //                uriResult.setPath(URI.addPaths(uriWrapper.getPath(),
                // uriResult.getPath()));
                uriResult.setPath(uriWrapper.getPath());
            //            uriResult.setQuery(URI.addPaths(uriResult.getQuery(), uriWrapper
            //                    .getQuery()));
            result = uriResult.toString();
        } else
            result = urlWrapper;
        logger.debug("buildWrapperURL returns=" + result);
        return result;
    }

    /**
     * Builds the wrapperURL to be set in the provider's interface. The URL is
     * created from the host URL concatenating the paht of the request URL.
     * 
     * @param urlRequest
     *            The URL of the current Request.
     * @param urlHost
     *            The URL of the provider's interface host.
     * @return The new wrapperURL or <code>null</code>.
     */
    private String buildHostURL(String urlRequest, String urlHost) {
        String result = null;
        logger.debug("buildHostURL urlRequest=" + urlRequest + " urlHost="
                + urlHost);
        if ((urlRequest != null) && (urlRequest.length() > 0)) {
            URI uriResult = new URI(urlHost);
            URI uriRequest = new URI(urlRequest);
            if ((uriRequest.getPath() != null)
                    || (uriRequest.getPath().length() > 1))
                uriResult.setPath(URI.addPaths(uriResult.getPath(), uriRequest
                        .getPath()));
            logger.debug("buildHostURL urlResult vor setQuery=" + uriResult);
            uriResult.setQuery(URI.addPaths(uriResult.getQuery(), uriRequest
                    .getQuery()));
            logger.debug("buildHostURL urlResult nach setQuery=" + uriResult);
            result = uriResult.toString();
        } else
            result = urlHost;
        logger.debug("buildHostURL returns=" + result);
        return result;
    }

    /**
     * Redefines the wrapper_url parameter of a Http-Request to the wrapperURL
     * build using the method
     * {@link #buildWrapperURL(String, String) buildWrapperURL(String, String)}.
     * 
     * @param httpRequest
     *            The request to modify.
     * @throws MalformedURLException
     */
    private void handleWrapperURLParameter(HttpRequest httpRequest)
            throws MalformedURLException {
        String wrapperURLParam = httpRequest.getParameter("wrapper_url");
        if ((wrapperURLParam != null) && (wrapperURLParam.length() > 0)) {
            wrapperURLParam = buildWrapperURL(httpRequest
                    .getParameter("wrapper_url"), wrapper.toString());
            if (wrapperURLParam != null)
                httpRequest.getParameters().put("wrapper_url", wrapperURLParam);
            logger.debug("handleWrapperURL() sets wrapperURL="
                    + httpRequest.getParameter("wrapper_url"));
        }
    }

    /**
     * Get the value of a parameter from the current query request.
     * 
     * @param query
     *            The query.
     * @param paramLabel
     *            The parameter to search for.
     * @return The parameter value or <code>null</code>.
     */
    private String getQueryParameterValue(String query, String paramLabel) {
        String result = null;
        if ((query != null) || query.length() > 0) {
            int paramEnd = -1;
            int paramStart = query.indexOf(paramLabel + "=");
            if (paramStart != -1) {
                if (query.indexOf("&", paramStart) == -1)
                    result = query.substring(paramStart + paramLabel.length()
                            + 1, query.length());
                else
                    result = query.substring(paramStart + paramLabel.length()
                            + 1, query.indexOf("&", paramStart));
            }
        }
        return result;
    }

    /**
     * Add a parameter to the given query request.
     * 
     * @param query
     *            The query of the current request.
     * @param paramLabel
     *            The label of the parameter to add.
     * @param paramValue
     *            The value of the parameter to add.
     * @return The new query or <code>null</code>.
     */
    private String addQueryParameter(String query, String paramLabel,
            String paramValue) {
        String result = null;
        if ((query == null) || query.length() == 0)
            result = paramLabel + "=" + paramValue;
        else if (query.indexOf(paramLabel + "=") == -1)
            result = query + "&" + paramLabel + "=" + paramValue;
        else {
            int paramEnd = -1;
            int paramStart = query.indexOf(paramLabel + "=");
            if (paramStart != -1) {
                if (query.indexOf("&", paramStart) == -1)
                    result = query.replaceFirst(paramLabel + "=.*", paramLabel
                            + "=" + paramValue);
                else
                    result = query.replaceFirst(paramLabel + "=.*?&",
                            paramLabel + "=" + paramValue + "&");
            }
        }
        return result;
    }

    private void addRoleQuery(Set roleIds, URI uri) throws ParsingException,
            URISyntaxException {
        logger.debug("URI vor addRoleQuery=\n" + uri);
        uri.setQuery(addQueryParameter(uri.getQuery(), "role",
                PDPFilterParameter.getRolesAsString(roleIds)));
        logger.debug("URI nach addRoleQuery=\n" + uri);
    }

    /**
     * Add the wrapper-URL to the url-header parameter for a Http-GET request.
     * 
     * @param uri
     *            The URI to build the wrapper url.
     * @throws ParsingException
     * @throws URISyntaxException
     */
    private void addURLQuery(URI uri) throws ParsingException,
            URISyntaxException {
        logger.debug("URI vor addURLQuery=\n" + uri);
        String wrapperURL = buildWrapperURL(getQueryParameterValue(uri
                .getQuery(), "url"), wrapper.toString());
        logger.debug("wrapperURL=\n" + wrapperURL);
        logger.debug("URI vor setQuery=\n" + uri);
        logger.debug("URI.getQuery() vor setQuery=\n" + uri.getQuery());
        uri.setQuery(addQueryParameter(uri.getQuery(), "url", wrapperURL));
        logger.debug("URI nach addURLQuery=\n" + uri);
    }

    /**
     * Reads the role parameter from the PyWrapper query, only if the requesting
     * role is set to guest.
     * 
     * @param roleIds
     *            The set of roles.
     * @param httpRequest
     *            The request to inspect.
     * @return The role parameter from the request.
     * @throws ParsingException
     * @throws URISyntaxException
     */
    private Set handleRoleParameter(Set roleIds, HttpRequest httpRequest)
            throws ParsingException, URISyntaxException {
        Set result = roleIds;

        // Read roles from role parameter ONLY when role is guest
        if ((roleIds != null)
                && (roleIds.size() == 1)
                && (((RoleAssignmentPolicyIdentifier) roleIds.toArray()[0])
                        .getLabel().equals(GuestRoleIdentifier.ROLE_LABEL))) {
            logger.debug("handleRoleParameter.getParameter(role)="
                    + httpRequest.getParameter("role"));
            Set paramRoleSet = PDPFilterParameter.getRoles(httpRequest
                    .getParameter("role"));
            if (paramRoleSet != null) {
                logger.debug("Got roles from Request Parameter: "
                        + paramRoleSet);
                result = paramRoleSet;
            }
            logger.debug("handleRoleParameter.result=" + result);
        }
        httpRequest.getParameters().put("role",
                PDPFilterParameter.getRolesAsString(result));
        logger.debug("HttpRequest nach handleRoleParameter=\n" + httpRequest);
        return result;
    }

    /**
     * Filters the incoming request filtering any BioCASE request using the
     * RBACPDP and rejecting all denied request. When an request is denied or an
     * error occurs, an error response is generated. The method returns the new
     * status for this PEPHandler.
     * 
     * @param httpRequest
     *            The request to filter.
     * @param roleIds
     *            The enabled role Id's for this request.
     * @param httpResponse
     *            The Http Resopnse
     * @return The request status.
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws UnsupportedEncodingException
     * @throws IOException
     */
    private int filterRequest(HttpRequest httpRequest, Set roleIds,
            HttpResponse httpResponse) throws ParserConfigurationException,
            SAXException, UnsupportedEncodingException, IOException {
        status = STATUS_REQUEST_NONE;
        InputStream xmlRequestIn = null;
        logger.debug("httpRequest=\n" + httpRequest);
        logger.debug("httpRequest.getContentType()="
                + httpRequest.getContentType());
        if (httpRequest.getContentType() != null) {
            if (httpRequest.getContentType().equals(
                    "application/x-www-form-urlencoded")) {
                String requestParam = null;
                logger.debug("http.getParameters="
                        + httpRequest.getParameters());
                if (((requestParam = httpRequest.getParameter("query")) != null)
                        || ((requestParam = httpRequest.getParameter("search")) != null)
                        || ((requestParam = httpRequest.getParameter("request")) != null)
                        || ((requestParam = httpRequest
                                .getParameter("protocol")) != null)
                        || ((requestParam = httpRequest.getParameter("biocase")) != null)
                        || ((requestParam = httpRequest.getParameter("doc")) != null)) {
                    logger.debug("requestParam=" + requestParam);
                    xmlRequestIn = new ByteArrayInputStream(requestParam
                            .getBytes());
                } else
                    logger.debug("No requestParam found");
            } else if (httpRequest.getContentType().equals("application/xml")
                    || httpRequest.getContentType().equals("text/xml")) {
                logger.debug("http.getParameters="
                        + httpRequest.getParameters());
                xmlRequestIn = httpRequest.getInputStream();
            } else
                throw new IllegalArgumentException("Unexpected Content-Type:"
                        + httpRequest.getContentType());

            if (xmlRequestIn != null) {
                SAXParserFactory factory = SAXParserFactory.newInstance();
                factory.setNamespaceAware(true);
                SAXParser saxParser = factory.newSAXParser();
                XMLReader reader = saxParser.getXMLReader();
                XMLDebugger requestDebugger = new XMLDebugger(reader,
                        new FileOutputStream("requestDebug.xml"));
                requestDebugger.setContentHandler(new XMLWriter(reader,
                        new FileOutputStream("request.xml")));
                RequestFilter requestFilter = new RequestFilter(reader,
                        ProtocolFilter.PROT_URI_13,
                        ProtocolFilter.FILTERMODE_FILTER);
                requestFilter.setFilterParameter(new PDPFilterParameter(pdp,
                        roleIds));
                requestFilter.setContentHandler(requestDebugger);
                reader.setContentHandler(requestFilter);
                try {
                    logger.debug("start Request PARSING");
                    // Parse the request using the RequestFilter
                    //                    reader.parse(new InputSource(xmlRequest));
                    reader.parse(new InputSource(new InputStreamReader(
                            xmlRequestIn, "UTF8")));
                    logger.debug("stop Request PARSING");
                    request = requestFilter.getRequest();
                    status = STATUS_REQUEST_OK;
                    logger.debug("Status set to " + status);
                } catch (SAXException e) {
                    //e.printStackTrace();
                    status = STATUS_REQUEST_ERROR;
                    logger.debug("STATUS_REQUEST_ERROR");
                    httpResponse = createErrorResponse(httpResponse,
                            requestFilter.getRequest(), e.getMessage());
                    logger.debug("HttpResponse-->" + httpResponse);
                }
            }
        }
        logger.debug("filterRequest() returns status " + status);
        return status;
    }

    /**
     * Filters the only responses corresponding to redirected requests and
     * filtering any BioCASE responses using the RBACPDP eliminating all XML
     * elements from the received document denied by the RBACPDP. When an error
     * occurs, an error response is generated. The method returns the new status
     * for this PEPHandler. Denied elements are reported in the BioCASE
     * Diagnostics.
     * 
     * @param httpConnection
     *            The connection to the PyWrapper
     * @param httpResponse
     *            The Http-response received
     * @param roleIds
     *            The roles enabled for this client request/response.
     * @return The response status.
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws UnsupportedEncodingException
     * @throws IOException
     */
    private int filterResponse(HttpURLConnection httpConnection,
            HttpResponse httpResponse, Set roleIds)
            throws ParserConfigurationException, SAXException,
            UnsupportedEncodingException, IOException {
        if (status == STATUS_REQUEST_OK) {
            // Filter the BioCASE response using RBACPDP when request was
            // permitted.
            logger.debug("STATUS_REQUEST_OK");
            logger.debug("httpConnection=\n" + httpConnection);
            // set response headers
            int h = 0;
            String hdr = httpConnection.getHeaderFieldKey(h);
            String val = httpConnection.getHeaderField(h);
            while (hdr != null || val != null) {
                if (hdr != null && val != null)
                    httpResponse.addField(hdr, val);
                h++;
                hdr = httpConnection.getHeaderFieldKey(h);
                val = httpConnection.getHeaderField(h);
            }

            // Set up SAXParser
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = null;
            factory.setNamespaceAware(true);
            //            httpConnection.setDoInput(true);

            OutputStream httpOut = httpResponse.getOutputStream();
            ByteArrayOutputStream contentFilterOut = new ByteArrayOutputStream();
            XMLWriter responseWriter = new XMLWriter(contentFilterOut);
            XMLDebugger responseDebugger = new XMLDebugger(
                    new FileOutputStream("responseDebug.xml"));
            responseDebugger.setContentHandler(new XMLWriter(
                    new FileOutputStream("response.xml")));
            responseWriter.setContentHandler(responseDebugger);

            try {
                saxParser = factory.newSAXParser();
                XMLReader reader = saxParser.getXMLReader();
                ResponseFilter responseFilter = new ResponseFilter(reader,
                        ProtocolFilter.PROT_URI_13,
                        ProtocolFilter.FILTERMODE_FILTER);
                responseFilter.setFilterParameter(new ResponseFilterParameter(
                        request, pdp, roleIds));
                responseFilter.setContentHandler(responseWriter);
                reader.setContentHandler(responseFilter);

                logger.debug("start PARSING");
                // Filter the response using the ResponseFilter
                //                reader.parse(new
                // InputSource(httpResponse.getBodyAsInputStream()));
                reader.parse(new InputSource(new InputStreamReader(
                        httpConnection.getInputStream(), "UTF8")));

                ByteArrayInputStream contentFilterIn = new ByteArrayInputStream(
                        contentFilterOut.toByteArray());
                logger.debug("stop PARSING");
                signResponse(contentFilterIn, httpOut, responseFilter);
                status = STATUS_RESPONSE_OK;
            } catch (Exception e) {
                status = STATUS_RESPONSE_ERROR;
                logger.debug("STATUS_RESPONSE_ERROR->\n" + e);
                httpResponse = createErrorResponse(httpResponse, request, e
                        .getMessage());
                logger.debug("HttpResponse-->" + httpResponse);
            }
        }
        return status;
    }

    private List createReferences(ResponseFilter responseFilter,
            XMLSignatureFactory fac) throws NoSuchAlgorithmException,
            InvalidAlgorithmParameterException {
        ArrayList ret = new ArrayList();
        Iterator seIt = responseFilter.getContentFilter().getSignElements()
                .iterator();
        logger
        .debug("createReferences(): responseFilter.getContentFilter().getContentNamespaceURI()= "
                + responseFilter.getContentFilter().getContentNamespaceURI());
        while (seIt.hasNext()) {
            String xPathRef = (String) seIt.next();
            xPathRef = xPathRef.replaceFirst(responseFilter.getContentFilter().getContentNamespaceURI(),"");
            ret.add(createXPathReference("//*[namespace-uri()='"+responseFilter.getContentFilter().getContentNamespaceURI()+"']" + xPathRef,
                    fac));
        }
        return ret;
    }

    private Reference createXPathReference(String xpath, XMLSignatureFactory fac)
            throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {
        logger.debug("createXPathReference( "+ xpath +" ):");
        XPathType xp = new XPathType(xpath, XPathType.Filter.INTERSECT);
        ArrayList xpathList = new ArrayList();
        xpathList.add(xp);
        ArrayList transforms = new ArrayList();
        transforms.add(fac.newTransform(Transform.XPATH2,
                new XPathFilter2ParameterSpec(xpathList)));
        return fac.newReference("", fac
                .newDigestMethod(DigestMethod.SHA1, null), transforms, null,
                null);
    }

    private void signResponse(ByteArrayInputStream contentIn,
            OutputStream contentOut, ResponseFilter responseFilter)
            throws Exception {
        logger.debug("START SIGNING: ");
        if (!responseFilter.getContentFilter().getSignElements().isEmpty()) {
            logger.debug("Elements to sign: "
                    + responseFilter.getContentFilter().getSignElements());

            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true); // must be set
            Document doc = dbf.newDocumentBuilder().parse(contentIn);
            String providerName = System.getProperty("jsr105Provider",
                    "org.jcp.xml.dsig.internal.dom.XMLDSigRI");
            XMLSignatureFactory fac = XMLSignatureFactory.getInstance("DOM",
                    (Provider) Class.forName(providerName).newInstance());

            // Create the SignedInfo
            SignedInfo si = fac.newSignedInfo(fac.newCanonicalizationMethod(
                    CanonicalizationMethod.INCLUSIVE,
                    (C14NMethodParameterSpec) null), fac.newSignatureMethod(
                    SignatureMethod.RSA_SHA1, null), createReferences(
                    responseFilter, fac));

            KeyInfoFactory kif = fac.getKeyInfoFactory();

            // Create a X509Data content containing providers X509Certificate
            ArrayList x509Content = new ArrayList();
            x509Content.add(cert);
            X509Data x509Data = kif.newX509Data(x509Content);

            // Create a KeyInfo content containing theprovider's X509Certificate
            ArrayList keyinfoContent = new ArrayList();
            keyinfoContent.add(x509Data);
            // Create a KeyInfo and add the KeyValue to it
            KeyInfo ki = kif.newKeyInfo(keyinfoContent);

            // Create the XMLSignature (but don't sign it yet)
            XMLSignature signature = fac.newXMLSignature(si, ki);

            // Create a DOMSignContext and set the signing Key to the DSA
            // PrivateKey and specify where the XMLSignature should be
            // inserted
            // in the target document (in this case, the document root)
            NodeList nl = doc.getElementsByTagName("content");
            if (nl.getLength() == 0) {
                logger.debug("No Content Element found to sign.");
                IO.copy(contentIn, contentOut);
            } else {

                DOMSignContext signContext = new DOMSignContext(privKey, nl
                        .item(0));
                signContext.setProperty("javax.xml.crypto.dsig.cacheReference",
                        Boolean.TRUE);

                // Marshal, generate (and sign) the detached XMLSignature. The
                // DOM
                // Document will contain the XML Signature if this method
                // returns
                // successfully.
                signature.sign(signContext);
                Iterator refIt = signature.getSignedInfo().getReferences()
                        .iterator();
                while (refIt.hasNext()) {
                    Reference ref = (Reference) refIt.next();
                    logger.debug("DereferencedData:"
                            + ref.getDereferencedData());
                    logger.debug("DigestInputStream:"
                            + ref.getDigestInputStream());
                }
                TransformerFactory tf = TransformerFactory.newInstance();
                Transformer trans = tf.newTransformer();
                Properties outputProperties = new Properties();
                outputProperties.setProperty(OutputKeys.INDENT, "yes" );
                trans.setOutputProperties(outputProperties);
                trans.transform(new DOMSource(doc),
                        new StreamResult(contentOut));
                trans.transform(new DOMSource(doc),
                        new StreamResult(new FileOutputStream("responseSigned.xml")));
            }
        } else {
            logger.debug("No elements to sign.");
            IO.copy(contentIn, contentOut);
        }
        logger.debug("STOP SIGNING: ");

    }

    /**
     * Creates an BioCASE error response.
     * 
     * @param httpResponse
     *            The Http-response to be filled with the error message
     * @param request
     *            The BioCASE-request
     * @param errMess
     *            The error message to be included in the Diagnostics of the
     *            response.
     * @return Http Error response.
     * @throws IOException
     */
    private HttpResponse createErrorResponse(HttpResponse httpResponse,
            Request request, String errMess) throws IOException {
        httpResponse.setStatus(HttpResponse.__200_OK);

        String protocolURI = request == null ? ProtocolFilter.PROT_URI_13
                : request.getProtocolURI();
        Response protResp = new Response(protocolURI);
        Header protRespHeader = new Header(protocolURI);
        Diagnostics protRespDiagnostics = new Diagnostics(protocolURI);
        protRespHeader
                .addVersion(PEPHandler.version, this.getClass().getName());
        protRespHeader.setSendTime((new Date()).toString());
        protRespHeader.setType("error");
        Diagnostic protRespDiagnostic = new Diagnostic(protocolURI);
        protRespDiagnostic.setValue(errMess);
        protRespDiagnostics.addDiagnostic(protRespDiagnostic);
        protResp.setHeader(protRespHeader);
        protResp.setDiagnostics(protRespDiagnostics);

        ByteArrayOutputStream respBody = new ByteArrayOutputStream();
        XMLWriter responseWriter = new XMLWriter(respBody);
        try {
            protResp.write(responseWriter);
        } catch (SAXException e) {
            e.printStackTrace();
        }
        if (respBody.size() > 0) {
            httpResponse.setContentLength(respBody.size());
            httpResponse.setContentType("text/xml");
            IO.copy(new ByteArrayInputStream(respBody.toByteArray()),
                    httpResponse.getOutputStream());
        }
        return httpResponse;
    }

    /**
     * The handlers main function called from Jetty. Processes the handling of
     * requests, responses and error responses calling the methods filterRequest
     * and filterResponse.
     * 
     * @see org.mortbay.http.HttpHandler#handle(java.lang.String,
     *      java.lang.String, org.mortbay.http.HttpRequest,
     *      org.mortbay.http.HttpResponse)
     */
    public void handle(String pathInContext, String pathParams,
            HttpRequest request, HttpResponse response) throws HttpException,
            IOException {
        HttpURLConnection hostConnection = null;
        logger.debug("handle().request.getParameters="
                + request.getParameters());
        Set roleIds = getRoles(request);
        try {
            if (request.getMethod() == HttpRequest.__POST) {
                handleWrapperURLParameter(request);
                logger.debug("roleIds=" + roleIds);
                roleIds = handleRoleParameter(roleIds, request);
                logger.debug("roleIds=" + roleIds);
            }
            filterRequest(request, roleIds, response);

            if (status == STATUS_REQUEST_OK) {

                logger.debug("STATUS_REQUEST_OK");
                request.setContentType("application/x-www-form-urlencoded");
                hostConnection = openHostConnection(request, roleIds);

                filterResponse(hostConnection, response, roleIds);
            } else if (status == STATUS_REQUEST_NONE) {
                hostConnection = openHostConnection(request, roleIds);
                InputStream proxy_in = null;

                // handler status codes etc.
                int code = HttpResponse.__500_Internal_Server_Error;
                if (hostConnection != null) {
                    proxy_in = hostConnection.getErrorStream();

                    code = hostConnection.getResponseCode();
                    response.setStatus(code);
                    response.setReason(hostConnection.getResponseMessage());
                }

                if (proxy_in == null) {
                    try {
                        proxy_in = hostConnection.getInputStream();
                    } catch (Exception e) {
                        proxy_in = hostConnection.getErrorStream();
                    }
                }

                response.removeField(HttpFields.__Date);
                response.removeField(HttpFields.__Server);

                if (proxy_in != null)
                    IO.copy(proxy_in, response.getOutputStream());

            }
            //            if(status==STATUS_REQUEST_ERROR) Do Nothing means send the
            // generated Error Response
            request.setHandled(true);
        } catch (ParsingException e) {
            e.printStackTrace();
        } catch (URISyntaxException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }
}
