/*
 * $Id: COP.java,v 1.8 2006/07/18 14:58:11 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: COP.java,v $
 * Revision 1.8  2006/07/18 14:58:11  suhrbier
 * Fehler behoben in ContentObjectFilter
 * + PEPClient send Request
 * Fehler in der Request.write()-Kette behoben.
 *
 * Revision 1.7  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.6  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.5  2005/08/07 16:15:31  suhrbier
 * Umstellung Request auf Filter
 *
 * Revision 1.4  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.3  2005/07/21 19:00:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/07/06 12:14:11  suhrbier
 * beginn RBACFilter
 *
 * Revision 1.1  2005/03/07 18:22:30  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/01/17 15:37:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/14 20:58:45  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/13 17:50:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:56  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.COPFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol operators COP.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.8 $
 * @since
 */

public class COP extends Operator {
    static Logger logger = Logger.getLogger(COP.class);

    /** Path attribute of COPs */
    protected String path;

    /** COP values */
    protected List values;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public COP(String protocolURI, String type) {
        super(protocolURI, type);
        this.type = type;
    }

    /**
     * Writes the COP to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        SAXElement element = new SAXElement();
        element.setURI(getProtocolURI());
        if ((values == null) || (values.size() <= 1)) {
            element.setLocalQName(getType());
            element.addAttribute(getProtocolURI(), COPFilter.LN_COP_PATH,
                    COPFilter.LN_COP_PATH, getPath());
            if (values != null)
                element.setValue(getValue());
            element.write(out);
        } else {
            out.startElement(getProtocolURI(), getType(), getType(),
                    SAXElement.EMPTY_ATTS);
            Iterator valueIt = values.iterator();
            while (valueIt.hasNext()) {
                element.setLocalQName(COPFilter.LN_VALUE);
                element.setValue((String) valueIt.next());
                element.write(out);
            }
            out.endElement(getProtocolURI(), getType(), getType());
        }
    }

    /**
     * Sets path
     * 
     * @param path
     *            The path to set.
     */
    public void setPath(String path) {
        this.path = path;
    }

    /**
     * Gets the path
     * 
     * @return Returns the path.
     */
    public String getPath() {
        return path;
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        if (path != null)
            ret.append("path=" + path + ",");
        return ret.toString();
    }

    /**
     * Gets the values
     * 
     * @return Returns the values.
     */
    public List getValues() {
        return values;
    }

    /**
     * Gets the value at the specified position
     * 
     * @param index
     *            index of the value to return
     * @return value at the specified index, or null.
     */
    public String getValue(int index) {
        return values == null ? null : (String) values.get(index);
    }

    /**
     * Gets the value at the first position.
     * 
     * @return value at the index 0, or null.
     */
    public String getValue() {
        return getValue(0);
    }

    /**
     * Sets values
     * 
     * @param values
     *            The values to set.
     */
    public void setValues(List values) {
        this.values = values;
    }

    /**
     * Sets the value at the first position.
     * 
     * @param value
     *            value at the index 0, or null.
     */
    public void setValue(String value) {
        if (values == null)
            values = new ArrayList(1);
        if (values.isEmpty())
            values.add(value);
        else
            values.set(0, value);
    }

    /**
     * Appends the specified value to the values list. If no values exists, a
     * new Vector will be created.
     * 
     * @param value
     *            value to be appended.
     * @return true
     */
    public void addValue(String value) {
        if (values == null)
            values = new ArrayList();
        values.add(value);
    }

    /**
     * Removes the specified value from the values list.
     * 
     * @param value
     *            value to be removed.
     * @return true if the the values list contained specified value.
     */
    public void removeValue(String value) {
        values.remove(value);
    }
}
