/*
 * $Id: Capabilities.java,v 1.2 2005/08/15 21:58:55 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Capabilities.java,v $
 * Revision 1.2  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.1  2005/03/07 18:22:29  suhrbier
 * Initiale Version
 *
 * Revision 1.3  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/19 17:17:24  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.Iterator;
import java.util.Vector;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.ContentFilter;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol content object of type Capabilities.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */
public class Capabilities extends ContentObject {

    /** Vector of supported schemas */
    Vector supportedSchemas;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public Capabilities(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes the capabilities content object to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        out.startElement(getProtocolURI(), ContentFilter.LN_CAPABILITIES,
                ContentFilter.LN_CAPABILITIES, SAXElement.EMPTY_ATTS);
        Iterator schemasIt = supportedSchemas.iterator();
        while (schemasIt.hasNext()) {
            ((SupportedSchema) schemasIt.next()).write(out);
        }
        out.endElement(getProtocolURI(), ContentFilter.LN_CAPABILITIES,
                ContentFilter.LN_CAPABILITIES);
    }
    /**
     * Gets the supported schema at the specified position
     * 
     * @param index
     *            index of the schema to return
     * @return schema at the specified index, or null.
     */
    public SupportedSchema getSupportedSchema(int index) {
        SupportedSchema ret = null;
        if (supportedSchemas != null)
            ret = (SupportedSchema) supportedSchemas.get(index);
        return ret;
    }

    /**
     * Appends the specified schema to the schema vector. If no schemas exists,
     * a new Vector will be created.
     * 
     * @param schema
     *            schema to be appended.
     * @return true
     */
    public boolean addSupportedSchema(SupportedSchema schema) {
        if (supportedSchemas == null)
            supportedSchemas = new Vector();
        return supportedSchemas.add(schema);
    }

    /**
     * Removes the specified supported schema from the schema vector.
     * 
     * @param schema
     *            supported schema to be removed.
     * @return true if the schema vector contained the specified schema.
     */
    public boolean removeSupportedSchema(SupportedSchema schema) {
        boolean ret = false;
        if (supportedSchemas != null)
            ret = supportedSchemas.remove(schema);
        return ret;
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("capabilities=(SupportedSchemas=");
        if (supportedSchemas != null)
            if (supportedSchemas.isEmpty())
                ret.append("empty");
            else {
                Iterator supportedSchemasIt = supportedSchemas.iterator();
                for (int i = 1; supportedSchemasIt.hasNext(); i++) {
                    ret.append("(");
                    ret
                            .append(i
                                    + ":"
                                    + (SupportedSchema) supportedSchemasIt
                                            .next() + ",");
                    ret.append(")");
                    if (supportedSchemasIt.hasNext())
                        ret.append(",\n");
                }
            }
        ret.append(")");
        return ret.toString();
    }

    /**
     * Gets the supportedSchemas
     * 
     * @return Returns the supportedSchemas.
     */
    public Vector getSupportedSchemas() {
        return supportedSchemas;
    }

    /**
     * Sets supportedSchemas
     * 
     * @param supportedSchemas
     *            The supportedSchemas to set.
     */
    public void setSupportedSchemas(Vector supportedSchemas) {
        this.supportedSchemas = supportedSchemas;
    }
}
