/*
 * $Id: Content.java,v 1.7 2005/12/19 19:11:04 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Content.java,v $
 * Revision 1.7  2005/12/19 19:11:04  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.6  2005/08/15 22:10:26  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.5  2005/08/15 21:58:54  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.4  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/07 16:15:31  suhrbier
 * Umstellung Request auf Filter
 *
 * Revision 1.1  2005/03/07 18:22:30  suhrbier
 * Initiale Version
 *
 * Revision 1.6  2005/01/24 20:55:36  suhrbier
 * Response-Scannen komplett.
 *
 * Revision 1.5  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/01/19 17:17:24  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/18 12:03:24  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/17 17:37:23  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:56  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.Iterator;
import java.util.Vector;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.ContentFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * This class wraps the BioCASE protocol element Content. BioCASE content objects are stored in the vector {@link #contentObjects contentObjects}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */

public class Content extends ProtocolType {
    static Logger logger = Logger.getLogger(Content.class);

    /** Content attribute <code>recordStart</code> */
    private Integer recordStart;

    /** Content attribute <code>recordCount</code> */
    private Integer recordCount;

    /** Content attribute <code>recordDropped</code> */
    private Integer recordDropped;

    /** Content attribute <code>totalSearchHits</code> */
    private Integer totalSearchHits;

    /** List of content objects */
    private Vector contentObjects;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public Content(String protocolURI) {
        super(protocolURI);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("(");
        ret.append("recordStart=" + recordStart + ",");
        ret.append("recordCount=" + recordCount + ";");
        ret.append("recordDropped=" + recordDropped + ",");
        ret.append("totalSearchHits=" + totalSearchHits + ",");
        ret.append("contentObjects=");
        if (contentObjects != null)
            if (contentObjects.isEmpty())
                ret.append("empty");
            else {
                Iterator contentObjectsIt = contentObjects.iterator();
                for (int i = 1; contentObjectsIt.hasNext(); i++) {
                    ret.append("(");
                    ret.append(i + ":"
                            + (ContentObject) contentObjectsIt.next() + ",");
                    ret.append(")");
                    if (contentObjectsIt.hasNext())
                        ret.append(",");
                }
            }
        ret.append(")");
        return ret.toString();
    }

    /**
     * Writes the content to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        AttributesImpl atts = SAXElement.EMPTY_ATTS;
        if (recordCount != null)
            atts.addAttribute(getProtocolURI(),
                    ContentFilter.LN_CONTENT_RECORDCOUNT,
                    ContentFilter.LN_CONTENT_RECORDCOUNT, "xsd:integer",
                    recordCount.toString());
        if (recordDropped != null)
            atts.addAttribute(getProtocolURI(),
                    ContentFilter.LN_CONTENT_RECORDDROPPED,
                    ContentFilter.LN_CONTENT_RECORDDROPPED, "xsd:integer",
                    recordDropped.toString());
        if (recordStart != null)
            atts.addAttribute(getProtocolURI(),
                    ContentFilter.LN_CONTENT_RECORDSTART,
                    ContentFilter.LN_CONTENT_RECORDSTART, "xsd:integer",
                    recordStart.toString());
        if (totalSearchHits != null)
            atts.addAttribute(getProtocolURI(),
                    ContentFilter.LN_CONTENT_TOTALSEARCHHITS,
                    ContentFilter.LN_CONTENT_TOTALSEARCHHITS, "xsd:integer",
                    totalSearchHits.toString());
        out.startElement(getProtocolURI(), ContentFilter.LN_ROOT,
                ContentFilter.LN_ROOT, atts);
        Iterator contentObjectsIt = contentObjects.iterator();
        while (contentObjectsIt.hasNext()) {
            ((ContentObject) contentObjectsIt.next()).write(out);
        }
        out.endElement(getProtocolURI(), ContentFilter.LN_ROOT,
                ContentFilter.LN_ROOT);
    }

    /**
     * Appends the specified content object to the contents vector. If no
     * content objects exist, a new Vector will be created.
     * 
     * @param content
     *            content object to be appended.
     * @return true
     */
    public void addContentObject(ContentObject content) {
        if (contentObjects == null)
            contentObjects = new Vector();
        contentObjects.add(content);
    }

    /**
     * Removes the specified content object from the content objects vector.
     * 
     * @param content
     *            content object to be removed.
     * @return true if the content object vector contained the specified content
     *         object.
     */
    public boolean removeContentObject(ContentObject content) {
        boolean ret = false;
        if (content != null)
            ret = contentObjects.remove(content);
        return ret;
    }

    /**
     * Gets the content object at the specified position
     * 
     * @param index
     *            index of the content object to return
     * @return content object at the specified index, or null.
     */
    public ContentObject getContentObject(int index) {
        return (ContentObject) contentObjects.get(index);
    }

    /**
     * Gets the contentObjects
     * 
     * @return Returns the contentObjects.
     */
    public Vector getContentObjects() {
        return contentObjects;
    }

    /**
     * Sets contentObjects
     * 
     * @param contentObjects
     *            The contentObjects to set.
     */
    public void setContentObjects(Vector contentObjects) {
        this.contentObjects = contentObjects;
    }

    /**
     * Gets the recordCount
     * 
     * @return Returns the recordCount or null if it is not set.
     */
    public Integer getRecordCount() {
        return recordCount;
    }

    /**
     * Sets recordCount
     * 
     * @param recordCount
     *            The recordCount to set.
     */
    public void setRecordCount(Integer recordCount) {
        this.recordCount = recordCount;
    }

    /**
     * Gets the recordDropped
     * 
     * @return Returns the recordDropped or null if it is not set.
     */
    public Integer getRecordDropped() {
        return this.recordDropped;
    }

    /**
     * Sets recordDropped
     * 
     * @param recordDropped
     *            The recordDropped to set or null.
     */
    public void setRecordDropped(Integer recordDropped) {
        this.recordDropped = recordDropped;
    }

    /**
     * Gets the recordStart
     * 
     * @return Returns the recordStart or null if it is not set.
     */
    public Integer getRecordStart() {
        return recordStart;
    }

    /**
     * Sets recordStart
     * 
     * @param recordStart
     *            The recordStart to set.
     */
    public void setRecordStart(Integer recordStart) {
        this.recordStart = recordStart;
    }

    /**
     * Gets the totalSearchHits
     * 
     * @return Returns the totalSearchHits or null if it is not set.
     */
    public Integer getTotalSearchHits() {
        return totalSearchHits;
    }

    /**
     * Sets totalSearchHits
     * 
     * @param totalSearchHits
     *            The totalSearchHits to set.
     */
    public void setTotalSearchHits(Integer totalSearchHits) {
        this.totalSearchHits = totalSearchHits;
    }
}
