/*
 * $Id: Diagnostic.java,v 1.7 2006/07/24 08:15:36 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Diagnostic.java,v $
 * Revision 1.7  2006/07/24 08:15:36  suhrbier
 * + class SecurityDiagnostic
 *
 * Revision 1.6  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.5  2005/10/07 18:27:46  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.4  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.3  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.2  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.1  2005/03/07 18:22:29  suhrbier
 * Initiale Version
 *
 * Revision 1.3  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/17 17:37:23  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.DiagnosticsFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol element Diagnostic.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */
public class Diagnostic extends ProtocolType {
    static Logger logger = Logger.getLogger(Diagnostic.class);

    /** Prefefined severity value <code>debug</code> */
    public static final String SEVERITY_DEBUG = "debug";

    /** Prefefined severity value <code>info</code> */
    public static final String SEVERITY_INFO = "info";

    /** Prefefined severity value <code>warn</code> */
    public static final String SEVERITY_WARN = "warn";

    /** Prefefined severity value <code>error</code> */
    public static final String SEVERITY_ERROR = "error";

    /** Prefefined severity value <code>fatal</code> */
    public static final String SEVERITY_FATAL = "fatal";

    /** Diagnostic value */
    private String value;

    /** Diagnostic code */
    private String code;

    /** Diagnostic severity */
    private String severity;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public Diagnostic(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     * @param code The code attribute of this diagnostic.
     * @param severity The severityattribute of this diagnostic.
     * @param value The value of this diagnostic.
     */
    public Diagnostic(String protocolURI, String code, String severity,
            String value) {
        super(protocolURI);
        setCode(code);
        setSeverity(severity);
        setValue(value);
    }

    /**
     * Writes the capabilities content object to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        SAXElement element = new SAXElement();
        element.setURI(getProtocolURI());
        element.setLocalQName(DiagnosticsFilter.LN_DIAGNOSTIC);
        if (code != null)
            element.addAttribute(getProtocolURI(),
                    DiagnosticsFilter.LN_DIAGNOSTIC_CODE,
                    DiagnosticsFilter.LN_DIAGNOSTIC_CODE, "xsd:string", code);
        if (severity != null)
            element.addAttribute(getProtocolURI(),
                    DiagnosticsFilter.LN_DIAGNOSTIC_SEVERITY,
                    DiagnosticsFilter.LN_DIAGNOSTIC_SEVERITY, "xsd:string",
                    severity);
        if (value != null)
            element.setValue(value);
        element.write(out);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(getSeverity());
        ret.append(",");
        ret.append(getValue());
        ret.append(",");
        ret.append(getCode());
        return ret.toString();
    }

    /**
     * Gets the code.
     * 
     * @return code or null.
     */
    public String getCode() {
        return code;
    }

    /**
     * Sets the code.
     * 
     * @param code
     *            code to set.
     */
    public void setCode(String code) {
        this.code = code;
    }

    /**
     * Gets the severity.
     * 
     * @return severity or null.
     */

    public String getSeverity() {
        return severity;
    }

    /**
     * Sets the severity. Severity must be one of "debug", "error", "fatal",
     * "info" or "warn".
     * 
     * @param severity
     *            severity to set.
     */
    public void setSeverity(String severity) {
        if (SEVERITY_DEBUG.equals(severity) || SEVERITY_ERROR.equals(severity)
                || SEVERITY_FATAL.equals(severity)
                || SEVERITY_INFO.equals(severity)
                || SEVERITY_WARN.equals(severity))
            this.severity = severity;
        //        else TODO: Muss wieder rein, wenn provider richtige diagnostics
        // liefert !!!
        //            throw new IllegalArgumentException("Unknown severity " + severity
        //                    + ".");
    }

    /**
     * Gets the value.
     * 
     * @return value or null.
     */
    public String getValue() {
        return value;
    }

    /**
     * Sets the value.
     * 
     * @param value
     *            value to set.
     */
    public void setValue(String value) {
        this.value = value;
    }
}
