/*
 * $Id: Diagnostics.java,v 1.7 2006/07/17 20:27:10 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Diagnostics.java,v $
 * Revision 1.7  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.6  2005/10/07 18:27:46  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.5  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.4  2005/08/15 22:10:26  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.3  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.2  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 * Revision 1.1  2005/03/07 18:22:30  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/01/24 20:55:36  suhrbier
 * Response-Scannen komplett.
 *
 * Revision 1.3  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/17 17:37:23  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.Iterator;
import java.util.Vector;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.DiagnosticsFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol element Diagnostic.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */
public class Diagnostics extends ProtocolType {
    static Logger logger = Logger.getLogger(Diagnostics.class);

    /** Diagnostic vector */
    Vector diagnostics = null;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public Diagnostics(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes the diagnostics to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        out.startElement(getProtocolURI(), DiagnosticsFilter.LN_ROOT,
                DiagnosticsFilter.LN_ROOT, SAXElement.EMPTY_ATTS);
        Iterator diagnosticIt = diagnostics.iterator();
        while (diagnosticIt.hasNext()) {
            ((Diagnostic) diagnosticIt.next()).write(out);
        }
        out.endElement(getProtocolURI(), DiagnosticsFilter.LN_ROOT,
                DiagnosticsFilter.LN_ROOT);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        if (diagnostics != null)
            if (diagnostics.isEmpty())
                ret.append("empty");
            else {
                Iterator diagnosticsIt = diagnostics.iterator();
                for (int i = 1; diagnosticsIt.hasNext(); i++) {
                    ret.append("(");
                    ret.append(i + ":" + ((Diagnostic) diagnosticsIt.next()).toString());
                    ret.append(")");
                    if (diagnosticsIt.hasNext())
                        ret.append(",");
                }
            }
        return ret.toString();
    }

    /**
     * Appends the specified diagnostic to the diagnostics vector. If no diagnostics exists,
     * a new Vector will be created.
     * 
     * @param diagnostic
     *            diagnostic to be appended.
     * @return true
     */
    public void addDiagnostic(Diagnostic diagnostic) {
        if (diagnostics == null)
            diagnostics = new Vector();
        diagnostics.add(diagnostic);
    }

    /**
     * Appends the specified diagnostics to the diagnostics vector. If no diagnostics exists,
     * a new Vector will be created.
     * 
     * @param diagnostics
     *            diagnostics to be appended.
     * @return true
     */
    public void addDiagnostics(Diagnostics diagnostics) {
        if ((diagnostics != null)&&(diagnostics.getDiagnostics()!=null)&&(!diagnostics.getDiagnostics().isEmpty()))
        {
            Iterator diagIt = diagnostics.getDiagnostics().iterator();
            while( diagIt.hasNext() )
            {
                addDiagnostic((Diagnostic)diagIt.next());
            }
        }
    }

    /**
     * Removes the specified supported schema from the schema vector.
     * 
     * @param diagnostic
     *            diagnostic to be removed.
     * @return true if the diagnostics vector contained the specified schema.
     */
    public boolean removeDiagnostic(Diagnostic diagnostic) {
        boolean ret = false;
        if (diagnostic != null)
            ret = diagnostics.remove(diagnostic);
        return ret;
    }

    /**
     * Gets the diagnostic at the specified position
     * 
     * @param index
     *            index of the diagnostic to return
     * @return diagnostic at the specified index, or null.
     */
    public Diagnostic getDiagnostic(int index) {
        return (Diagnostic) diagnostics.get(index);
    }
    /**
     * Gets the diagnostics
     * @return Returns the diagnostics.
     */
    public Vector getDiagnostics() {
        return diagnostics;
    }
    /**
     * Sets diagnostics
     * @param diagnostics The diagnostics to set.
     */
    public void setDiagnostics(Vector diagnostics) {
        this.diagnostics = diagnostics;
    }
}
