/*
 * $Id: Header.java,v 1.8 2006/07/17 20:27:10 suhrbier Exp $
 * Created on 11.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Header.java,v $
 * Revision 1.8  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.7  2006/05/08 17:21:57  suhrbier
 * Transform Response into DOM Document.
 *
 * Revision 1.6  2005/10/07 18:27:46  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.5  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.4  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 * Revision 1.2  2005/07/22 17:28:06  suhrbier
 * ResponseFilter implementiert, aber ungetestet.
 *
 * Revision 1.1  2005/03/07 18:22:29  suhrbier
 * Initiale Version
 *
 * Revision 1.8  2005/02/11 17:48:07  suhrbier
 * Erste Verbindung BioCASE->PDP
 *
 * Revision 1.7  2005/01/24 20:55:35  suhrbier
 * Response-Scannen komplett.
 *
 * Revision 1.6  2005/01/20 14:37:31  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/01/17 17:37:23  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/01/14 18:13:41  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/14 17:05:09  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/13 17:50:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/11 17:40:28  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.HeaderFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol element Header.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.8 $
 * @since
 */

public class Header extends ProtocolType {
    static Logger logger = Logger.getLogger(Header.class);

    /** List of Header version treeElements */
    private ArrayList version = null;

    /** List of Header version treeElements Software attributes */
    private ArrayList versionSoftware = null;

    /** Header sendtime element */
    private String sendTime;

    /** List of Header source treeElements */
    private ArrayList source = null;

    /** List of Header source treeElements Resource attributes */
    private ArrayList sourceResource = null;

    /** List of Header destination treeElements */
    private ArrayList destination = null;

    /** List of Header destination treeElements Resource attributes */
    private ArrayList destinationResource = null;

    /** Header type element */
    private String type;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public Header(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes single value/attribute pairs of two lists to a SAX
     * {@link ContentHandler ContentHandler}. Value and attribute type is set
     * to String(xsd:string). The value/attribute pairs are determined by the
     * index in the lists. Local name and qualified name are set to the same
     * value.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @param values
     *            List of values
     * @param attributes
     *            List of attributes
     * @param localNameValue
     *            Local name of all values
     * @param localNameAttr
     *            Local name of all attributes.
     * @throws SAXException
     */
    public void writeStringLists(ContentHandler out, List values,
            List attributes, String localNameValue, String localNameAttr)
            throws SAXException {
        Iterator valuesIt = values.iterator();
        Iterator attributesIt = attributes.iterator();
        // Init SAX element to write
        SAXElement element = new SAXElement();
        element.setURI(protocolURI);
        element.setLocalQName(localNameValue);
        while (valuesIt.hasNext() && attributesIt.hasNext()) {
            // For all values, write the value and the corresponding attribute
            // to the SAX stream
            String value = (String) valuesIt.next();
            String attribute = (String) attributesIt.next();
            if (attribute != null)
                element.addAttribute(protocolURI, localNameAttr, localNameAttr,
                        "xsd:string", attribute);
            if (value != null)
                element.setValue(value);
            element.write(out);
            element.clearAttributes();
        }
    }

    /**
     * Writes the header to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        SAXElement element = new SAXElement();
        element.setURI(protocolURI);
        out.startElement(protocolURI, HeaderFilter.LN_ROOT,
                HeaderFilter.LN_ROOT, SAXElement.EMPTY_ATTS);

        if ((version != null) && (versionSoftware != null))
            writeStringLists(out, version, versionSoftware,
                    HeaderFilter.LN_VERSION, HeaderFilter.LN_VERSION_SOFTWARE);

        if (sendTime != null) {
            element.setLocalQName(HeaderFilter.LN_SENDTIME);
            element.setValue(sendTime);
            element.write(out);
        }

        if ((source != null) && (sourceResource != null))
            writeStringLists(out, source, sourceResource, HeaderFilter.LN_SRC,
                    HeaderFilter.LN_SRC_RESOURCE);
        if ((destination != null) && (destinationResource != null))
            writeStringLists(out, destination, destinationResource,
                    HeaderFilter.LN_DEST, HeaderFilter.LN_DEST_RESOURCE);

        if (type != null) {
            element.setLocalQName(HeaderFilter.LN_TYPE);
            element.setValue(type);
            element.write(out);
        }

        out.endElement(protocolURI, HeaderFilter.LN_ROOT, HeaderFilter.LN_ROOT);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("(");
        if (version.isEmpty())
            ret.append("version=empty,");
        else {
            ret.append("version=");
            Iterator versionIt = version.iterator();
            Iterator versionSoftwareIt = versionSoftware.iterator();
            for (int i = 0; i < version.size(); i++) {
                ret.append("(");
                ret.append((i + 1) + ":" + (String) versionIt.next() + ","
                        + (String) versionSoftwareIt.next());
                ret.append(")");
                if (i < (version.size() - 1))
                    ret.append(",");
            }
        }

        if (sendTime != null)
            ret.append("sendTime=" + sendTime + ",");
        if (source.isEmpty())
            ret.append("source=empty,");
        else {
            ret.append("source=");
            Iterator sourceIt = source.iterator();
            Iterator sourceResourceIt = sourceResource.iterator();
            for (int i = 0; i < source.size(); i++) {
                ret.append("(");
                ret.append((i + 1) + ":" + (String) sourceIt.next() + ","
                        + (String) sourceResourceIt.next());
                ret.append(")");
                if (i < (source.size() - 1))
                    ret.append(",");
            }
        }
        if (destination.isEmpty())
            ret.append("destination=empty,");
        else {
            ret.append("destination=");
            Iterator destinationIt = destination.iterator();
            Iterator destinationResourceIt = destinationResource.iterator();
            for (int i = 0; i < destination.size(); i++) {
                ret.append("(");
                ret.append((i + 1) + ":" + (String) destinationIt.next() + ","
                        + (String) destinationResourceIt.next());
                ret.append(")");
                if (i < (destination.size() - 1))
                    ret.append(",");
            }
        }
        if (type != null)
            ret.append("type=" + type);
        ret.append(")");
        return ret.toString();
    }

    public ArrayList getDestination() {
        return destination;
    }

    public ArrayList getDestinationResource() {
        return destinationResource;
    }

    public String getSendTime() {
        return sendTime;
    }

    public String getSource() {
        String ret = null;
        if (source != null)
            if (!source.isEmpty())
                ret = (String) source.get(0);
        return ret;
    }

    public String getSourceResource() {
        return (String) sourceResource.get(0);
    }

    public String getType() {
        return type;
    }

    public ArrayList getVersion() {
        return version;
    }

    public ArrayList getVersionSoftware() {
        return versionSoftware;
    }

    public void addVersion(String version, String versionSoftware) {
        if (version != null) {
            if (this.version == null) {
                this.version = new ArrayList();
                this.versionSoftware = new ArrayList();
            }
            logger.debug("addVersion("+version+", "+versionSoftware+")");
            this.version.add(version);
            this.versionSoftware.add(versionSoftware);
        }
    }

    public void addSource(String source, String sourceResource) {
        // In version 1.3 only one Source is allowed !!!
        if (source != null) {
            if (this.source == null) {
                this.source = new ArrayList(1);
                this.source.add("");
                this.sourceResource = new ArrayList(1);
                this.sourceResource.add("");
            }
            this.source.set(0, source);
            this.sourceResource.set(0, sourceResource);
        }
    }

    public void addDestination(String destination, String destinationResource) {
        if (destination != null) {
            if (this.destination == null) {
                this.destination = new ArrayList();
                this.destinationResource = new ArrayList();
            }
            this.destination.add(destination);
            this.destinationResource.add(destinationResource);
        }
    }

    public void setSendTime(String sendTime) {
        this.sendTime = sendTime;
    }

    public void setType(String type) {
        this.type = type;
    }
}
