/*
 * $Id: LOP.java,v 1.6 2006/07/18 14:58:12 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: LOP.java,v $
 * Revision 1.6  2006/07/18 14:58:12  suhrbier
 * Fehler behoben in ContentObjectFilter
 * + PEPClient send Request
 * Fehler in der Request.write()-Kette behoben.
 *
 * Revision 1.5  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.4  2005/08/10 20:57:13  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.3  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.2  2005/08/07 16:15:31  suhrbier
 * Umstellung Request auf Filter
 *
 * Revision 1.1  2005/03/07 18:22:30  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/01/17 15:37:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/14 20:58:45  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/13 17:50:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import nbi.xml.sax.SAXElement;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * This class wraps the BioCASE protocol operators LOP.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class LOP extends Operator {
    static Logger logger = Logger.getLogger(LOP.class);

    /** The LOP attribute syntax */
    protected String syntax;

    /** The LOP list of operators */
    protected List operators;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public LOP(String protocolURI, String type) {
        super(protocolURI, type);
        this.type = type;
    }

    /**
     * Writes the LOP to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        AttributesImpl atts = SAXElement.EMPTY_ATTS;
        if (getSyntax() != null)
            atts.addAttribute(getProtocolURI(), getType(), getType(),
                    "xsd:string", getSyntax());
        out.startElement(getProtocolURI(), getType(), getType(), atts);
        if (operators != null) {
            Iterator opIt = operators.iterator();
            while (opIt.hasNext()) {
                ((Operator) opIt.next()).write(out);
            }
        }
        out.endElement(getProtocolURI(), getType(), getType());
    }

    /**
     * Sets the syntax
     * 
     * @param syntax
     *            The syntax to set.
     */
    public void setSyntax(String syntax) {
        this.syntax = syntax;
    }

    /**
     * Gets the path
     * 
     * @return Returns the path.
     */
    public String getSyntax() {
        return syntax;
    }

    /**
     * Gets the values
     * 
     * @return Returns the values.
     */
    public List getOperators() {
        return operators;
    }

    /**
     * Gets the first operator (for unary mode).
     * 
     * @return Returns the first operator.
     */
    public Operator getOperator() {
        return getOperator(0);
    }

    /**
     * Gets the operator with the given index in the operator list.
     * 
     * @return Returns the operator with the given index.
     */
    public Operator getOperator(int index) {
        return operators == null ? null : (Operator) operators.get(index);
    }

    /**
     * Sets the operators
     * 
     * @param operators
     *            The operators to set.
     */
    public void setOperators(List operators) {
        this.operators = operators;
    }

    /**
     * Sets the operator with the given index in the operator list.
     * 
     * @param index index to set the operator
     * @param operator the operator with the given index.
     */
    public void setOperator(int index, Operator operator) {
        if ((index > 0) && (index < 2)) {
            if (operators == null)
                operators = new ArrayList(2);
            if (operators.size() < 2)
                operators.add(index, operator);
            else
                operators.set(index, operator);
        }
    }

    /**
     * Appends the operator to the operator list.
     * 
     * @param operator the operator to append to the list.
     */
    public void addOperator(Operator operator) {
        if (operators == null)
            operators = new ArrayList();
        operators.add(operator);
    }

    /**
     * Removes an operator from the operator list.
     * 
     * @param operator the operator to remove from the list.
     */
    public void removeOperator(Operator operator) {
        operators.remove(operator);
    }

    /**
     * Removes the operator at the specified index from the operator list.
     * 
     * @param index index of the operator to be removed from the list.
     */
    public void removeOperator(int index) {
        if ((index > 0) && (index < 2)) {
            operators.remove(index);
        }
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        if (syntax != null)
            ret.append("syntax=" + syntax + ",");
        return ret.toString();
    }
}
