/*
 * $Id: Response.java,v 1.5 2005/08/15 21:58:55 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Response.java,v $
 * Revision 1.5  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.4  2005/07/23 10:58:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/07/15 19:55:30  suhrbier
 * Javadoc added/updated
 *
 * Revision 1.2  2005/07/05 18:24:03  suhrbier
 * Einbau Response-Filter (unvollstndig)
 *
 * Revision 1.1  2005/03/07 18:22:29  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/01/24 20:55:36  suhrbier
 * Response-Scannen komplett.
 *
 * Revision 1.3  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/17 17:37:23  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:56  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.ResponseFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * This class wraps the BioCASE protocol element Response.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.5 $
 * @since
 */
public class Response extends ProtocolType {
    static Logger logger = Logger.getLogger(Response.class);

    /** Response Header */
    Header header;

    /** Response Content */
    Content content;

    /** Response Diagnostics */
    Diagnostics diagnostics;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public Response(String protocolURI) {
        super(protocolURI);
    }
    
    /**
     * Writes the response to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException
    {
    	AttributesImpl atts = SAXElement.EMPTY_ATTS;
    	out.startElement(getProtocolURI(), ResponseFilter.LN_ROOT,ResponseFilter.LN_ROOT, atts);
    	if(header!=null)
    	    header.write(out);
    	if(content!=null)
    	    content.write(out);
    	if(diagnostics!=null)
    	    diagnostics.write(out);
    	out.endElement(getProtocolURI(), ResponseFilter.LN_ROOT,ResponseFilter.LN_ROOT);
    }
    
    public String toString() {
        StringBuffer ret = new StringBuffer();
        if (header != null)
            ret.append("header=" + header.toString() + "\n");
        if (content != null)
            ret.append("content=" + content.toString() + "\n");
        if (diagnostics != null)
            ret.append("diagnostics=" + diagnostics.toString() + "\n");
        return ret.toString();
    }

    /**
     * Gets the content
     * @return Returns the content.
     */
    public Content getContent() {
        return content;
    }
    /**
     * Sets content
     * @param content The content to set.
     */
    public void setContent(Content content) {
        this.content = content;
    }
    /**
     * Gets the diagnostics
     * @return Returns the diagnostics.
     */
    public Diagnostics getDiagnostics() {
        return diagnostics;
    }
    /**
     * Sets diagnostics
     * @param diagnostics The diagnostics to set.
     */
    public void setDiagnostics(Diagnostics diagnostics) {
        this.diagnostics = diagnostics;
    }
    /**
     * Gets the header
     * @return Returns the header.
     */
    public Header getHeader() {
        return header;
    }
    /**
     * Sets header
     * @param header The header to set.
     */
    public void setHeader(Header header) {
        this.header = header;
    }
}
