/*
 * $Id: Scan.java,v 1.3 2005/08/15 21:58:55 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Scan.java,v $
 * Revision 1.3  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.2  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/03/07 18:22:29  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/20 11:21:26  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/19 17:17:24  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.Iterator;
import java.util.Vector;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.ContentFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol content object of type Scan.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */

public class Scan extends ContentObject {
    static Logger logger = Logger.getLogger(Scan.class);

    /** Local name of the sub element <code>value</code> */
    public static final String LN_VALUE = "value";

    /** String Values of the content object Scan */
    Vector values;

    public Scan(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes the scan content object to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        out.startElement(getProtocolURI(), ContentFilter.LN_SCAN,
                ContentFilter.LN_SCAN, SAXElement.EMPTY_ATTS);
        SAXElement element = new SAXElement();
        element.setURI(getProtocolURI());
        element.setLocalQName(LN_VALUE);
        Iterator valuesIt = values.iterator();
        while (valuesIt.hasNext()) {
            element.setValue((String) valuesIt.next());
            element.write(out);
        }
        out.endElement(getProtocolURI(), ContentFilter.LN_SCAN,
                ContentFilter.LN_SCAN);
    }

    /**
     * Gets the value at the specified position
     * 
     * @param index
     *            index of the value to return
     * @return value at the specified index, or null.
     */
    public String getValue(int index) {
        String ret = null;
        if (values != null)
            ret = (String) values.get(index);
        return ret;
    }

    /**
     * Appends the specified value to the values list. If no values exists, a
     * new Vector will be created.
     * 
     * @param value
     *            value to be appended.
     * @return true
     */
    public boolean addValue(String value) {
        if (values == null)
            values = new Vector();
        return values.add(value);
    }

    /**
     * Removes the specified value from the values list.
     * 
     * @param value
     *            value to be removed.
     * @return true if the the values list contained specified value.
     */
    public boolean removeValue(String value) {
        boolean ret = false;
        if (values != null)
            ret = values.remove(value);
        return ret;
    }

    /**
     * Gets the values
     * 
     * @return Returns the values.
     */
    public Vector getValues() {
        return values;
    }

    /**
     * Sets values
     * 
     * @param values
     *            The values to set.
     */
    public void setValues(Vector values) {
        this.values = values;
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("values=");
        if (values != null)
            if (values.isEmpty())
                ret.append("empty");
            else {
                ret.append("(\n");
                Iterator valuesIt = values.iterator();
                for (int i = 1; valuesIt.hasNext(); i++) {
                    ret.append(i + ":" + (String) valuesIt.next());
                    if (valuesIt.hasNext())
                        ret.append(",\n");
                }
            }
        ret.append(")");
        return ret.toString();
    }
}
