/*
 * $Id: ScanOperationType.java,v 1.7 2006/07/18 15:11:54 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ScanOperationType.java,v $
 * Revision 1.7  2006/07/18 15:11:54  suhrbier
 * Fehler behoben in ContentObjectFilter
 * + PEPClient send Request
 * Fehler in der Request.write()-Kette behoben.
 *
 * Revision 1.6  2005/10/07 18:27:46  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.5  2005/08/31 18:09:51  suhrbier
 * Scan-Request um filter-element erweitert.
 *
 * Revision 1.4  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.3  2005/08/10 20:57:13  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/06 23:28:44  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/03/07 18:22:29  suhrbier
 * Initiale Version
 *
 * Revision 1.5  2005/02/11 17:48:07  suhrbier
 * Erste Verbindung BioCASE->PDP
 *
 * Revision 1.4  2005/01/17 15:37:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/14 18:13:41  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/14 17:05:09  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/13 17:50:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:56  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.OperationTypeFilter;
import nbi.xmlsec.biocase.protocol.filter.OperatorFilter;
import nbi.xmlsec.biocase.protocol.filter.ScanOperationTypeFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol treeElements scan type.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */
public class ScanOperationType extends OperationType {
    static Logger logger = Logger.getLogger(ScanOperationType.class);

    /** Concept of the scan operation */
    String concept;

    /**
     * The filter operator of this search operation
     */
    Operator filter = null;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public ScanOperationType(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes the Operation type to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        out.startElement(getProtocolURI(), ScanOperationTypeFilter.LN_ROOT,
                ScanOperationTypeFilter.LN_ROOT, SAXElement.EMPTY_ATTS);
        SAXElement element = new SAXElement();
        element.setURI(getProtocolURI());
        element.setLocalQName(OperationTypeFilter.LN_REQUESTFORMAT);
        element.setValue(getRequestFormat());
        element.write(out);
        element.setLocalQName(ScanOperationTypeFilter.LN_CONCEPT);
        element.setValue(getConcept());
        element.write(out);
        if (filter != null) {
            out.startElement(getProtocolURI(), OperatorFilter.LN_ROOT,
                    OperatorFilter.LN_ROOT, SAXElement.EMPTY_ATTS);
            filter.write(out);
            out.endElement(getProtocolURI(), OperatorFilter.LN_ROOT,
                    OperatorFilter.LN_ROOT);
        }

        out.endElement(getProtocolURI(), ScanOperationTypeFilter.LN_ROOT,
                ScanOperationTypeFilter.LN_ROOT);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        if (concept != null)
            ret.append("concept=" + concept);
        ret.append(")");
        return ret.toString();
    }

    /**
     * Gets the concept.
     * 
     * @return The concept.
     */
    public String getConcept() {
        return concept;
    }

    /**
     * Sets the concept.
     * 
     * @param concept
     *            The concept to set.
     */
    public void setConcept(String concept) {
        this.concept = concept;
    }

    /**
     * Sets filter
     * 
     * @param filter
     *            The filter to set.
     */
    public void setFilter(Operator filter) {
        this.filter = filter;
    }

    /**
     * Gets the filter
     * 
     * @return Returns the filter.
     */
    public Operator getFilter() {
        return filter;
    }
}
