/*
 * $Id: SearchOperationType.java,v 1.8 2006/07/18 14:58:11 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: SearchOperationType.java,v $
 * Revision 1.8  2006/07/18 14:58:11  suhrbier
 * Fehler behoben in ContentObjectFilter
 * + PEPClient send Request
 * Fehler in der Request.write()-Kette behoben.
 *
 * Revision 1.7  2005/10/07 18:27:46  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.6  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.5  2005/08/10 20:57:13  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.4  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.3  2005/08/07 16:15:31  suhrbier
 * Umstellung Request auf Filter
 *
 * Revision 1.2  2005/08/06 23:28:44  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/03/07 18:22:30  suhrbier
 * Initiale Version
 *
 * Revision 1.7  2005/03/01 12:43:55  suhrbier
 * Nach Urlaub
 *
 * Revision 1.6  2005/01/14 20:58:45  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/01/14 18:14:37  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/01/14 18:13:41  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/14 17:05:09  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/13 17:50:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.OperationTypeFilter;
import nbi.xmlsec.biocase.protocol.filter.OperatorFilter;
import nbi.xmlsec.biocase.protocol.filter.SearchOperationTypeFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class wraps the BioCASE protocol treeElements search operation type.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.8 $
 * @since
 */

public class SearchOperationType extends OperationType {
    static Logger logger = Logger.getLogger(SearchOperationType.class);

    /** Default value for the start attribute of the response format */
    private static final Integer DEFAULT_RESPONSEFORMAT_START = new Integer(0);

    /** Default value for the limit attribute of the response format */
    private static final Integer DEFAULT_RESPONSEFORMAT_LIMIT = new Integer(10);

    /** Default value of the count */
    private static final Boolean DEFAULT_COUNT = Boolean.FALSE;

    /** The response format */
    String responseFormat = null;

    /** The attribute start of the response format */
    Integer responseFormatStart = DEFAULT_RESPONSEFORMAT_START;

    /** The attribute limit of the response format */
    Integer responseFormatLimit = DEFAULT_RESPONSEFORMAT_LIMIT;

    /** The attribute transformation of the response format */
    String responseFormatTransformation = null;

    /** The filter operator of this search operation */
    Operator filter = null;

    /** The count */
    Boolean count = DEFAULT_COUNT;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public SearchOperationType(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes the Operation type to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        out.startElement(getProtocolURI(), SearchOperationTypeFilter.LN_ROOT,
                SearchOperationTypeFilter.LN_ROOT, SAXElement.EMPTY_ATTS);
        SAXElement element = new SAXElement();
        element.setURI(getProtocolURI());
        if (getRequestFormat() != null) {
            element.setLocalQName(OperationTypeFilter.LN_REQUESTFORMAT);
            element.setValue(getRequestFormat());
            element.write(out);
        }
        if (getResponseFormat() != null) {
            element.setLocalQName(SearchOperationTypeFilter.LN_RESPONSEFORMAT);

            element.addAttribute(getProtocolURI(),
                    SearchOperationTypeFilter.LN_RESPONSEFORMAT_LIMIT,
                    SearchOperationTypeFilter.LN_RESPONSEFORMAT_LIMIT,
                    getResponseFormatLimit());
            element.addAttribute(getProtocolURI(),
                    SearchOperationTypeFilter.LN_RESPONSEFORMAT_START,
                    SearchOperationTypeFilter.LN_RESPONSEFORMAT_START,
                    getResponseFormatStart());
            if (getResponseFormatTransformation() != null)
                element
                        .addAttribute(
                                getProtocolURI(),
                                SearchOperationTypeFilter.LN_RESPONSEFORMAT_TRANSFORMATION,
                                SearchOperationTypeFilter.LN_RESPONSEFORMAT_TRANSFORMATION,
                                getResponseFormatTransformation());
            element.setValue(getResponseFormat());
            element.write(out);
        }
        if (getCount() != null) {
            element.clearAttributes();
            element.setLocalQName(SearchOperationTypeFilter.LN_COUNT);
            element.setValue(getCount().toString());
            element.write(out);
        }
        if (filter != null)
        {
            out.startElement(getProtocolURI(), OperatorFilter.LN_ROOT,
                    OperatorFilter.LN_ROOT, SAXElement.EMPTY_ATTS);
            filter.write(out);
            out.endElement(getProtocolURI(), OperatorFilter.LN_ROOT,
                    OperatorFilter.LN_ROOT);
        }
        out.endElement(getProtocolURI(), SearchOperationTypeFilter.LN_ROOT,
                SearchOperationTypeFilter.LN_ROOT);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        if (responseFormat != null)
            ret.append("responseFormat=" + responseFormat + ",");
        ret.append("responseFormatStart=" + responseFormatStart + ",");
        ret.append("responseFormatLimit=" + responseFormatLimit + ",");
        if (responseFormatTransformation != null)
            ret.append("responseFormatTransformation="
                    + responseFormatTransformation + ",");
        if (filter != null)
            ret.append("filter=" + filter.toString() + ",");
        ret.append("count=" + count);
        ret.append(")");
        return ret.toString();
    }

    /**
     * If the value is null, the defined default value (
     * {@link #DEFAULT_COUNT DEFAULT_COUNT}) is returned.
     * 
     * @return The value of count
     */
    public boolean isCount() {
        return count == null ? DEFAULT_COUNT.booleanValue() : count
                .booleanValue();
    }

    /**
     * Gets the limit attribute of the response format. If the value is null,
     * the defined default value (
     * {@link #DEFAULT_RESPONSEFORMAT_LIMIT DEFAULT_RESPONSEFORMAT_LIMIT}) is
     * returned.
     * 
     * @return The value of count
     */
    public Integer getResponseFormatLimit() {
        return responseFormatLimit == null ? DEFAULT_RESPONSEFORMAT_LIMIT
                : responseFormatLimit;
    }

    /**
     * Gets the start attribute of the response format. If the value is null,
     * the defined default value (
     * {@link #DEFAULT_RESPONSEFORMAT_START DEFAULT_RESPONSEFORMAT_START}) is
     * returned.
     * 
     * @return The value of count
     */
    public Integer getResponseFormatStart() {
        return responseFormatStart == null ? DEFAULT_RESPONSEFORMAT_START
                : responseFormatStart;

    }

    /**
     * Sets count. If the given parameter is null, the defined default value
     * (false) is set.
     * 
     * @param count
     *            The count to set.
     */
    public void setCount(Boolean count) {
        this.count = count == null ? Boolean.FALSE : count;
    }

    /**
     * Sets count. If the given parameter is null, the defined default value (
     * {@link #DEFAULT_RESPONSEFORMAT_LIMIT DEFAULT_RESPONSEFORMAT_LIMIT}) is
     * set.
     * 
     * @param count
     *            The count to set.
     */
    public void setCount(String count) {
        this.count = count == null ? Boolean.FALSE : Boolean.valueOf(count);
    }

    /**
     * Sets the limit attribute of the response format. If the given parameter
     * is null or non-negative, the defined default value (
     * {@link #DEFAULT_RESPONSEFORMAT_LIMIT DEFAULT_RESPONSEFORMAT_LIMIT}) is
     * set.
     * 
     * @param responseFormatLimit
     *            The responseFormatLimit to set.
     */
    public void setResponseFormatLimit(Integer responseFormatLimit) {
        this.responseFormatLimit = DEFAULT_RESPONSEFORMAT_LIMIT;
        if (responseFormatLimit != null)
            if (responseFormatLimit.intValue() >= 0)
                this.responseFormatLimit = responseFormatLimit;
    }

    /**
     * Sets the start attribute of the response format. If the given parameter
     * is null or non-negative, the defined default value (
     * {@link #DEFAULT_RESPONSEFORMAT_START DEFAULT_RESPONSEFORMAT_START}) is
     * set.
     * 
     * @param responseFormatStart
     *            The responseFormatStart to set.
     */
    public void setResponseFormatStart(Integer responseFormatStart) {
        this.responseFormatStart = DEFAULT_RESPONSEFORMAT_START;
        if (responseFormatStart != null)
            if (responseFormatStart.intValue() >= 0)
                this.responseFormatStart = responseFormatStart;
    }

    /**
     * Gets the filter
     * 
     * @return Returns the filter.
     */
    public Operator getFilter() {
        return filter;
    }

    /**
     * Sets filter
     * 
     * @param filter
     *            The filter to set.
     */
    public void setFilter(Operator filter) {
        this.filter = filter;
    }

    /**
     * Gets the responseFormat
     * 
     * @return Returns the responseFormat.
     */
    public String getResponseFormat() {
        return responseFormat;
    }

    /**
     * Sets responseFormat
     * 
     * @param responseFormat
     *            The responseFormat to set.
     */
    public void setResponseFormat(String responseFormat) {
        this.responseFormat = responseFormat;
    }

    /**
     * Gets the responseFormatTransformation
     * 
     * @return Returns the responseFormatTransformation.
     */
    public String getResponseFormatTransformation() {
        return responseFormatTransformation;
    }

    /**
     * Sets responseFormatTransformation
     * 
     * @param responseFormatTransformation
     *            The responseFormatTransformation to set.
     */
    public void setResponseFormatTransformation(
            String responseFormatTransformation) {
        this.responseFormatTransformation = responseFormatTransformation;
    }

    /**
     * Gets the count
     * 
     * @return Returns the count.
     */
    public Boolean getCount() {
        return count;
    }
}
