/*
 * $Id: SupportedSchema.java,v 1.2 2005/08/15 21:58:55 suhrbier Exp $
 * Created on 12.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: SupportedSchema.java,v $
 * Revision 1.2  2005/08/15 21:58:55  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.1  2005/03/07 18:22:30  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/01/20 14:37:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/01/20 11:21:26  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/19 17:17:24  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/12 11:13:57  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol;

import java.util.Iterator;
import java.util.Vector;

import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.filter.ScanOperationTypeFilter;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * This class wraps the SupportedSchma subelement of the BioCASE protocol
 * content object of type Capabilities.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */
public class SupportedSchema extends ProtocolType {
    static Logger logger = Logger.getLogger(SupportedSchema.class);

    /** Local name of the root element <code>SupportedSchemas</code> */
    public static final String LN_ROOT = "SupportedSchema";

    /**
     * Local name of the attribute <code>namespace</code> of the root element
     * <code>SupportedSchemas</code>
     */
    public static final String LN_SUPPORTEDSCHEMA_NAMESPACE = "namespace";

    /**
     * Local name of the attribute <code>request</code> of the root element
     * <code>SupportedSchemas</code>
     */
    public static final String LN_SUPPORTEDSCHEMA_REQUEST = "request";

    /**
     * Local name of the attribute <code>responses</code> of the root element
     * <code>SupportedSchemas</code>
     */
    public static final String LN_SUPPORTEDSCHEMA_RESPONSE = "response";

    /** Vector of concepts */
    private Vector concepts;

    /** The attribute <code>namespace</code> of this element */
    private String namespace;

    /** The attribute <code>request</code> of this element */
    private Boolean request;

    /** The attribute <code>response</code> of this element */
    private Boolean response;

    /**
     * Inits this instance.
     * 
     * @param protocolURI
     *            The protocol version of this instance.
     */
    public SupportedSchema(String protocolURI) {
        super(protocolURI);
    }

    /**
     * Writes the supported schema to a SAX Stream.
     * 
     * @param out
     *            {@link ContentHandler ContentHandler}to write to.
     * @throws SAXException
     */
    public void write(ContentHandler out) throws SAXException {
        AttributesImpl atts = SAXElement.EMPTY_ATTS;
        if (namespace != null)
            atts.addAttribute(getProtocolURI(), LN_SUPPORTEDSCHEMA_NAMESPACE,
                    LN_SUPPORTEDSCHEMA_NAMESPACE, "xsd:anyURI", namespace);
        if (request != null)
            atts.addAttribute(getProtocolURI(), LN_SUPPORTEDSCHEMA_REQUEST,
                    LN_SUPPORTEDSCHEMA_REQUEST, "xsd:boolean", request
                            .toString());
        if (response != null)
            atts.addAttribute(getProtocolURI(), LN_SUPPORTEDSCHEMA_RESPONSE,
                    LN_SUPPORTEDSCHEMA_RESPONSE, "xsd:boolean", response
                            .toString());
        out.startElement(getProtocolURI(), LN_ROOT, LN_ROOT, atts);
        SAXElement element = new SAXElement();
        element.setURI(getProtocolURI());
        element.setLocalQName(ScanOperationTypeFilter.LN_CONCEPT);
        Iterator conceptsIt = concepts.iterator();
        while (conceptsIt.hasNext()) {
            element.setValue((String) conceptsIt.next());
            element.write(out);
        }
        out.endElement(getProtocolURI(), LN_ROOT, LN_ROOT);
    }

    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("(");
        if (namespace != null)
            ret.append("nameSpace=" + namespace + ",");
        ret.append("request=" + request + ",");
        ret.append("response=" + response + ",");
        ret.append("concepts=");
        if (concepts != null)
            if (concepts.isEmpty())
                ret.append("empty");
            else {
                ret.append("(\n");
                Iterator conceptsIt = concepts.iterator();
                for (int i = 1; conceptsIt.hasNext(); i++) {
                    ret.append(i + ":" + (String) conceptsIt.next());
                    if (conceptsIt.hasNext())
                        ret.append(",\n");
                }
            }
        ret.append(")");
        return ret.toString();
    }

    /**
     * Gets the concept at the specified position
     * 
     * @param index
     *            index of the concept to return
     * @return concept at the specified index, or null.
     */
    public String getConcept(int index) {
        String ret = null;
        if (concepts != null)
            ret = (String) concepts.get(index);
        return ret;
    }

    /**
     * Appends the specified concept to the concept vector. If no concepts
     * exists, a new Vector will be created.
     * 
     * @param concept
     *            concept to be appended.
     * @return true
     */
    public boolean addConcept(String concept) {
        if (concepts == null)
            concepts = new Vector();
        return concepts.add(concept);
    }

    /**
     * Removes the specified concept from the concept vector.
     * 
     * @param concept
     *            concept to be removed.
     * @return true if the concept vector contained the specified concept.
     */
    public boolean removeConcept(String concept) {
        boolean ret = false;
        if (concepts != null)
            ret = concepts.remove(concept);
        return ret;
    }

    /**
     * Gets the concepts
     * 
     * @return Returns the concepts.
     */
    public Vector getConcepts() {
        return concepts;
    }

    /**
     * Sets concepts
     * 
     * @param concepts
     *            The concepts to set.
     */
    public void setConcepts(Vector concepts) {
        this.concepts = concepts;
    }

    /**
     * Gets the namespace
     * 
     * @return Returns the namespace.
     */
    public String getNamespace() {
        return namespace;
    }

    /**
     * Sets namespace
     * 
     * @param namespace
     *            The namespace to set.
     */
    public void setNamespace(String namespace) {
        this.namespace = namespace;
    }

    /**
     * Gets the request
     * 
     * @return Returns the request.
     */
    public Boolean getRequest() {
        return request;
    }

    /**
     * Sets request
     * 
     * @param request
     *            The request to set.
     */
    public void setRequest(Boolean request) {
        this.request = request;
    }

    /**
     * Gets the response
     * 
     * @return Returns the response.
     */
    public Boolean getResponse() {
        return response;
    }

    /**
     * Sets response
     * 
     * @param response
     *            The response to set.
     */
    public void setResponse(Boolean response) {
        this.response = response;
    }
}
