/*
 * $Id: COPFilter.java,v 1.6 2006/07/14 21:17:57 suhrbier Exp $
 * Created on 07.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: COPFilter.java,v $
 * Revision 1.6  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.5  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.4  2005/08/15 13:24:45  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.1  2005/08/07 16:15:32  suhrbier
 * Umstellung Request auf Filter
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.COP;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * The class COPFilter parses the content of a COP operator of BioCASE search
 * request from a SAX stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * {@link COP COP}may be retrieved with {@link #getOperator() getOperator()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class COPFilter extends ProtocolFilter {

	/**
	 * Local name of the attribute <code>path</code> of any COP element
	 */
	public static String LN_COP_PATH = "path";

	// Unary COP
	/** Local name of the Unary COP <code>isNull</code> */
	public static String LN_ISNULL = "isNull";

	/** Local name of the Unary COP <code>isNotNull</code> */
	public static String LN_ISNOTNULL = "isNotNull";

	// Simple COP
	/** Local name of the Simple COP <code>equals</code> */
	public static String LN_EQUALS = "equals";

	/** Local name of the Simple COP <code>greaterThan</code> */
	public static String LN_GREATERTHAN = "greaterThan";

	/** Local name of the Simple COP <code>greaterThanEquals</code> */
	public static String LN_GREATERTHANOREQUALS = "greaterThanOrEquals";

	/** Local name of the Simple COP <code>lessThan</code> */
	public static String LN_LESSTHAN = "lessThan";

	/** Local name of the Simple COP <code>lessThanEquals</code> */
	public static String LN_LESSTHANOREQUALS = "lessThanOrEquals";

	/** Local name of the Simple COP <code>like</code> */
	public static String LN_LIKE = "like";

	/** Local name of the Simple COP <code>notEquals</code> */
	public static String LN_NOTEQUALS = "notEquals";

	// Multi COP
	/** Local name of the Multi COP <code>in</code> */
	public static String LN_IN = "in";

	/**
	 * Local name of the element <code>value</code> fo the Multi COP
	 * <code>in</code>
	 */
	public static String LN_VALUE = "value";

	/**
	 * Track mode for the filter stack
	 */
	static final int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
			| FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME
			| FilterStack.TRACK_MODE_CHARACTERS;

	/** Stores the parsed COP */
	protected COP cop;

	/**
	 * Inits the COP filter.
	 * 
	 * @param protocolURI
	 *            URI of the protocol version.
	 * @param filterRoot
	 *            Local name of the root element.
	 * @param trackMode
	 *            Define track mode for the internal
	 *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public COPFilter(String protocolURI, String filterRoot, int trackMode, int filterMode) {
		super(protocolURI, filterRoot, trackMode, filterMode);
		initSubFilter();
	}

	/**
	 * Inits the COP filter with the specified parent.
	 * 
	 * @param parent
	 *            Parent reader.
	 * @param protocolURI
	 *            URI of the protocol version.
	 * @param filterRoot
	 *            Local name of the root element.
	 * @param trackMode
	 *            Define track mode for the internal
	 *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public COPFilter(XMLReader parent, String protocolURI, String filterRoot,
			int trackMode, int filterMode) {
		super(parent, protocolURI, filterRoot, trackMode, filterMode);
		initSubFilter();
	}

	/** Initialises subfilters of this filter */
	private void initSubFilter() {
		registerSubFilter(LN_VALUE, null);
	}

	/**
	 * Initialises the COP setting its parsed type and path(Unary).
	 * 
	 * @see ProtocolFilter#parseInit(SAXElement)
	 */
	protected SAXElement parseInit(SAXElement element)
			throws SAXException {
		String operator = element.getLocalName();
		// Init the COP with type and path
		cop = new COP(getProtocolURI(), operator);
		cop.setPath(element.getAttributeValue(LN_COP_PATH));
		return element;
	}

	/**
	 * Adds a parsed value subelement to the cop. Is only called when a
	 * registered <code>value</code> element is finished.
	 * 
	 * @see ProtocolFilter#parseInit(SAXElement)
	 */
	protected SAXElement parseElement(SAXElement element)
			throws SAXException {
		if (isMulti(filterRoot))
			if (LN_VALUE.equals(element.getLocalName())) {
				cop.addValue(element.getValueAsString());
			}
		return element;
	}

	/**
	 * If parsing complete, keeps Unary unchanged, add the value of the root
	 * element for a simple operator.
	 * 
	 * @see ProtocolFilter#parseComplete(SAXElement)
	 */
	protected SAXElement parseComplete(SAXElement element)
			throws SAXException {
		String operator = element.getLocalName();
		if (isSimple(operator))
			cop.addValue(element.getValueAsString());
		return element;
	}

	/**
	 * Checks if the given operator is a UnaryCOP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is unary, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isUnary(String operator) {
		boolean ret = false;
		if (LN_ISNOTNULL.equals(operator) || LN_ISNULL.equals(operator))
			ret = true;
		return ret;
	}

	/**
	 * Checks if the given operator is a SimpleCOP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is simple, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isSimple(String operator) {
		boolean ret = false;
		if (LN_EQUALS.equals(operator) || LN_GREATERTHAN.equals(operator)
				|| LN_GREATERTHANOREQUALS.equals(operator)
				|| LN_LESSTHAN.equals(operator)
				|| LN_LESSTHANOREQUALS.equals(operator)
				|| LN_LIKE.equals(operator) || LN_NOTEQUALS.equals(operator))
			ret = true;
		return ret;
	}

	/**
	 * Checks if the given operator is a MultiCOP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is multi, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isMulti(String operator) {
		boolean ret = false;
		if (LN_IN.equals(operator))
			ret = true;
		return ret;
	}

	/**
	 * Checks if the given operator is a COP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is COP, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isOperator(String operator) {
		return isUnary(operator) || isSimple(operator) || isMulti(operator);
	}

	/**
	 * Gets the cop
	 * 
	 * @return Returns the cop.
	 */
	public COP getOperator() {
		return cop;
	}
}
