/*
 * $Id: ContentFilter.java,v 1.19 2006/07/25 14:34:48 suhrbier Exp $
 * Created on 22.07.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ContentFilter.java,v $
 * Revision 1.19  2006/07/25 14:34:48  suhrbier
 * signed Elements with namespace-URI working.
 *
 * Revision 1.18  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.17  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.16  2006/07/17 10:56:04  suhrbier
 * Fehler korrigiert.
 *
 * Revision 1.15  2006/07/17 10:24:18  suhrbier
 * Parsen von ContentObject ohne Filter als byte[].
 * Document neuer Typ von ContentObject.
 *
 * Revision 1.14  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.13  2006/05/08 17:21:57  suhrbier
 * Transform Response into DOM Document.
 *
 * Revision 1.12  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.11  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.10  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.9  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.8  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.7  2005/08/07 16:15:32  suhrbier
 * Umstellung Request auf Filter
 *
 * Revision 1.6  2005/08/05 14:28:03  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.5  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 * Revision 1.4  2005/07/25 18:59:00  suhrbier
 * Updated JavaDoc
 *
 * Revision 1.3  2005/07/25 15:30:07  suhrbier
 * HttpInputStream updated.
 *
 * Revision 1.2  2005/07/23 10:58:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/07/22 17:28:06  suhrbier
 * ResponseFilter implementiert, aber ungetestet.
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.util.Set;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.Content;
import nbi.xmlsec.biocase.protocol.Document;
import nbi.xmlsec.biocase.protocol.SearchOperationType;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * The class ContentFilter parses the content of a BioCASE response from a SAX
 * stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler or to the
 * {@link RBACFilter RBACFilter}subfilter configured during the filter
 * initialisation in {@link #parseInit(SAXElement) parseInit(SAXElement)}
 * respectively. The {@link RBACFilter RBACFilter}provides filtering of the
 * incorporated content document and gets control over the SAX filtering
 * process, until the document processing has finished. The parsed
 * {@link Content Content}may be retrieved with
 * {@link #getContent() getContent()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.19 $
 * @since
 */

public class ContentFilter extends ProtocolFilter {
    static Logger logger = Logger.getLogger(ContentFilter.class);

    /** Local name of the root element <code>content</code> */
    public static final String LN_ROOT = "content";

    /** Local name of the root element attribute <code>totalSearchHits</code> */
    public static final String LN_CONTENT_TOTALSEARCHHITS = "totalSearchHits";

    /** Local name of the root element attribute <code>recordDropped</code> */
    public static final String LN_CONTENT_RECORDDROPPED = "recordDropped";

    /** Local name of the root element attribute <code>recordCount</code> */
    public static final String LN_CONTENT_RECORDCOUNT = "recordCount";

    /** Local name of the root element attribute <code>recordStart</code> */
    public static final String LN_CONTENT_RECORDSTART = "recordStart";

    /** Local name of the subelement <code>capabilities</code> */
    public static final String LN_CAPABILITIES = "capabilities";

    /** Local name of the subelement <code>scan</code> */
    public static final String LN_SCAN = "scan";

    /** Local name of the subelement <code>search</code> */
    public static final String LN_SEARCH = "search";

    /** The currently configured content filter */
//    RBACFilter contentFilter = null;
    ProtocolFilter contentFilter = null;

    /** Track mode for the filter stack */
    final static int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME;

    /** To store the content attributes */
    private Content content = null;

    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ContentFilter(String protocolURI, int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
    }

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ContentFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(parent, protocolURI, LN_ROOT, trackMode, filterMode);
    }

    /**
     * Configures the content filter ((@link RBACFilter RBACFilter}) based on
     * the content filter parameters request, response header, RBACPDP and role
     * set.
     * <p>
     * The configuration must processed here, because the filter parameter
     * response header may be unknown when creating the content subfilter in the
     * parent filter ({@link ResponseFilter ResponseFilter}). In case of a
     * search response, the root element name of the (@link RBACFilter
     * RBACFilter} subfilter is set to null, which means unknown. Then, the
     * element of the next startElement event will define the root element of
     * the subfilter (see
     * {@link ProtocolFilter#startElement(String, String, String, Attributes) ProtocolFilter#startElement(String, String, String, Attributes)}).
     * The root element names for the response types <code>capabilities</code>
     * and <code>scan</code> are defined by the BioCASE protocol and were
     * deduced from the response header and must be set as filter parameter
     * before.
     * </p>
     * <p>When the filter mode is {@link ProtocolFilter#FILTERMODE_PARSER ProtocolFilter.FILTERMODE_PARSER} then a {@link ContentObjectFilter ContentObjectFilter} is configured, parsing the content document.
     * </p>
     * 
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element) throws SAXException {
        // checks if filterParameter set
        checkFilterParameter();
        // Set the attributes
        content = new Content(getProtocolURI());
        content.setRecordCount(element
                .getAttributeValueAsInteger(LN_CONTENT_RECORDCOUNT));
        content.setRecordDropped(element
                .getAttributeValueAsInteger(LN_CONTENT_RECORDDROPPED));
        content.setRecordStart(element
                .getAttributeValueAsInteger(LN_CONTENT_RECORDSTART));
        content.setTotalSearchHits(element
                .getAttributeValueAsInteger(LN_CONTENT_TOTALSEARCHHITS));

        // checks request type == response type
        if (getFilterMode() == ProtocolFilter.FILTERMODE_FILTER) {
            if (getParameter().getRequest().getHeader().getType().equals(
                    getParameter().getResponseHeader().getType())) {
                // build up parameter for the content filter
                RBACFilterParameter filterParam = new RBACFilterParameter(
                        getParameter().getResponseHeader().getType()
                                + "-response", getParameter().getPdp(),
                        getParameter().getRoles(), getParameter()
                                .getDiagnostics());
                // Remember root element of the content sub filter for
                // addSubFilter
                String filterRoot = null;
                // Configure content filter
                if (LN_SEARCH.equals(getParameter().getResponseHeader()
                        .getType())) {
                    // search type -> undefined root element
                    contentFilter = new RBACFilter(getParent(),
                            ((SearchOperationType) getParameter().getRequest()
                                    .getOpType()).getResponseFormat(),
                            filterRoot, getFilterMode());
                } else {
                    // capabilities or scan type -> root element == response
                    // type
                    contentFilter = new RBACFilter(getParent(),
                            getProtocolURI(), filterRoot = getParameter()
                                    .getResponseHeader().getType(),
                            getFilterMode());
                }
                // Set the content filter
                ((RBACFilter)contentFilter).setContentHandler(getContentHandler());
                // Set content filter parameter
                ((RBACFilter)contentFilter).setFilterParameter(filterParam);
                // register content filter as subfilter
                registerSubFilter(filterRoot, contentFilter);
            } else
                // request type != response type
                throw new SAXException("Invalid response type");
        }
        else
        {
            contentFilter = new ContentObjectFilter(getParent(), getProtocolURI(), getFilterMode());
            // Set the content filter
            ((ContentObjectFilter)contentFilter).setContentHandler(getContentHandler());
            // register content filter as subfilter for the next subelement of content
            registerSubFilter(null, contentFilter);
        }
        // pass the element unchanged
        return element;
    }

    /* (non-Javadoc)
     * @see nbi.xmlsec.biocase.protocol.filter.ProtocolFilter#parseComplete(nbi.xml.sax.SAXElement)
     */
    protected SAXElement parseComplete(SAXElement element) throws SAXException {
        if (getFilterMode() == ProtocolFilter.FILTERMODE_PARSER) {
            content.addContentObject( new Document(getProtocolURI(),((ContentObjectFilter)contentFilter).getContentObject()));
            
        }
        return super.parseComplete(element);
    }
    /**
     * Gets the parameter of this filter.
     * 
     * @return filter parameter.
     */
    protected ContentFilterParameter getParameter() {
        return (ContentFilterParameter) getFilterParameter();
    }

    /**
     * Gets the content
     * 
     * @return Returns the content.
     */
    public Content getContent() {
        return content;
    }

    /**
     * Gets the set of element paths evaluated for signing.
     * 
     * @return The set of element paths evaluated for signing.
     */
    public Set getSignElements() {
        return ((RBACFilter)contentFilter).getSignElements();
    }
    
    /**
     * Gets the namespace URI of the content.
     * The namespace URI is retrieved from the protocolURI of the internal content filter.
     * 
     * @return The namespace URI of the content.
     */
    public String getContentNamespaceURI()
    {
        return contentFilter.getProtocolURI();
    }
}
