/*
 * $Id: ContentObjectFilter.java,v 1.5 2006/07/18 14:58:12 suhrbier Exp $
 * Created on 14.07.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ContentObjectFilter.java,v $
 * Revision 1.5  2006/07/18 14:58:12  suhrbier
 * Fehler behoben in ContentObjectFilter
 * + PEPClient send Request
 * Fehler in der Request.write()-Kette behoben.
 *
 * Revision 1.4  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.3  2006/07/17 10:56:04  suhrbier
 * Fehler korrigiert.
 *
 * Revision 1.2  2006/07/17 10:24:18  suhrbier
 * Parsen von ContentObject ohne Filter als byte[].
 * Document neuer Typ von ContentObject.
 *
 * Revision 1.1  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xml.sax.XMLWriter;

import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * This filter simply reads in all incoming SaxElements and stores them in an
 * internal Buffer. The content of this internal Buffer may be retrieved using
 * {@link #getContentObject() getContentObject()}. The SaxElements are stored
 * using the {@link nbi.xml.sax.XMLWriter XMLWriter}class as content handler to
 * write the SaxElements to the internal Buffer.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.5 $
 * @since
 */

public class ContentObjectFilter extends ProtocolFilter {
    static Logger logger = Logger.getLogger(ContentObjectFilter.class);

    /** Track mode for the filter stack */
    final static int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME
            | FilterStack.TRACK_MODE_CHARACTERS;

    /** The buffer to write the content object to. */
    private ByteArrayOutputStream contentObjectOut = null;

    /** The content handler to write the content to the internal buffer. */
    private XMLWriter contentObjectWriter = null;
    
    /** Variable to store the original ContentHandler of this filter. */
    private ContentHandler origContentHandler = null;

    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ContentObjectFilter(String protocolURI, int filterMode) {
        super(protocolURI, null, trackMode, filterMode);
        initSubFilter();
        initBuffer();
    }

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ContentObjectFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(parent, protocolURI, null, trackMode, filterMode);
        initSubFilter();
        initBuffer();
    }

    /**
     * Registers subfilters for subelements.
     */
    private void initSubFilter() {
        registerSubFilter(null, null);
    }

    private void initBuffer() {
        contentObjectOut = new ByteArrayOutputStream();
        contentObjectWriter = new XMLWriter(contentObjectOut);
    }

    /**
     * Sets the protocolURI of the filter to the URI of the document to parse and activates the {@link ContentObjectFilter#contentObjectWriter contentObjectWriter} as ContenHandler to read the content object into the buffer.
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element) throws SAXException {
        setProtocolURI(element.getURI().toString());
        origContentHandler = getContentHandler();
        setContentHandler(contentObjectWriter);
        return element;
    }

    /**
     * Reactivates the  orignal ContenHandler deactivated to read the content object into the buffer in {@link #parseInit(SAXElement) parseInit(SAXElement)}.
     * @see ProtocolFilter#parseComplete(SAXElement)
     */
    protected SAXElement parseComplete(SAXElement element) throws SAXException {
        setContentHandler(origContentHandler);
        return element;
    }

    /**
     * Retrieves the content object from the internal buffer.
     * 
     * @return The content object from the internal buffer.
     */
    public byte[] getContentObject() {
        return contentObjectOut.toByteArray();
    }
}
