/*
 * $Id: DiagnosticsFilter.java,v 1.13 2006/07/14 21:17:57 suhrbier Exp $
 * Created on 21.07.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: DiagnosticsFilter.java,v $
 * Revision 1.13  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.12  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.11  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.10  2005/08/15 21:58:39  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.9  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.8  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.7  2005/08/05 14:28:02  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.6  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 * Revision 1.5  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 * Revision 1.4  2005/07/25 18:59:00  suhrbier
 * Updated JavaDoc
 *
 * Revision 1.3  2005/07/23 10:58:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/07/22 17:28:06  suhrbier
 * ResponseFilter implementiert, aber ungetestet.
 *
 * Revision 1.1  2005/07/21 19:00:32  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.util.Iterator;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.Diagnostic;
import nbi.xmlsec.biocase.protocol.Diagnostics;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * This class parses a BioCASE diagnostics from a SAX stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * {@link Diagnostics Diagnostics}may be retrieved with
 * {@link #getDiagnostics() getDiagnostics()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.13 $
 * @since
 */

public class DiagnosticsFilter extends ProtocolFilter {
    /** Local name of the root element <code>diagnostics</code> */
    public static String LN_ROOT = "diagnostics";

    /** Local name of the element <code>diagnostic</code> */
    public static final String LN_DIAGNOSTIC = "diagnostic";

    /**
     * Local name of the attribute <code>code</code> of the element
     * <code>diagnostic</code>
     */
    public static final String LN_DIAGNOSTIC_CODE = "code";

    /**
     * Local name of the attribute <code>severity</code> of the element
     * <code>diagnostic</code>
     */
    public static final String LN_DIAGNOSTIC_SEVERITY = "severity";

    /** Track mode for the filter stack */
    final static int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_CHARACTERS
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME;

    /** The parsed BioCASE diagnostics */
    Diagnostics diagnostics = null;

    /** Signals if the first diagnostic subelement has been written */
    private boolean firstElement = true;
    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public DiagnosticsFilter(String protocolURI, int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
        initSubFilter();
    }

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public DiagnosticsFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
        initSubFilter();
    }

    /**
     * Registers subfilters for subelements.
     */
    protected void initSubFilter() {
        registerSubFilter(LN_DIAGNOSTIC, null);
    }

    /**
     * Writes predefined Diagnostics to the content handler.
     * @see ProtocolFilter#filterElement(SAXElement)
     */
    protected SAXElement filterElement(SAXElement element) throws SAXException {
        if (LN_DIAGNOSTIC.equals(element.getLocalName())) {
            if( firstElement )
            {
                writeDiagnostics();
                firstElement=false;
            }
        }
        return element;
    }
    
    /**
     * Parses a Diagnostic treeElements and adds it to the Diagnostics.
     * 
     * @see ProtocolFilter#parseElement(SAXElement)
     */
    protected SAXElement parseElement(SAXElement element) throws SAXException {
        if (LN_DIAGNOSTIC.equals(element.getLocalName())) {
            Diagnostic diagnostic = new Diagnostic(getProtocolURI());
            diagnostic.setCode(element.getAttributeValue(LN_DIAGNOSTIC_CODE));
            diagnostic.setSeverity(element
                    .getAttributeValue(LN_DIAGNOSTIC_SEVERITY));
            diagnostic.setValue(element.getValue().toString());
            diagnostics.addDiagnostic(diagnostic);
        }
        return element;
    }

    /**
     * Initialises Diagnostics
     * 
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element) throws SAXException {
        if (diagnostics == null)
            diagnostics = new Diagnostics(getProtocolURI());
        return element;
    }

    /**
     * Gets the diagnostics
     * 
     * @return Returns the diagnostics.
     */
    public Diagnostics getDiagnostics() {
        return diagnostics;
    }

    /**
     * Adds diagnostics, and creates them when null.
     * 
     * @return Returns the diagnostics.
     */
    public Diagnostics addDiagnostics(Diagnostics diagnostics) {
        if (this.diagnostics == null)
            this.diagnostics = new Diagnostics(getProtocolURI());
        this.diagnostics.addDiagnostics(diagnostics);
        return this.diagnostics;
    }

    /**
     * Write diagnostics to the content handler.
     * 
     * @return Returns the diagnostics.
     */
    public void writeDiagnostics() throws SAXException{
        if ((this.diagnostics != null)&&(this.diagnostics.getDiagnostics()!=null)) {
            
            Iterator diagIt = this.diagnostics.getDiagnostics().iterator();
            while (diagIt.hasNext())
                ((Diagnostic) diagIt.next()).write(getContentHandler());
        }
    }
}
