/*
 * $Id: HeaderFilter.java,v 1.12 2006/07/14 13:12:07 suhrbier Exp $
 * Created on 21.07.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: HeaderFilter.java,v $
 * Revision 1.12  2006/07/14 13:12:07  suhrbier
 * FilterMode integrieren
 *
 * Revision 1.11  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.10  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.9  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.8  2005/08/05 14:28:03  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.7  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 * Revision 1.6  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 * Revision 1.5  2005/07/25 18:59:00  suhrbier
 * Updated JavaDoc
 *
 * Revision 1.4  2005/07/23 10:58:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/07/22 17:28:06  suhrbier
 * ResponseFilter implementiert, aber ungetestet.
 *
 * Revision 1.2  2005/07/21 21:18:38  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/07/21 19:00:32  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.Header;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * This class parses an BioCASE header from a SAX stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * {@link Header Header} may be retrieved with {@link #getHeader() getHeader()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.12 $
 * @since
 */

public class HeaderFilter extends ProtocolFilter {

    /** Local name of the root element <code>header</code> */
    public static String LN_ROOT = "header";

    /** Local name of the element <code>version</code> */
    public static final String LN_VERSION = "version";

    /**
     * Local name of the attribute <code>software</code> of the element
     * <code>version</code>
     */
    public static final String LN_VERSION_SOFTWARE = "software";

    /** Local name of the element <code>sendTime</code> */
    public static final String LN_SENDTIME = "sendTime";

    /** Local name of the element <code>source</code> */
    public static final String LN_SRC = "source";

    /**
     * Local name of the attribute <code>resource</code> of the element
     * <code>source</code>
     */
    public static final String LN_SRC_RESOURCE = "resource";

    /** Local name of the element <code>destination</code> */
    public static final String LN_DEST = "destination";

    /**
     * Local name of the attribute <code>resource</code> of the element
     * <code>destination</code>
     */
    public static final String LN_DEST_RESOURCE = "resource";

    /** Local name of the element <code>type</code> */
    public static final String LN_TYPE = "type";

    /** Track mode for the filter stack */
    final static int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_CHARACTERS
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME;

    /** The parsed BioCASE header */
    private Header header = null;

    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public HeaderFilter(String protocolURI,int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
        initSubFilter();
    }

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public HeaderFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(parent, protocolURI, LN_ROOT, trackMode, filterMode);
        initSubFilter();
    }

    /**
     * Registers subfilters for subelements.
     */
    protected void initSubFilter() {
        registerSubFilter(LN_VERSION, null);
        registerSubFilter(LN_SENDTIME, null);
        registerSubFilter(LN_SRC, null);
        registerSubFilter(LN_DEST, null);
        registerSubFilter(LN_TYPE, null);
    }

    /**
     * Parses Header subelements and build up the Header.
     * 
     * @see ProtocolFilter#parseElement(SAXElement)
     */
    protected SAXElement parseElement(SAXElement element)
            throws SAXException {
        if (LN_VERSION.equals(element.getLocalName()))
            header.addVersion(element.getValueAsString(), element
                    .getAttributeValue(LN_VERSION_SOFTWARE));
        else if (LN_SRC.equals(element.getLocalName()))
            header.addSource(element.getValueAsString(), element
                    .getAttributeValue(LN_SRC_RESOURCE));
        else if (LN_DEST.equals(element.getLocalName()))
            header.addDestination(element.getValueAsString(), element
                    .getAttributeValue(LN_DEST_RESOURCE));
        else if (LN_SENDTIME.equals(element.getLocalName()))
            header.setSendTime(element.getValueAsString());
        else if (LN_TYPE.equals(element.getLocalName()))
            header.setType(element.getValueAsString());
        return element;
    }

    /**
     * Initialises Header
     * 
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element)
            throws SAXException {
        header = new Header(getProtocolURI());
        return element;
    }

    /**
     * Gets the header.
     * 
     * @return Returns the header.
     */
    public Header getHeader() {
        return header;
    }
}
