/*
 * $Id: OperationTypeFilter.java,v 1.6 2006/07/14 21:17:57 suhrbier Exp $
 * Created on 06.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: OperationTypeFilter.java,v $
 * Revision 1.6  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.5  2005/08/15 21:58:39  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.4  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/06 23:28:44  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/08/06 22:08:28  suhrbier
 * Zwischenspeicherung
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.OperationType;
import nbi.xmlsec.biocase.protocol.ScanOperationType;
import nbi.xmlsec.biocase.protocol.SearchOperationType;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * The class OperationTypeFilter parses the request format element of an search
 * or scan operation type of a BioCASE search or scan request from a SAX stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * operation type may be retrieved with the method
 * {@link ScanOperationTypeFilter#getOperationType() getOperationType()}of the
 * classes {@link ScanOperationType ScanOperationType}or
 * {@link SearchOperationType SearchOperationType}respectively.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class OperationTypeFilter extends ProtocolFilter {

	/** For the parsed operation type */
	protected OperationType operationType;

	/** Local name of the element <code>requestFormat</code> */
	public static final String LN_REQUESTFORMAT = "requestFormat";

	/**
	 * Inits this filter with the specified parent.
	 * 
	 * @param parent
	 *            Parent reader.
	 * @param protocolURI
	 *            URI of the protocol version.
	 * @param filterRoot
	 *            Local name of the root element.
	 * @param trackMode
	 *            Define track mode for the internal
	 *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public OperationTypeFilter(XMLReader parent, String protocolURI,
			String filterRoot, int trackMode, int filterMode) {
		super(parent, protocolURI, filterRoot, trackMode, filterMode);
	}

	/**
	 * Inits this filter.
	 * 
	 * @param protocolURI
	 *            URI of the protocol version.
	 * @param filterRoot
	 *            Local name of the root element.
	 * @param trackMode
	 *            Define track mode for the internal
	 *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public OperationTypeFilter(String protocolURI, String filterRoot,
			int trackMode, int filterMode) {
		super(protocolURI, filterRoot, trackMode, filterMode);
	}

	/**
	 * Registers subfilters for subelements.
	 */
	protected void initSubFilter() {
		registerSubFilter(LN_REQUESTFORMAT, null);
	}

	/**
	 * Parses subelements and sets the request format of the operation type.
	 * 
	 * @see ProtocolFilter#parseElement(SAXElement)
	 */
	protected SAXElement parseElement(SAXElement element)
			throws SAXException {
		if (LN_REQUESTFORMAT.equals(element.getLocalName()))
			operationType.setRequestFormat(element.getValue().toString());
		return element;
	}
}
