/*
 * $Id: PDPFilterParameter.java,v 1.10 2006/07/12 09:42:41 suhrbier Exp $
 * Created on 04.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PDPFilterParameter.java,v $
 * Revision 1.10  2006/07/12 09:42:41  suhrbier
 * Fehler behoben in PEPHandler bzgl. getRoles/getCertificates wenn kein clientCert-Parameter vorliegt.
 *
 * Revision 1.9  2006/05/31 14:55:20  suhrbier
 * *** empty log message ***
 *
 * Revision 1.8  2005/12/14 15:34:31  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.7  2005/11/02 20:10:32  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.6  2005/10/15 21:37:40  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.4  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.3  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/10 11:59:38  suhrbier
 * All Filters tested.
 *
 * Revision 1.1  2005/08/06 16:16:14  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nbi.xmlsec.xacml.profile.rbac.Identifier;
import nbi.xmlsec.xacml.profile.rbac.RBACPDP;
import nbi.xmlsec.xacml.profile.rbac.RoleAssignmentPolicyIdentifier;

import com.sun.xacml.ParsingException;

/**
 * Container class to store Parameters for the {@link RBACPDP RBACPDP}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.10 $
 * @since
 */

public class PDPFilterParameter {

    /** The pdp with evaluation access rights from XACML-policies */
    private RBACPDP pdp;

    /** Set of roles to be evaluated from the pdp */
    private Set roles;

    /**
     * Inits the RBACFilterParameter. Arguments must not be null.
     * 
     * @param pdp
     *            The pdp with evaluation access rights from XACML-policies.
     * @param roles
     *            Set of roles to be evaluated from the pdp.
     */
    public PDPFilterParameter(RBACPDP pdp, Set roles) {
        super();
        this.pdp = pdp;
        this.roles = roles;
        this.checkInit();
    }

    /**
     * Checks the correct initialisation of the class initialisation parameters.
     */
    private void checkInit() {
        if (this.pdp == null || this.roles == null)
            throw new IllegalArgumentException("Arguments must not be null");
    }

    /**
     * Gets the pdp
     * 
     * @return Returns the pdp.
     */
    public RBACPDP getPdp() {
        return pdp;
    }

    /**
     * Gets the roles
     * 
     * @return Returns the roles.
     */
    public Set getRoles() {
        return roles;
    }

    public static String getRolesAsString(Set roles) {
        StringBuffer result = null;
        if (roles != null) {
            Iterator roleIt = roles.iterator();
            while (roleIt.hasNext()) {
                if (result == null)
                    result = new StringBuffer(((RoleAssignmentPolicyIdentifier) roleIt.next())
                            .toString());
                else
                    result.append(","
                            + ((RoleAssignmentPolicyIdentifier) roleIt.next()).toString());
            }
        }
        return result == null ? null : result.toString();
    }

    public static Set getRoles(String roles) throws ParsingException,
            URISyntaxException {
        HashSet result = new HashSet();
        if (roles != null) {
            int endRole = 0;
            int startRole = 0;
            while ((endRole = roles.indexOf(',', startRole)) != -1) {
                result.add(Identifier.getInstance(roles.substring(startRole,
                        endRole)));
                startRole = endRole + 1;
            }
            if ((endRole == -1) && (roles.length() > 0))
                result.add(Identifier.getInstance(roles));
        }
        if (result.isEmpty())
            result = null;
        return result;
    }

}
