/*
 * $Id: ProtocolFilter.java,v 1.21 2006/07/17 20:27:10 suhrbier Exp $
 * Created on 03.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ProtocolFilter.java,v $
 * Revision 1.21  2006/07/17 20:27:10  suhrbier
 * Fehler behoben
 * + validate Diagnostics
 * Fehler beim parsen des ContentObject Documents !
 *
 * Revision 1.20  2006/07/14 21:17:58  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.19  2006/07/14 13:12:07  suhrbier
 * FilterMode integrieren
 *
 * Revision 1.18  2005/11/02 20:10:32  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.17  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.16  2005/08/31 11:23:03  suhrbier
 * Fehler mit verschachtelten LOP-Filtern behoben.
 * HttpRequestMethod um URIParameter erweitert.
 *
 * Revision 1.15  2005/08/29 12:35:16  suhrbier
 * Neuer Provider fast lauffhig, XMLWriter schreibt jetzt ESC, Protocolfilter sendet Character Events nicht mehr aufgestaut weiter.
 *
 * Revision 1.14  2005/08/24 17:00:27  suhrbier
 * Anpassungen an neuen Provider, Versuche mit RolePolicySet Parameter
 *
 * Revision 1.13  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.12  2005/08/15 21:58:39  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.11  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.10  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.9  2005/08/10 12:24:51  suhrbier
 * Updated ProtocolFilter to push element on the stack before calling filterElement
 *
 * Revision 1.8  2005/08/10 11:59:37  suhrbier
 * All Filters tested.
 *
 * Revision 1.7  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.6  2005/08/06 16:16:14  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.5  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.4  2005/08/05 14:28:03  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.3  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 * Revision 1.2  2005/08/04 11:21:11  suhrbier
 * update protocolfilter
 *
 * Revision 1.1  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.util.HashMap;
import java.util.Map;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;

/**
 * Base class to parse and filter protocol treeElements of the BioCASE protocol.
 * <p>
 * The class must be initialised with the protocolURI, the root element.
 * Furthermore, subfilters must be registered to declare the accepted
 * subelements of the filter
 * {@link #registerSubFilter(String, ProtocolFilter) registerSubFilter()}/
 * {@link #unregisterSubFilter(String) unregisterSubFilter()}. Subfilters are
 * stored in a {@link HashMap HashMap}where the element's local name serves as
 * key for the registered subfilter. If the registered subfilter for a local
 * name is null, then all incoming treeElements will be handled within the current
 * filter. If a real subfilter is set for a local name, all events including the
 * startElement event are redirected to the defined subfilter. Therefore, the
 * subfilter should be configured, before it will be registered! Note, that for
 * those subfilters, the protocol version URI is not checked, because this
 * should be controlled individually by the subfilter!
 * {@link #setFilterParameter(Object) setFilterParameter(Object)}allows to set
 * special filter parameters. When the subfilter signals parse completion (
 * {@link #isParsingComplete() isParsingComplete()}) after the processing of
 * the corresponding endElement event, the subfilter will be unset. For all
 * incoming SAX events, the protocolURI and the validity of the incoming
 * treeElements according to the registered subfilters are checked. To support
 * filtering processes in subclasses, the following callback methods may be
 * overwritten:
 * <ul>
 * <li>{@link #parseInit(SAXElement) parseInit(SAXElement)}startElement event
 * for filter's root element received</li>
 * <li>{@link #filterElement(SAXElement) filterElement(SAXElement)}
 * startElement event for registred subfilter element received</li>
 * <li>{@link #filterCharacters(SAXElement) filterCharacters(SAXElement)}
 * characters event for registred subfilter element received</li>
 * <li>{@link #parseElement(SAXElement) parseElement(SAXElement)}endElement
 * event for registred subfilter element received</li>
 * <li>{@link #parseComplete(SAXElement) parseComplete(SAXElement)}endElement
 * event for filter's root element received</li>
 * </ul>
 * All methods (except parseElement when real subfilter has completed) are
 * offering the possibility to influence the further event processing, because
 * they are called before the corresponding event is forwarded to the content
 * handler or subfilter. Events may be removed from further processing if a
 * callback method returns null. The completion of the parsing process is
 * signaled by the method {@link #isParsingComplete() isParsingComplete()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.21 $
 * @since
 */

public class ProtocolFilter extends XMLFilterImpl {

    static Logger logger = Logger.getLogger(ProtocolFilter.class);

    /** ProtocolURI of the BioCASE protocol version 1.3 */
    public static final String PROT_URI_13 = "http://www.biocase.org/schemas/protocol/1.3";

    /** ProtocolURI of the BioCASE protocol version 2.0 */
    public static final String PROT_URI_20 = "http://www.biocase.org/schemas/protocol/2.0";

    /** Filter mode unspecified of this protocol filter */
    public static final int FILTERMODE_NONE = 0;

    /** Filter mode filter of this protocol filter */
    public static final int FILTERMODE_FILTER = 1;

    /** Filter mode parser of this protocol filter */
    public static final int FILTERMODE_PARSER = 2;

    /** Namespace URI of the BioCASE protocol version */
    private String protocolURI;

    /**
     * Stack storing all treeElements for this filter, if not redirected to
     * subfilters
     */
    protected FilterStack elementStack = null;

    /** The completion state of the parsing process */
    protected int filterMode;

    /** The completion state of the parsing process */
    private boolean parsingComplete = false;

    /** The local name of the element to be parsed */
    protected String filterRoot;

    /** Map of registered subfilters */
    private Map filterElements;

    /** Parameter object for the current filter */
    private Object filterParameter;

    /** Current active subfilter */
    private ProtocolFilter subFilter;
    
    private boolean filterInitialised = false;

    /**
     * Inits the protocol filter.
     * 
     * @param protocolURI
     *            URI of the protocol version.
     * @param filterRoot
     *            Local name of the root element.
     * @param trackMode
     *            Define track mode for the internal
     *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            Define filter mode for this filter {@link #FILTERMODE_FILTER FILTERMODE_FILTER(inkl. PDPFilter)} or {@link #FILTERMODE_PARSER FILTERMODE_PARSER(Parses all elements)}.
     */
    public ProtocolFilter(String protocolURI, String filterRoot, int trackMode, int filterMode) {
        super();
        init(protocolURI, filterRoot, trackMode, filterMode);
    }

    /**
     * Inits the protocol filter with the specified parent.
     * 
     * @param parent
     *            Parent reader.
     * @param protocolURI
     *            URI of the protocol version.
     * @param filterRoot
     *            Local name of the root element.
     * @param trackMode
     *            Define track mode for the internal
     *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            Define filter mode for this filter {@link #FILTERMODE_FILTER FILTERMODE_FILTER(inkl. PDPFilter)} or {@link #FILTERMODE_PARSER FILTERMODE_PARSER(Parses all elements)}.
     */
    public ProtocolFilter(XMLReader parent, String protocolURI,
            String filterRoot, int trackMode,int filterMode) {
        super(parent);
        init(protocolURI, filterRoot, trackMode, filterMode);
    }

    /**
     * Inits the protocol filter.
     * 
     * @param protocolURI
     *            URI of the protocol version.
     * @param filterRoot
     *            Local name of the root element.
     * @param trackMode
     *            Define track mode for the internal
     *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            Define filter mode for this filter {@link #FILTERMODE_FILTER FILTERMODE_FILTER(inkl. PDPFilter)} or {@link #FILTERMODE_PARSER FILTERMODE_PARSER(Parses all elements)}.
     */
    private void init(String protocolURI, String filterRoot, int trackMode,  int filterMode) {
        this.protocolURI = protocolURI;
        this.filterRoot = filterRoot;
        this.filterElements = new HashMap();
        this.elementStack = new FilterStack(trackMode);
        this.filterMode = filterMode;
    }

    /**
     * Checks the parameters of the start/endElement events against null
     * parameters.
     * 
     * @param checkAtts
     *            Signals, if attributes have to be checked or not.
     * @throws SAXException
     *             if null parameter occurs.
     */
    private void checkElementParameters(boolean checkAtts, String uri,
            String localName, String qName, Attributes atts)
            throws SAXException {
        // Null parameter check
        if (uri == null || localName == null || qName == null)
            throw new SAXException("Parameters must not be null");
        if (checkAtts && atts == null)
            throw new SAXException("Parameters must not be null");
    }

    /**
     * Checks the protocol URI against the protocolURI of this filter.
     * TODO: Provider muss wieder namespaceURI senden !
     * @throws SAXException
     *             if incorrect protocol URI occurs.
     */
    private void checkProtocolURI(String uri) throws SAXException {
        
/*
        if (!protocolURI.equals(uri))
            // Unsupported protocol version
            throw new SAXException("Unsupported protocol version " + uri);
*/    }

    /**
     * Prepares and processes the data collection on the stacks, controls event
     * forwarding and callback methods.
     * 
     * @see ContentHandler#startElement(String,String,String,Attributes)
     */
    public void startElement(String uri, String localName, String qName,
            Attributes atts) throws SAXException {
        // check for null parameter
        checkElementParameters(true, uri, localName, qName, atts);
        // Remember this event for forwarding purposes
        SAXElement element = new SAXElement(uri, localName, qName, atts);

        // StartElement event for (undefined) root element
        if ((!filterInitialised)&&((filterRoot == null) || filterRoot.equals(localName))) {
            // check protocol version
            checkProtocolURI(uri);
            // set filter root element (if it was null before)
            filterRoot = localName;
            // push event on the stack
            elementStack.push(element);
            // init parsing
            parsingComplete = false;
            // callback parseInit
            element = parseInit(element);
            filterInitialised = true;
            // active subfilter ?
        } else if (subFilter != null) {
            // redirect only to subfilter
            subFilter.startElement(uri, localName, qName, atts);
            // prevent delivery to content handler
            element = null;
            // registered (undefined) filter element (all element ==
            // containsKey(null)? (no active subfilter)
        } else if (filterElements.isEmpty() || filterElements.containsKey(null)
                || filterElements.containsKey(localName)) {
            // push event on the stack
            elementStack.push(element);
            // callback filterElement (perhaps, there is a new subfilter
            // registered ?
            element = filterElement(element);
            // find subfilter for this element
            if (filterElements.containsKey(null))
                subFilter = (ProtocolFilter) filterElements.get(null);
            else
                subFilter = (ProtocolFilter) filterElements.get(localName);
            if (subFilter == null) {
                // No registered subFilter, check protocol version and push
                // event on the stack
                checkProtocolURI(uri);
            } else {
                // registered subfilter found-> redirect event only to subfilter
                // remove current event from this filter stack
                elementStack.pop();
                if (element != null) {
                    // event not removed by callback filter
                    subFilter.startElement(element.getURI().toString(), element
                            .getLocalName(), element.getQName(), element
                            .getAttributes());
                    // prevent delivery to content handler
                    element = null;
                }
            }
        } else
            // Invalid element occured
            throw new SAXException("Invalid element " + uri + "/" + localName);
        	;
        if (element != null)
            // Forward event if not removed or redirected to subfilter
            super.startElement(element.getURI().toString(), element
                    .getLocalName(), element.getQName(), element
                    .getAttributes());
    }

    /**
     * Completes the processesing of data collection on the stacks, controls
     * event forwarding and callback methods.
     * 
     * @see ContentHandler#endElement(String,String,String)
     */
    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        // check null parameter
        checkElementParameters(false, uri, localName, qName, null);
        // Remember element for forwarding purposes
        SAXElement element = null;
        // root element found -> parsing complete
        if ((subFilter==null)&&filterRoot.equals(localName)) {
            // check protocol version
            checkProtocolURI(uri);
            // callback parsingComplete
            element = parseComplete(elementStack.pop(uri, localName, qName));
            // Signal parsing complete
            parsingComplete = true;
            // active subfilter ?
        } else if (subFilter != null) {
            // redirect event to subfilter
            subFilter.endElement(uri, localName, qName);
            // subfilter complete
            if (subFilter.isParsingComplete()) {
                // remove subfilter
                subFilter = null;
                // callback parseElement
                parseElement(new SAXElement(uri, localName, qName, null));
            }
            // prevent delivery to content handler
            element = null;
            // no registered filter treeElements ? (no active subfilter)
        } else if (filterElements.isEmpty()) {
            // check protocol version
            checkProtocolURI(uri);
            // Pop element from the stack and forward it because no subfilter
            // and not registered!
            element = elementStack.pop(uri, localName, qName);

            // ??? Pop element from the stack and callback parseElement ???
            // parse element for unregistered filter ???
            //element = parseElement(elementStack.pop(uri, localName, qName));
            // registered filter element (all treeElements == containsKey(null))?
            // (no active subfilter)
        } else if (filterElements.containsKey(localName)
                || filterElements.containsKey(null)) {
            // check protocol version
            checkProtocolURI(uri);
            // Pop element from the stack and callback parseElement
            element = parseElement(elementStack.pop(uri, localName, qName));
        } else
            // Invalid element occured
            throw new SAXException("Invalid element " + uri + "/" + localName);
        if (element != null)
            // Forward event if not removed or redirected to subfilter
            super.endElement(element.getURI().toString(), element
                    .getLocalName(), element.getQName());
    }

    /**
     * Add the characters events to the characters stack and forwards it to the
     * subfilter or content handler.
     * 
     * @see ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        // Remember event for forwarding
        SAXElement element = null;

        // No active subFilter
        if (subFilter == null) {
            // Push characters on the stack
            elementStack.characters(ch, start, length);
            // callback filterCharacters
            element = filterCharacters(elementStack.peek());
            if (element != null) {
                // event not removed, update element on the stack
                elementStack.pop();
                elementStack.push(element);
            }
            // active subFilter
        } else {
            // callback filterCharacters
// nicht fr subfilter oder? muss doch direkt hingehen, ohne filter            element = filterCharacters(elementStack.peek());
            // redirect event to subfilter
            subFilter.characters(ch, start, length);
        }
        if (element != null)
            // Forward event if not removed or redirected to subfilter
            if (element.getValue() != null)
            {                // and if element has character value
//                super.characters(element.getValueAsString().toCharArray(), 0, element.getValue().length());
                super.characters(ch, start, length);
            }
    }

    /**
     * Gets the protocol version URI.
     * 
     * @return Returns the protocol URI.
     */
    public String getProtocolURI() {
        return protocolURI;
    }

    /**
     * Sets the protocol version URI.
     * 
     * @param protocolURI The protocol URI.
     */
    protected void setProtocolURI(String protocolURI) {
        this.protocolURI=protocolURI;
    }

    /**
     * Return the completion state of parsing process.
     * 
     * @return <b>true </b>, when the header is parsed completely, <b>false </b>
     *         otherwise.
     */
    boolean isParsingComplete() {
        return parsingComplete;
    }

    /**
     * This method is called when the startElement event of the root element was
     * received, before it will be forwarded to the content handler.
     * 
     * @param element
     *            The element of the startElement event (contains no value).
     * @return The (modified) element or null when the event shall be removed.
     * @throws SAXException
     *             If any parsing error occurs.
     */
    protected SAXElement parseInit(SAXElement element) throws SAXException {
        return element;
    }

    /**
     * This method is called when the startElement event of any element for a
     * registered subfilter was received, before it is forwarded to a registered
     * subfilter or the content handler.
     * 
     * @param element
     *            The element corresponding to the startElement event.
     * @return The (modified) element or null when element shall be removed.
     * @throws SAXException
     *             If any parsing error occurs.
     */
    protected SAXElement filterElement(SAXElement element) throws SAXException {
        return element;
    }

    /**
     * This method is called when the characters event of any element for a
     * registered subfilter was received, before it is forwarded to a registered
     * subfilter or the content handler.
     * 
     * @param element
     *            The element corresponding to the startElement event.
     * @return The (modified) element or null when element shall be removed.
     * @throws SAXException
     *             If any parsing error occurs.
     */
    protected SAXElement filterCharacters(SAXElement element)
            throws SAXException {
        return element;
    }

    /**
     * This method is called when the endElement event of any element for a
     * registered element was received, before it is forwarded to the content
     * handler. This method is also called, when a registered subfilter signals
     * {@link #isParsingComplete() isParsingComplete()}. But, the returned
     * element will not influence the event processing, because this has to be
     * done by the subfilter.
     * 
     * @param element
     *            The element corresponding to the endElement event.
     * @return The (modified) element or null when element shall be removed.
     * @throws SAXException
     *             If any parsing error occurs.
     */
    protected SAXElement parseElement(SAXElement element) throws SAXException {
        return element;
    }

    /**
     * This method is called when the endElement event of root element was
     * received, before it is forwarded to the content handler
     * 
     * @param element
     *            The complete root element (including the value).
     * @return The (modified) element or null when element shall be removed.
     * @throws SAXException
     *             If any parsing error occurs.
     */
    protected SAXElement parseComplete(SAXElement element) throws SAXException {
        return element;
    }

    /**
     * Registers a subfilter. Registering overwrites already registered
     * subfilters for the same element.
     * 
     * @param name
     *            local name for the subfilter to register.
     * @param filter
     *            subfilter for this element or null.
     */
    protected void registerSubFilter(String name, ProtocolFilter filter) {
        filterElements.put(name, filter);
    }

    /**
     * Unregisters a subfilter.
     * 
     * @param name
     *            local name for the subfilter to unregister.
     * @return The removed subfilter.
     */
    protected ProtocolFilter unregisterSubFilter(String name) {
        return (ProtocolFilter) (filterElements==null?null:filterElements.remove(name));
    }

    /**
     * Gets the parameter object of this filter
     * 
     * @return Returns the filter parameter.
     */
    public Object getFilterParameter() {
        return filterParameter;
    }

    /**
     * Sets the parameter object of this filter
     * 
     * @param parameter
     *            Returns the filter parameter.
     */
    public void setFilterParameter(Object parameter) {
        this.filterParameter = parameter;
    }

    /**
     * Checks, if a filter parameter is set.
     * 
     * @throws SAXException
     *             If no parameter is set (null)
     */
    protected void checkFilterParameter() throws SAXException {
        if ((getFilterMode()==ProtocolFilter.FILTERMODE_FILTER) && (filterParameter == null))
            throw new SAXException("Uninitialised filter parameter");
    }
    
    /**
     * Gets the filterMode.
     * May be on of these:
     * <li>{@link #FILTERMODE_FILTER FILTERMODE_FILTER(inkl. PDPFilter)}</li>
     * <li>{@link #FILTERMODE_PARSER FILTERMODE_PARSER(Parses all elements)}</li>.
     * 
     * @return Returns the filterMode.
     */
    public int getFilterMode() {
        return filterMode;
    }
}
