/*
 * $Id: RBACFilterParameter.java,v 1.6 2005/10/07 18:27:45 suhrbier Exp $
 * Created on 04.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACFilterParameter.java,v $
 * Revision 1.6  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.5  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.4  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.3  2005/08/10 11:59:37  suhrbier
 * All Filters tested.
 *
 * Revision 1.2  2005/08/06 16:16:14  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.util.Set;

import nbi.xmlsec.biocase.protocol.Diagnostics;
import nbi.xmlsec.xacml.profile.rbac.RBACPDP;

/**
 * Container class to store Parameters for the {@link RBACFilter RBACFilter}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class RBACFilterParameter extends PDPFilterParameter {
    /** PDP action to be checked with the pdp. */
    String pdpAction;

    /** Diagnostics to be filled up with PDP evaluation results */
    Diagnostics diagnostics;

    /**
     * Inits the RBACFilterParameter. Arguments must not be null.
     * 
     * @param action
     *            PDP action to be checked for with the pdp.
     * @param pdp
     *            {@link RBACPDP RBACPDP}to be used for access control
     *            evaluation.
     * @param roles
     *            Set of roles to be evaluated with the pdp.
     * @param diagnostics
     *            Diagnostics to be filled up with PDP evaluation results.
     */
    public RBACFilterParameter(String action, RBACPDP pdp, Set roles, Diagnostics diagnostics) {
        super(pdp, roles);
        this.pdpAction = action;
        setDiagnostics(diagnostics);
        this.checkInit();
    }

    /**
     * Checks the correct initialisation of the class initialisation parameters.
     */
    private void checkInit() {
        if ((this.pdpAction == null)||(diagnostics==null))
            throw new IllegalArgumentException("Arguments must not be null");
    }

    /**
     * Gets the diagnostics
     * 
     * @return Returns the diagnostics.
     */
    public Diagnostics getDiagnostics() {
        return diagnostics;
    }

    /**
     * Sets diagnostics
     * 
     * @param diagnostics
     *            The diagnostics to set.
     */
    public void setDiagnostics(Diagnostics diagnostics) {
        this.diagnostics = diagnostics;
    }

    /**
     * Gets the pdpAction
     * 
     * @return Returns the pdpAction.
     */
    public String getPdpAction() {
        return pdpAction;
    }

    /**
     * Sets pdpAction
     * 
     * @param pdpAction
     *            The pdpAction to set.
     */
    public void setPdpAction(String pdpAction) {
        this.pdpAction = pdpAction;
    }
}
