/*
 * $Id: RequestFilter.java,v 1.22 2006/08/09 17:24:13 suhrbier Exp $
 * Created on 06.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RequestFilter.java,v $
 * Revision 1.22  2006/08/09 17:24:13  suhrbier
 * *** empty log message ***
 *
 * Revision 1.21  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.20  2006/05/08 17:21:57  suhrbier
 * Transform Response into DOM Document.
 *
 * Revision 1.19  2006/05/03 19:18:32  suhrbier
 * weitere environment-Attribute hinzugefgt.
 *
 * Revision 1.18  2006/04/27 19:38:30  suhrbier
 * Condition im RoleManager eingebaut und getestet.
 *
 * Revision 1.17  2006/04/24 19:50:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.16  2006/04/20 17:15:27  suhrbier
 * *** empty log message ***
 *
 * Revision 1.15  2006/04/19 15:31:20  suhrbier
 * *** empty log message ***
 *
 * Revision 1.14  2005/11/02 20:10:32  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.13  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.12  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.11  2005/09/01 19:50:53  suhrbier
 * EvaluationsFehler ni RequestFilter und PEPFilter beseitigt
 *
 * Revision 1.10  2005/08/31 18:32:06  suhrbier
 * Resource fr Scan-Request aus RequestFormat+Concept zusammengesetzt.
 *
 * Revision 1.9  2005/08/20 20:36:04  suhrbier
 * LOP Request Evaluation implementiert.
 *
 * Revision 1.8  2005/08/20 18:52:05  suhrbier
 * Umstellung auf Target Filterung in PermissionPolicySet
 *
 * Revision 1.7  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.6  2005/08/15 13:24:45  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.5  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.4  2005/08/10 11:59:37  suhrbier
 * All Filters tested.
 *
 * Revision 1.3  2005/08/08 19:39:08  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.2  2005/08/06 22:08:28  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/08/06 16:16:14  suhrbier
 * Zwischenspeicherung
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.net.URI;
import java.util.Iterator;
import java.util.List;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.COP;
import nbi.xmlsec.biocase.protocol.LOP;
import nbi.xmlsec.biocase.protocol.Operator;
import nbi.xmlsec.biocase.protocol.Request;
import nbi.xmlsec.biocase.protocol.SearchOperationType;
import nbi.xmlsec.xacml.profile.rbac.RoleAssignmentPolicyIdentifier;
import nbi.xmlsec.xacml.profile.rbac.builder.RBACRequestCtxBuilder;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import com.sun.xacml.attr.AnyURIAttribute;
import com.sun.xacml.attr.BooleanAttribute;
import com.sun.xacml.attr.IntegerAttribute;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.ctx.Attribute;
import com.sun.xacml.ctx.Result;

/**
 * The RequestFilter parses and evaluates requests of the BioCASE protocol. It
 * consists of two internal filters working together:
 * <ul>
 * <li>HeaderFilter</li>
 * <li>OperationTypeFilter</li>
 * </ul>
 * 
 * All incoming SAX Events are forwarded to the ContentHandler.The
 * {@link HeaderFilter HeaderFilter}parsed the BioCASE header. The
 * {@link OperationTypeFilter OperationTypeFilter}parses the request's scan or
 * search operation type. The request may be retrieved with the method
 * {@link #getRequest() getRequest()}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.22 $
 * @since
 */

public class RequestFilter extends ProtocolFilter {

    /** Local name of the root element <code>request</code> */
    public static final String LN_ROOT = "request";

    /** PDP Action <code>capabilities</code> */
    public static final String ACTION_REQUEST_CAPABILITIES = "capabilities-request";

    /** PDP Action <code>scan</code> */
    public static final String ACTION_REQUEST_SCAN = ScanOperationTypeFilter.LN_ROOT
            + "-request";

    /** PDP Action <code>search</code> */
    public static final String ACTION_REQUEST_SEARCH = SearchOperationTypeFilter.LN_ROOT
            + "-request";

    /** PDP Resource <code>any</code> */
    public static final String RESOURCE_ANY = "any";

    /** Track mode for the filter stack */
    final static int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME
            | FilterStack.TRACK_MODE_CHARACTERS;

    /** Subfilter for the header */
    HeaderFilter headerFilter = null;

    /** Subfilter for the request type */
    OperationTypeFilter opTypeFilter = null;

    /** Request to be evaluated */
    Request request = null;

    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public RequestFilter(String protocolURI, int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
    }

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public RequestFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(parent, protocolURI, LN_ROOT, trackMode, filterMode);
    }

    /**
     * Registers the subfilter for the header.
     * 
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element) throws SAXException {
        // register the header filter
        headerFilter = new HeaderFilter(getParent(), getProtocolURI(), getFilterMode());
        headerFilter.setContentHandler(getContentHandler());
        registerSubFilter(HeaderFilter.LN_ROOT, headerFilter);
        return element;
    }

    /**
     * Registers and sets the parameters for the OperationTypeFilter. This must
     * be done here, after parsing the Request header completely.
     * 
     * @see ProtocolFilter#parseElement(SAXElement)
     */
    protected SAXElement parseElement(SAXElement element) throws SAXException {
        // if header filter parsing complete
        if (HeaderFilter.LN_ROOT.equals(element.getLocalName())) {
            // creates the request with parsed header
            this.request = new Request(getProtocolURI());
            this.request.setHeader(headerFilter.getHeader());
            // register subfilter depending on the request type
            if (isSearchRequest()) {
                opTypeFilter = new SearchOperationTypeFilter(getParent(),
                        getProtocolURI(), getFilterMode());
                opTypeFilter.setContentHandler(getContentHandler());
                registerSubFilter(SearchOperationTypeFilter.LN_ROOT,
                        opTypeFilter);
            } else if (isScanRequest()) {
                opTypeFilter = new ScanOperationTypeFilter(getParent(),
                        getProtocolURI(), getFilterMode());
                opTypeFilter.setContentHandler(getContentHandler());
                registerSubFilter(ScanOperationTypeFilter.LN_ROOT, opTypeFilter);
            }
        }
        return element;
    }

    /**
     * Evaluates the access conditions, when
     * {@link PDPFilterParameter PDPFilterParameter}set. This must be done
     * here, when the request has been completely parsed.
     * 
     * @see ProtocolFilter#parseComplete(SAXElement)
     */
    protected SAXElement parseComplete(SAXElement element) throws SAXException {
        // If request type supported
        if (isCapabilitiesRequest() || isScanRequest() || isSearchRequest()) {
            // set request's operation type from the parsed header and operation
            // type
            if (isScanRequest())
                this.request.setOpType(((ScanOperationTypeFilter) opTypeFilter)
                        .getOperationType());
            else if (isSearchRequest())
                this.request
                        .setOpType(((SearchOperationTypeFilter) opTypeFilter)
                                .getOperationType());
            else {
                this.request = new Request(getProtocolURI());
                this.request.setHeader(headerFilter.getHeader());
            }
            // Define action and resource for PDP evaluation request depending
            // on request type
            String action = request.getHeader().getType() + "-request";
            String resource = RESOURCE_ANY; // capabilities request
            if (isCapabilitiesRequest()) {
                // evaluate request
                if (!evaluateRequest(action, resource))
                    throw new SAXException(
                            action
                                    + " denied on "
                                    + resource
                                    + " for role(s) "
                                    + PDPFilterParameter
                                            .getRolesAsString(getParameter()
                                                    .getRoles()));
            } else if (isScanRequest()) {
                resource = ((ScanOperationTypeFilter) opTypeFilter)
                        .getOperationType().getRequestFormat()
                        + ((ScanOperationTypeFilter) opTypeFilter)
                                .getOperationType().getConcept();
                // evaluate request
                if (!evaluateRequest(action, resource))
                    throw new SAXException(
                            action
                                    + " denied on "
                                    + resource
                                    + " for role(s) "
                                    + PDPFilterParameter
                                            .getRolesAsString(getParameter()
                                                    .getRoles()));
            } else if (isSearchRequest()) {
                // Search request may have different Types (COP or LOP) with
                // different methods to retrieve the resource and to evaluate
                // access
                if (COPFilter
                        .isOperator(((SearchOperationTypeFilter) opTypeFilter)
                                .getOperationType().getFilter().getType())) {
                    // resource = requestFormat + COP-Path
                    resource = ((SearchOperationTypeFilter) opTypeFilter)
                            .getOperationType().getRequestFormat()
                            + ((COP) ((SearchOperationTypeFilter) opTypeFilter)
                                    .getOperationType().getFilter()).getPath();
                    // evaluate request
                    if (!evaluateRequest(action, resource))
                        throw new SAXException(action
                                + " denied on "
                                + resource
                                + " for role(s) "
                                + PDPFilterParameter
                                        .getRolesAsString(getParameter()
                                                .getRoles()));
                } else if (LOPFilter
                        .isOperator(((SearchOperationTypeFilter) opTypeFilter)
                                .getOperationType().getFilter().getType())) {
                    List operators = ((LOP) ((SearchOperationTypeFilter) opTypeFilter)
                            .getOperationType().getFilter()).getOperators();
                    if (operators != null) {
                        Iterator opIt = operators.iterator();
                        // For all LOP operators, check Path
                        while (opIt.hasNext()) {
                            Operator op = (Operator) opIt.next();
                            if (COPFilter.isOperator(op.getType())) {
                                // resource = requestFormat + COP-Path
                                resource = ((SearchOperationTypeFilter) opTypeFilter)
                                        .getOperationType().getRequestFormat()
                                        + ((COP) op).getPath();
                                // evaluate request
                                if (!evaluateRequest(action, resource))
                                    throw new SAXException(
                                            action
                                                    + " denied on "
                                                    + resource
                                                    + " for role(s) "
                                                    + PDPFilterParameter
                                                            .getRolesAsString(getParameter()
                                                                    .getRoles()));
                            }
                        }
                    }
                }
            }
        }
        return element;
    }

    /**
     * Checks, if the parsed header signals a capabilities request.
     * 
     * @return true, if it's an capabilities request, false otherwise.
     */
    boolean isCapabilitiesRequest() {
        return "capabilities".equals(request.getHeader().getType());
    }

    /**
     * Checks, if the parsed header signals a scan request.
     * 
     * @return true, if it's an scan request, false otherwise.
     */
    boolean isScanRequest() {
        return ScanOperationTypeFilter.LN_ROOT.equals(request.getHeader()
                .getType());

    }

    /**
     * Checks, if the parsed header signals a search request.
     * 
     * @return true, if it's an search request, false otherwise.
     */
    boolean isSearchRequest() {
        return SearchOperationTypeFilter.LN_ROOT.equals(request.getHeader()
                .getType());
    }

    /**
     * Evaluates request access for a given resource and action. Adds the
     * following Request elements to the environment attributes of the
     * RequestCtx:
//     * <li>sendTime (Request.Header.sendTime as DateTimeAttribute)</li>
     * <li>source (Request.Header.source as StringAttribute)</li>
     * <li>requestFormat (Request.Scan|Search.requestFormat as AnyUriAttribute)
     * </li>
     * <li>responseFormat (Request.Search.responseFormat as AnyUriAttribute)
     * </li>
     * <li>start (Request.Search.responseFormat#start as IntegerAttribute)
     * </li>
     * <li>limit (Request.Search.responseFormat#limit as IntegerAttribute)
     * </li>
     * <li>count (Request.Search.responseFormat#count as BooleanAttribute)
     * </li>
     * 
     * @param action
     *            The action to be evaluated.
     * @param resource
     *            The resource to be evaluated.
     * @return <code>true</code> ,if access is permitted, <code>false</code>
     *         otherwise.
     */
    private boolean evaluateRequest(String action, String resource) {
        // Evaluate request with the RBACPDP
        boolean ret = false;
        if (getParameter() != null) {
            RBACRequestCtxBuilder requestBuilder = new RBACRequestCtxBuilder();
            requestBuilder.addRoles(getParameter().getRoles());
            requestBuilder.addResource(resource);
            requestBuilder.addAction(action);
            try {
                // Add ResponseFormatLimit as Environment Attribute
                /*
                if (request.getHeader().getSendTime() != null)
                 * requestBuilder.addEnvironment(new Attribute(URI
                 * .create("sendTime"), null, null, DateTimeAttribute
                 * .getInstance(request.getHeader().getSendTime())));
                 */
                if (request.getHeader().getSource() != null)
                    requestBuilder.addEnvironment(new Attribute(URI
                            .create("source"), null, null, StringAttribute
                            .getInstance(request.getHeader().getSource())));
            } catch (Exception e) {
                logger.fatal("This should never happen", e);
            }

            if (RequestFilter.ACTION_REQUEST_SEARCH.equals(action)
                    || RequestFilter.ACTION_REQUEST_SCAN.equals(action)) {
                requestBuilder.addEnvironment(new Attribute(URI
                        .create("requestFormat"), null, null,
                        new AnyURIAttribute(URI.create(request.getOpType()
                                .getRequestFormat()))));
                if (RequestFilter.ACTION_REQUEST_SEARCH.equals(action)) {
                    requestBuilder
                            .addEnvironment(new Attribute(
                                    URI.create("responseFormat"),
                                    null,
                                    null,
                                    new AnyURIAttribute(
                                            URI
                                                    .create(((SearchOperationType) request
                                                            .getOpType())
                                                            .getResponseFormat()))));
                    requestBuilder.addEnvironment(new Attribute(URI
                            .create("limit"), null, null, new IntegerAttribute(
                            ((SearchOperationType) request.getOpType())
                                    .getResponseFormatLimit().longValue())));
                    requestBuilder.addEnvironment(new Attribute(URI
                            .create("start"), null, null, new IntegerAttribute(
                            ((SearchOperationType) request.getOpType())
                                    .getResponseFormatStart().longValue())));
                    requestBuilder.addEnvironment(new Attribute(URI
                            .create("count"), null, null, BooleanAttribute
                            .getInstance(((SearchOperationType) request
                                    .getOpType()).getCount().booleanValue())));
                }

            }

            if (getParameter().getPdp().evaluate(requestBuilder.getInstance())
                    .getDecision() == Result.DECISION_PERMIT) {
                ret = true;
            } else {
                StringBuffer logInfo = new StringBuffer("DENY " + action
                        + " on " + resource + " for role(s)");
                Iterator roleIt = getParameter().getRoles().iterator();
                while (roleIt.hasNext()) {
                    RoleAssignmentPolicyIdentifier roleId = (RoleAssignmentPolicyIdentifier) roleIt
                            .next();
                    logInfo.append(" ");
                    logInfo.append(roleId.getLabel());
                    logger.info(logInfo);
                }
            }
        }
        return ret;
    }

    /**
     * Gets the parameter of this filter.
     * 
     * @return filter parameter.
     */
    protected PDPFilterParameter getParameter() {
        return (PDPFilterParameter) getFilterParameter();
    }

    /**
     * Gets the request
     * 
     * @return Returns the request.
     */
    public Request getRequest() {
        return request;
    }
}
