/*
 * $Id: ScanOperationTypeFilter.java,v 1.11 2006/07/24 17:31:09 suhrbier Exp $
 * Created on 06.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ScanOperationTypeFilter.java,v $
 * Revision 1.11  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.10  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.9  2005/08/31 18:09:51  suhrbier
 * Scan-Request um filter-element erweitert.
 *
 * Revision 1.8  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.7  2005/08/15 21:58:39  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.6  2005/08/15 13:24:45  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.5  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.4  2005/08/10 11:59:37  suhrbier
 * All Filters tested.
 *
 * Revision 1.3  2005/08/08 19:39:08  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.2  2005/08/06 23:28:44  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.1  2005/08/06 22:08:28  suhrbier
 * Zwischenspeicherung
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.ScanOperationType;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * This class parses an BioCASE ScanOperationType from a SAX stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * {@link ScanOperationType ScanOperationType}may be retrieved via
 * {@link #getOperationType() getOperationType()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.11 $
 * @since
 */
public class ScanOperationTypeFilter extends OperationTypeFilter {

    /**
     * Local name of the root element <code>scan</code>
     */
    public static final String LN_ROOT = "scan";

    /** Local name of the element <code>concept</code> */
    public static final String LN_CONCEPT = "concept";

    /**
     * Track mode for the filter stack
     */
    static final int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME
            | FilterStack.TRACK_MODE_CHARACTERS;

    /**
     * The current subfilter for the search operation type filter 
     */
    OperatorFilter opFilter = null;

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ScanOperationTypeFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(parent, protocolURI, LN_ROOT, trackMode, filterMode);
        initSubFilter();
    }

    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ScanOperationTypeFilter(String protocolURI, int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
        initSubFilter();
    }

    /**
     * Registers subfilters for subelements.
     */
    protected void initSubFilter() {
        super.initSubFilter();
        registerSubFilter(LN_CONCEPT, null);
    }

    /**
     * Initialises the ScanOperationType.
     * 
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element)
            throws SAXException {
        // Setup operator filter (must be done here, because content handler is null during initialisation
        opFilter = new OperatorFilter(getParent(), getProtocolURI(), getFilterMode());
        // set content handler
        opFilter.setContentHandler(getContentHandler());
        // register the subfilter
        registerSubFilter(OperatorFilter.LN_ROOT, opFilter);

        // init instance of parsed operation type
        operationType = new ScanOperationType(getProtocolURI());
        return element;
    }

    /**
     * Parses subelements and build up the ScanOperationType.
     * 
     * @see ProtocolFilter#parseElement(SAXElement)
     */
    protected SAXElement parseElement(SAXElement element)
            throws SAXException {
        // parse request format
        super.parseElement(element);
        // parse concept
        if (LN_CONCEPT.equals(element.getLocalName()))
            getOperationType().setConcept(element.getValue().toString());
        else if (OperatorFilter.LN_ROOT.equals(element.getLocalName()))
                getOperationType().setFilter(opFilter.getOperator());
        
        return element;
    }

    /**
     * Gets the OperationType
     * 
     * @return Returns the operation type.
     */
    public ScanOperationType getOperationType() {
        return (ScanOperationType) operationType;
    }
}
