/*
 * $Id: AbstractPolicyTreeElementBuilder.java,v 1.3 2005/10/14 18:31:00 suhrbier Exp $
 * Created on 07.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: AbstractPolicyTreeElementBuilder.java,v $
 * Revision 1.3  2005/10/14 18:31:00  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.1  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;
import java.net.URISyntaxException;

import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Target;

/**
 * Abstract base class for all policy tree element building classes.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */

abstract public class AbstractPolicyTreeElementBuilder implements
        PolicyTreeElementBuilder {

    /** The policy identifier */
    protected URI identifier = null;

    /** a String describing the policy */
    protected String description = null;

    /** the policy's Target */
    protected Target target = null;

    
    /**
     * 
     */
    protected AbstractPolicyTreeElementBuilder() {
        super();
    }
    /**
     * Creates an instance of the policy tree element built.
     * 
     * @see PolicyTreeElementBuilder#getInstance()
     */
    abstract public PolicyTreeElement getInstance();

    /**
     * Sets the policy's description.
     * 
     * @param description
     *            The policy's description to set.
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Sets the policy's identifier.
     * 
     * @param id
     *            The policy identifier to set.
     */
    public void setIdentifier(URI id) {
        this.identifier = id;
    }

    /**
     * Sets the policy's target.
     * 
     * @param target
     *            The policy's target to set.
     */
    public void setTarget(Target target) {
        this.target = target;
    }

    /**
     * Sets the policy's identifier.
     * 
     * @param id
     *            The policy identifier to set.
     * @throws URISyntaxException
     * @see URI#URI(java.lang.String).
     */
    public void setIdentifier(String id) throws URISyntaxException {
        setIdentifier(new URI(id));
    }

    /**
     * Sets the policy's target.
     * 
     * @param target
     *            The policy's target to set.
     */
    public void setTarget(TargetBuilder target) {
        this.target = target.getInstance();
    }
}
