/*
 * $Id: ApplyBuilder.java,v 1.8 2006/07/24 17:31:10 suhrbier Exp $
 * Created on 10.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ApplyBuilder.java,v $
 * Revision 1.8  2006/07/24 17:31:10  suhrbier
 * update JavaDoc
 *
 * Revision 1.7  2006/05/03 19:18:33  suhrbier
 * weitere environment-Attribute hinzugefgt.
 *
 * Revision 1.6  2006/04/27 18:21:48  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2006/04/26 20:46:25  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2006/04/26 19:01:00  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2006/04/26 10:52:30  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2006/04/24 19:50:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;
import java.util.ArrayList;
import java.util.Iterator;

import nbi.xmlsec.xacml.util.SyntaxScheme;

import com.sun.xacml.ParsingException;
import com.sun.xacml.UnknownIdentifierException;
import com.sun.xacml.attr.StandardAttributeFactory;
import com.sun.xacml.cond.Apply;
import com.sun.xacml.cond.BagFunction;
import com.sun.xacml.cond.Evaluatable;
import com.sun.xacml.cond.Function;
import com.sun.xacml.cond.FunctionTypeException;

/**
 * Base Class to create ConditionTypeBuilder or ApplyTypeBuilder instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.8 $
 * @since
 */

public class ApplyBuilder extends EvaluatableBuilder {

    static public final String XMLSCHEMA_NAMESPACE = "http://www.w3.org/2001/XMLSchema#";

    /** the Function to use in evaluating the elements in the apply. */
    private Function function = null;

    /** the higher-order bag function to use. */
    private Function bagFunction = null;

    /** true if this Apply is a Condition, false otherwise. */
    protected boolean isCondition = false;

    protected ApplyBuilder() {
        evaluatables = new ArrayList();
    }

    /**
     * Creates an ApplyBuilder instance from a {@link SyntaxScheme SyntaxScheme}.
     * @param scheme {@link SyntaxScheme SyntaxScheme} to be parsed
     * @throws ParsingException
     * @throws UnknownIdentifierException
     * @throws FunctionTypeException
     */
    protected ApplyBuilder(SyntaxScheme scheme) throws ParsingException,
            UnknownIdentifierException, FunctionTypeException {
        this();
        parse(scheme);
    }

    /**
     * Parsed an ApplyBuilder instance from a {@link SyntaxScheme SyntaxScheme}.
     * It recognises the following new functions:
     * <li>sub</li>
     * <li>res</li>
     * <li>act</li>
     * <li>env</li>
     * and creates an one-and-only bag of the corresponding type, and fills it with the related argument value from the syntax scheme.
     * 
     * @param scheme {@link SyntaxScheme SyntaxScheme} to be parsed
     * @throws ParsingException
     * @throws UnknownIdentifierException
     * @throws FunctionTypeException
     */
    protected void parse(SyntaxScheme scheme) throws ParsingException,
            UnknownIdentifierException, FunctionTypeException {
        String function = scheme.getFunction();
        if (function != null) {
            setFunction(FunctionBuilder.getInstance(function, false));
            Iterator argIt = scheme.getArgs().iterator();
            while (argIt.hasNext()) {
                SyntaxScheme arg = (SyntaxScheme) argIt.next();
                if (arg.getFunction() != null) {
                    AttributeDesignatorBuilder attDesBuilder = null;
                    if ("sub".equals(arg.getFunction())
                            || "res".equals(arg.getFunction())
                            || "act".equals(arg.getFunction())
                            || "env".equals(arg.getFunction())) {
                        if ("env".equals(arg.getFunction())) {
                            attDesBuilder = new EnvironmentAttributeDesignatorBuilder(
                                    ((SyntaxScheme) arg.getArgs().get(0))
                                            .getValue());
                        } else if ("sub".equals(arg.getFunction())) {
                            attDesBuilder = new SubjectAttributeDesignatorBuilder(
                                    ((SyntaxScheme) arg.getArgs().get(0))
                                            .getValue());
                        } else if ("res".equals(arg.getFunction())) {
                            attDesBuilder = new ResourceAttributeDesignatorBuilder(
                                    ((SyntaxScheme) arg.getArgs().get(0))
                                            .getValue());
                        } else if ("act".equals(arg.getFunction())) {
                            attDesBuilder = new ActionAttributeDesignatorBuilder(
                                    ((SyntaxScheme) arg.getArgs().get(0))
                                            .getValue());
                        }
                        ApplyBuilder attDes = new ApplyBuilder();
                        attDes.setFunction(FunctionBuilder.getInstance(scheme
                                .getFunctionType()
                                + BagFunction.NAME_BASE_ONE_AND_ONLY, false));
                        attDesBuilder.setType(XMLSCHEMA_NAMESPACE
                                + scheme.getFunctionType());
                        attDes.add(attDesBuilder.getInstance());
                        add(attDes);
                    } else
                        add(new ApplyBuilder(arg));
                } else if (arg.getValue() != null) {
                    add(StandardAttributeFactory.getFactory()
                            .createValue(
                                    URI.create(XMLSCHEMA_NAMESPACE
                                            + scheme.getFunctionType()),
                                    arg.getValue()));
                } else
                    throw new ParsingException(
                            "SyntaxScheme argument contains no function or value");
            }
        } else
            throw new ParsingException("SyntaxScheme contains no function");
    }

    /**
     * Creates a a new Apply instance.
     * 
     * @see EvaluatableBuilder#getInstance()
     */
    public Evaluatable getInstance() {
        return new Apply(function, evaluatables, bagFunction, isCondition);
    }

    /**
     * Sets the higher-order bag function to use. Parses the function from an
     * syntax scheme expression.
     * 
     * @param expression
     *            The syntax scheme of the bagFunction to set.
     * @throws FunctionTypeException
     * @throws UnknownIdentifierException
     */
    public void setBagFunction(String expression)
            throws UnknownIdentifierException, FunctionTypeException {
        this.bagFunction = FunctionBuilder.getInstance(expression, false);
    }

    /**
     * Sets the higher-order bag function to use.
     * 
     * @param bagFunction
     *            The bagFunction to set.
     */
    public void setBagFunction(Function bagFunction) {
        this.bagFunction = bagFunction;
    }

    /**
     * Sets the Function to use in evaluating the elements in the apply. Parses
     * the function from an syntax scheme expression.
     * 
     * @param expression
     *            The syntax scheme of the function to set.
     * @throws FunctionTypeException
     * @throws UnknownIdentifierException
     */
    public void setFunction(String expression)
            throws UnknownIdentifierException, FunctionTypeException {
        this.function = FunctionBuilder.getInstance(expression, false);
    }

    /**
     * Sets the Function to use in evaluating the elements in the apply.
     * 
     * @param function
     *            The function to set.
     */
    public void setFunction(Function function) {
        this.function = function;
    }

    /**
     * Adds an Evaluatable to the evaluation list.
     * @param eval Adds an Evaluatable to the evaluation list.
     * @return true if the evaluatable has been added succesfully.
     * @see nbi.xmlsec.xacml.builder.EvaluatableBuilder#add(com.sun.xacml.cond.Evaluatable)
     */
    public boolean add(Evaluatable eval) {
        return super.add(eval);
    }

    /**
     * Adds an Evaluatable to the evaluation list.
     * @param evalBuilder The evaluation builder enclosing the evaluatable.
     * @return true if the evaluatable has been added succesfully.
     * @see nbi.xmlsec.xacml.builder.EvaluatableBuilder#add(nbi.xmlsec.xacml.builder.EvaluatableBuilder)
     */
    public boolean add(EvaluatableBuilder evalBuilder) {
        return super.add(evalBuilder);
    }

    /**
     * Removes an Evaluatable to the evaluation list.
     * @param eval The evaluatable to remove.
     * @return true if the evaluatable has been removed succesfully.
     * @see nbi.xmlsec.xacml.builder.EvaluatableBuilder#remove(com.sun.xacml.cond.Evaluatable)
     */
    public boolean remove(Evaluatable eval) {
        return super.remove(eval);
    }
}
