/*
 * $Id: AttributeDesignatorBuilder.java,v 1.4 2005/10/12 10:36:27 suhrbier Exp $
 * Created on 02.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: AttributeDesignatorBuilder.java,v $
 * Revision 1.4  2005/10/12 10:36:27  suhrbier
 * Target und TargetMatchBuilder completed.
 *
 * Revision 1.3  2005/10/10 18:51:43  suhrbier
 * AttributeSelector finished.
 *
 * Revision 1.2  2005/10/10 18:27:17  suhrbier
 * AttributeDesignator finished.
 *
 * Revision 1.1  2005/10/07 17:05:50  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.1  2005/02/02 17:20:09  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;

import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.cond.Evaluatable;

/**
 * Abstract base class for all {@link AttributeDesignator AttributeDesignator}
 * building classes.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

public class AttributeDesignatorBuilder extends AttributeBaseBuilder {
    /** The type of designator (Subject, Resource, Action, Environment) */
    protected int target = -1;

    /** The attribute id looked for by this designator */
    private URI id = null;

    /** The issuer of the values to search for or null if no issuer is specified */
    private URI issuer = null;
    
    protected AttributeDesignatorBuilder() {}

    /**
     * Creates an AttributeDesignerBuilder based on the type and identifier of the designator to build.
     * @param target type of the designator
     * @param id the attribute id
     */
    protected AttributeDesignatorBuilder(int target, String id) {
        this.target = target;
        setId(id);
    }

    /**
     * Createa a new AttributeDesignator instance.
     * 
     * @see EvaluatableBuilder#getInstance()
     */
    public Evaluatable getInstance() {
        return new AttributeDesignator(target, type, id, mustBePresent, issuer);
    }

    /**
     * Sets the attribute id.
     * 
     * @param id
     *            The attribute id to set.
     */
    protected void setId(URI id) {
        this.id = id;
    }

    /**
     * Sets the attribute id.
     * 
     * @param id
     *            The attribute id to set.
     */
    protected void setId(String id) {
        setId(URI.create(id));
    }

    /**
     * Sets the issuer of the values.
     * 
     * @param issuer
     *            The issuer of the values to set.
     */
    public void setIssuer(URI issuer) {
        this.issuer = issuer;
    }

    /**
     * Sets the issuer of the values.
     * 
     * @param issuer
     *            The issuer of the values to set.
     */
    public void setIssuer(String issuer) {
        this.issuer = URI.create(issuer);
    }
}
