/*
 * $Id: EvaluatableBuilder.java,v 1.3 2006/04/24 19:50:20 suhrbier Exp $
 * Created on 08.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: EvaluatableBuilder.java,v $
 * Revision 1.3  2006/04/24 19:50:20  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/10 11:20:28  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.1  2005/10/08 18:29:38  suhrbier
 * Klasse CollecctionUtil eingefhrt.
 *
 */
package nbi.xmlsec.xacml.builder;

import java.util.ArrayList;
import java.util.List;

import nbi.util.CollectionUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.cond.Evaluatable;

/**
 * Abstract base class for all evaluatable building classes.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */

abstract public class EvaluatableBuilder {
    static Logger logger = Logger.getLogger(EvaluatableBuilder.class);

    protected List evaluatables = null;

    public abstract Evaluatable getInstance();

    /**
     * Adds an Evaluatable to the evaluation list.
     * 
     * @param eval
     *            The evaluatable to add.
     * @return true if the evaluatable has been added succesfully.
     */
    public boolean add(Evaluatable eval) {
        try {
            evaluatables = (List) CollectionUtil.create(evaluatables,
                    ArrayList.class);
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        return CollectionUtil.add(evaluatables, eval);
    }

    /**
     * Adds an Evaluatable to the evaluation list.
     * 
     * @param evalBuilder
     *            The evaluation builder enclosing the evaluatable.
     * @return true if the evaluatable has been added succesfully.
     */
    protected boolean add(EvaluatableBuilder evalBuilder) {
        return add(evalBuilder.getInstance());
    }

    /**
     * Removes an Evaluatable to the evaluation list.
     * 
     * @param eval
     *            The evaluatable to remove.
     * @return true if the evaluatable has been removed succesfully.
     */
    protected boolean remove(Evaluatable eval) {
        return CollectionUtil.remove(evaluatables, eval);
    }
}
