/*
 * $Id: FunctionBuilder.java,v 1.2 2006/04/26 10:52:30 suhrbier Exp $
 * Created on 24.04.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: FunctionBuilder.java,v $
 * Revision 1.2  2006/04/26 10:52:30  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2006/04/24 19:50:20  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import com.sun.xacml.UnknownIdentifierException;
import com.sun.xacml.cond.BagFunction;
import com.sun.xacml.cond.Function;
import com.sun.xacml.cond.FunctionBase;
import com.sun.xacml.cond.FunctionTypeException;
import com.sun.xacml.cond.StandardFunctionFactory;

/**
 * Creates a {@link Function Function}instance.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */

public class FunctionBuilder {

    /* Namespace of all function names */
    public static final String FUNCTION_NAMESPACE = "urn:oasis:names:tc:xacml:1.0:function:";

    /**
     * Returns the {@link Function Function}instance.
     * 
     * @param function
     *            SyntaxScheme expression to parse.
     * @param namespace
     *            Indicates, if <I>function </I> has a namespace, otherwise
     *            {@link FunctionBase#FUNCTION_NS FunctionBase.FUNCTION_NS}is
     *            prepended.
     * @return the {@link Function Function}instance. or null.
     * @throws FunctionTypeException
     * @throws UnknownIdentifierException
     */
    static public Function getInstance(String function, boolean namespace) throws UnknownIdentifierException, FunctionTypeException {
        String functionId = namespace ? function : FunctionBase.FUNCTION_NS
                + function.toLowerCase();
        return StandardFunctionFactory.getGeneralFactory().createFunction(functionId);
    }

    static public boolean isBagFunction(String function, boolean namespace) {
        boolean result = false;
        String functionId = namespace ? function : FunctionBase.FUNCTION_NS
                + function.toLowerCase();
        result = BagFunction.getSupportedIdentifiers().contains(functionId);
        return result;
    }

    static public boolean isBagFunction(Function function) {
        return isBagFunction(function.getIdentifier().toString(),true);
    }
}
