/*
 * $Id: ObligationBuilder.java,v 1.3 2005/10/21 17:43:03 suhrbier Exp $
 * Created on 07.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ObligationBuilder.java,v $
 * Revision 1.3  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/18 18:15:55  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import nbi.xmlsec.xacml.profile.rbac.Identifier;

import com.sun.xacml.Obligation;
import com.sun.xacml.attr.AttributeValue;

/**
 * This class supports the creation of obligations.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since 
 */

public class ObligationBuilder implements XACMLObjectBuilder {
    
    /**
     * The obligation's identifier.
     */
    URI identifier = null;
    
    /** effect that will cause this obligation to be included in a response. */
    int fulfillOn;

    /** attribute assignment data in this obligation. */
    List assignments = null;

    /**
     * Sets the obligation's identifier.
     * 
     * @param id
     *            The obligation identifier to set.
     * @throws URISyntaxException @see URI#URI(java.lang.String).
     */
    public void setIdentifier(Identifier id) throws URISyntaxException {
        this.identifier = id.toURI();
    }

    Obligation getInstance()
    {
        return new Obligation(identifier, fulfillOn, assignments);
    }

    /**
     * Sets the obligation's identifier.
     * @param id
     *            The obligation identifier to set.
     */
    public void setIdentifier(URI id) {
        this.identifier = id;
    }

    /**
     * Sets the obligation's identifier.
     * @param id
     *            The obligation identifier to set.
     * @throws URISyntaxException @see URI#URI(java.lang.String).
     */
    public void setIdentifier(String id) throws URISyntaxException {
        setIdentifier(new URI(id));
    }
    
    /**
     * Adds an attribute assignment data to this obligation.
     * 
     * @param assignment
     *            The attribute assignment data to add.
     * @return true if the assignment has been added succesfully.
     */
    public void addAssignment(AttributeValue assignment) {
        if (assignments == null)
            assignments = new ArrayList();
        assignments.add(assignment);
    }

    /**
     * Removes an attribute assignment data to this obligation.
     * 
     * @param assignment
     *            The attribute assignment data to remove.
     * @return true if the assignment has been removed succesfully.
     */
    public void removeAssignment(AttributeValue assignment) {
        if (assignments != null)
            assignments.remove(assignment);
    }

    /**
     * Sets the attribute assignment data in this obligation.
     * @param assignments The assignments to set.
     */
    public void setAssignments(List assignments) {
        this.assignments = assignments;
    }
    /**
     * Sets the effect that will cause this obligation to be included in a response.
     * @param fulfillOn The fulfillOn effect to set.
     */
    public void setFulfillOn(int fulfillOn) {
        this.fulfillOn = fulfillOn;
    }
}
