/*
 * $Id: PolicyBuilder.java,v 1.6 2005/10/25 18:18:24 suhrbier Exp $
 * Created on 03.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PolicyBuilder.java,v $
 * Revision 1.6  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.5  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.4  2005/10/14 18:31:00  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.2  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.2  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.1  2005/03/07 18:22:33  suhrbier
 * Initiale Version
 *
 * Revision 1.1  2005/02/03 17:30:09  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;
import java.util.List;

import com.sun.xacml.Policy;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.Target;
import com.sun.xacml.combine.DenyOverridesRuleAlg;
import com.sun.xacml.combine.FirstApplicableRuleAlg;
import com.sun.xacml.combine.PermitOverridesRuleAlg;
import com.sun.xacml.combine.RuleCombiningAlgorithm;

/**
 * Class to create policy instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class PolicyBuilder extends AbstractPolicyBuilder {

    /**
     * Creates a policy builder.
     * 
     * @param identifier
     *            rule identifier.
     * @param combAlg Rule combining Algorithm ({@link DenyOverridesRuleAlg DenyOverridesRuleAlg}, {@link FirstApplicableRuleAlg FirstApplicableRuleAlg} or {@link PermitOverridesRuleAlg PermitOverridesRuleAlg}).
     */
    public PolicyBuilder(URI identifier, RuleCombiningAlgorithm combAlg) {
        super();
        setIdentifier(identifier);
        setCombiningAlg(combAlg);
    }

    /**
     * Creates a a new policy instance.
     * 
     * @see PolicyTreeElementBuilder#getInstance()
     */
    public PolicyTreeElement getInstance() {
        return new Policy(identifier, (RuleCombiningAlgorithm) combiningAlg,
                description, target==null?new Target(null,null,null):target, defaultVersion, treeElements, obligations);
    }

    /**
     * Adds a rule to this policy.
     * 
     * @param rule
     *            The rule to add.
     * @return true if the rule has been added succesfully.
     */
    public boolean add(Rule rule) {
        return super.add(rule);
    }

    /**
     * Adds a rule to this policy.
     * 
     * @param rule
     *            The rule element to add.
     * @return true if the rule has been added succesfully.
     */
    public boolean add(RuleBuilder rule) {
        return add(rule.getInstance());
    }

    /**
     * Removes a rule from this policy.
     * 
     * @param rule
     *            The rule to remove.
     * @return true if the rule has been removed succesfully.
     */
    public boolean remove(Rule rule) {
        return super.remove(rule);
    }

    /**
     * Sets the treeElements of this policy.
     * 
     * @param rules
     *            The rules to set.
     */
    public void setRules(List rules) {
        setTreeElements(rules);
    }
}
