/*
 * $Id: PolicyReferenceBuilder.java,v 1.2 2005/10/31 16:25:35 suhrbier Exp $
 * Created on 14.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PolicyReferenceBuilder.java,v $
 * Revision 1.2  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;

import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.finder.PolicyFinder;

/**
 * Class to create policy reference instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */

public class PolicyReferenceBuilder extends AbstractPolicyBuilder {

    /** The PolicyFinder used to handle the reference. */
    PolicyFinder finder = null;

    /**
     * The type of the policy reference (
     * {@link PolicyReference#POLICYSET_REFERENCE PolicyReference.POLICYSET_REFERENCE}
     * or
     * {@link PolicyReference#POLICY_REFERENCE PolicyReference.POLICY_REFERENCE}).
     */
    int type = -1;

    /**
     * Creates a policy reference builder.
     * 
     * @param identifier
     *            policy reference identifier.
     * @param type
     *            type of the policy reference (
     *            {@link PolicyReference#POLICYSET_REFERENCE PolicyReference.POLICYSET_REFERENCE}
     *            or
     *            {@link PolicyReference#POLICY_REFERENCE PolicyReference.POLICY_REFERENCE}).
     */
    public PolicyReferenceBuilder(URI identifier, int type) {
        super();
        setIdentifier(identifier);
        this.type = type;
    }

    /**
     * Creates a a new policy reference instance.
     * 
     * @see PolicyTreeElementBuilder#getInstance()
     */
    public PolicyTreeElement getInstance() {
        return new PolicyReference(identifier,
                this.type, finder);
    }

    /**
     * Sets the policy finder used to handle the reference.
     * 
     * @param finder
     *            The policy finder to set.
     */
    public void setFinder(PolicyFinder finder) {
        this.finder = finder;
    }
}
