/*
 * $Id: PolicySetBuilder.java,v 1.5 2005/10/25 18:18:24 suhrbier Exp $
 * Created on 03.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PolicySetBuilder.java,v $
 * Revision 1.5  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.4  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.2  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.1  2005/10/07 17:05:50  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.3  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.2  2005/03/24 20:01:18  suhrbier
 * Erzeugen von RBAC-Policies nach V2.0, Schreiben in Verzeichnisstruktur.
 * Es fehlt noch:
 * - checken des Einlesens
 * - PolicyFindetModule um die IdReferences einzulesen
 *
 * Revision 1.1  2005/03/07 18:22:33  suhrbier
 * Initiale Version
 *
 * Revision 1.2  2005/02/10 22:05:11  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/03 17:30:09  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;
import java.util.List;

import com.sun.xacml.AbstractPolicy;
import com.sun.xacml.PolicySet;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Target;
import com.sun.xacml.combine.DenyOverridesPolicyAlg;
import com.sun.xacml.combine.FirstApplicablePolicyAlg;
import com.sun.xacml.combine.OnlyOneApplicablePolicyAlg;
import com.sun.xacml.combine.PermitOverridesPolicyAlg;
import com.sun.xacml.combine.PolicyCombiningAlgorithm;

/**
 * Class to create policy set instances. 
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.5 $
 * @since
 */

public class PolicySetBuilder extends AbstractPolicyBuilder {

    /**
     * Creates a policy set builder instance.
     * 
     * @param identifier
     *            policy set identifier.
     * @param combAlg Policy combining Algorithm ({@link DenyOverridesPolicyAlg DenyOverridesPolicyAlg}, {@link FirstApplicablePolicyAlg FirstApplicablePolicyAlg}, {@link OnlyOneApplicablePolicyAlg OnlyOneApplicablePolicyAlg} or {@link PermitOverridesPolicyAlg PermitOverridesPolicyAlg}).
     */
    public PolicySetBuilder(URI identifier, PolicyCombiningAlgorithm combAlg) {
        super();
        setIdentifier(identifier);
        setCombiningAlg(combAlg);
    }

    /**
     * Createa a new policy set instance.
     * 
     * @see PolicyTreeElementBuilder#getInstance()
     */
    public PolicyTreeElement getInstance() {
        return new PolicySet(identifier,
                (PolicyCombiningAlgorithm) combiningAlg, description, target==null?new Target(null,null,null):target,
                treeElements, defaultVersion, obligations);
    }

    /**
     * Adds a policy to this policy set.
     * 
     * @param policy
     *            The policy to add.
     * @return true if the policy has been added succesfully.
     */
    public boolean add(AbstractPolicy policy) {
        return super.add(policy);
    }

    /**
     * Adds a policy to this policy set.
     * 
     * @param policy
     *            The policy to add.
     * @return true if the policy has been added succesfully.
     */
    public boolean add(AbstractPolicyBuilder policy) {
        return add(policy.getInstance());
    }

    /**
     * Adds a policy to this policy set.
     * 
     * @param policy
     *            The policy to add.
     * @return true if the obligation has been removed succesfully.
     */
    public boolean remove(AbstractPolicy policy) {
        return super.remove(policy);
    }

    /**
     * Sets the policies of this policy set.
     * 
     * @param policies
     *            The policies to set.
     */
    public void setPolicies(List policies) {
        setTreeElements(policies);
    }
}
