/*
 * $Id: RuleBuilder.java,v 1.4 2005/10/14 18:31:00 suhrbier Exp $
 * Created on 03.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RuleBuilder.java,v $
 * Revision 1.4  2005/10/14 18:31:00  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.2  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.2  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.1  2005/02/03 17:30:09  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;

import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.cond.Apply;
import com.sun.xacml.ctx.Result;

/**
 * Class to create rule instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

public class RuleBuilder extends AbstractPolicyTreeElementBuilder {

    /**
     * effect that this Rule will return from the evaluate method (Permit or
     * Deny) if the request applies.
     */
    int effect = -1;

    /** condition for this Rule or null if there is no condition. */
    Apply condition = null;

    /**
     * Creates a rule builder with required elements.
     * 
     * @param identifier
     *            rule identifier.
     * @param permit
     *            true, to set the effect to
     *            {@link Result#DECISION_PERMIT PERMIT}, false sets the effect
     *            to {@link Result#DECISION_DENY DENY}.
     */
    public RuleBuilder(URI identifier, boolean permit) {
        setIdentifier(identifier);
        setEffect(permit);
    }

    public PolicyTreeElement getInstance() {
        return new Rule(identifier, effect, description, target, condition);
    }

    /**
     * Sets the effect that this Rule will return from the evaluate method
     * (Permit or Deny) if the request applies.
     * 
     * @param permit
     *            true, to set the effect to permit, false sets the effect to
     *            deny.
     */
    public void setEffect(boolean permit) {
        if (permit)
            this.effect = Result.DECISION_PERMIT;
        else
            this.effect = Result.DECISION_DENY;
    }

    /**
     * Sets the condition for this rule
     * 
     * @param condition
     *            The condition for this rule to set.
     */
    public void setCondition(Apply condition) {
        this.condition = condition;
    }

    /**
     * Sets the condition for this rule
     * 
     * @param condition
     *            The condition for this rule to set.
     */
    public void setCondition(ConditionTypeBuilder condition) {
        this.condition = (Apply) condition.getInstance();
    }
}
