/*
 * $Id: StringResourceMatchBuilder.java,v 1.1 2005/11/01 18:45:19 suhrbier Exp $
 * Created on 13.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: StringResourceMatchBuilder.java,v $
 * Revision 1.1  2005/11/01 18:45:19  suhrbier
 * Permission upgedated.
 *
 * Revision 1.4  2005/10/15 21:37:40  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/13 16:53:18  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.attr.AttributeValue;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.cond.EqualFunction;
import com.sun.xacml.cond.MatchFunction;

/**
 * Implements an ResourceMatchBuilder for String attributes.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.1 $
 * @since
 */

public class StringResourceMatchBuilder extends ResourceMatchBuilder {

    public StringResourceMatchBuilder() {
        this(null);
    }

    /**
     * Creates the instance.
     * 
     * @param resource
     *            The String resource to match.
     * @param match
     *            <code>true</code>, selects the
     *            {@link MatchFunction#NAME_REGEXP_STRING_MATCH MatchFunction.NAME_REGEXP_STRING_MATCH}
     *            function, <code>false</code>
     *            {@link EqualFunction#NAME_STRING_EQUAL EqualFunction.NAME_STRING_EQUAL}.
     */
    public StringResourceMatchBuilder(String resource, boolean match) {
        super(match ? MatchFunction.NAME_REGEXP_STRING_MATCH
                : EqualFunction.NAME_STRING_EQUAL);
        setMatchEvaluatable((AttributeDesignator) new StringResourceAttributeDesignatorBuilder()
                .getInstance());
        setAttribute(resource);
    }

    /**
     * Creates the instance and selects the
     * {@link EqualFunction#NAME_STRING_EQUAL EqualFunction.NAME_STRING_EQUAL}.
     * 
     * @param resource
     *            The String resource to match.
     */
    public StringResourceMatchBuilder(String resource) {
        this(resource, false);
    }

    /**
     * Sets an String attribute.
     * 
     * @param attr
     *            The String attribute.
     */
    public void setAttribute(String attr) {
        super.setAttribute(attr);
    }

    /**
     * Creates the attribute match value from the attribute.
     * 
     * @return The created attribute value.
     */
    public AttributeValue createMatchValue() {
        return attribute != null ? new StringAttribute((String) attribute)
                : null;
    }
}
