/*
 * $Id: SubjectAttributeDesignatorBuilder.java,v 1.3 2005/11/02 20:10:34 suhrbier Exp $
 * Created on 02.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: SubjectAttributeDesignatorBuilder.java,v $
 * Revision 1.3  2005/11/02 20:10:34  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.2  2005/10/10 18:27:17  suhrbier
 * AttributeDesignator finished.
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.3  2005/02/07 17:37:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/02/03 17:30:09  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/02 17:20:09  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;

import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.cond.Evaluatable;

/**
 * Class to create SubjectAttributeDesignator instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */

public class SubjectAttributeDesignatorBuilder extends
        AttributeDesignatorBuilder {
    /** Identifier of the subject attribute */
    public static final String attributeId = "urn:oasis:names:tc:xacml:1.0:subject:subject-id";
    
    public static final String SUBJECT_CATEGORY = "urn:oasis:names:tc:xacml:1.0:subject:subject-category";

    public static final String SUBJECT_CATEGORY_DEFAULT_VALUE = "urn:oasis:names:tc:xacml:1.0:subject:subject-category:access-subject";
    
    public static final String SUBJECT_ID_QUALIFIER = "urn:oasis:names:tc:xacml:1.0:subject:subject-identifier-qualifier";

    public static final String SUBJECT_KEY_INFO = "urn:oasis:names:tc:xacml:1.0:subject:key-info";

    public static final String SUBJECT_AUTH_TIME = "urn:oasis:names:tc:xacml:1.0:subject:authentication-time";

    public static final String SUBJECT_AUTH_METHOD = "urn:oasis:names:tc:xacml:1.0:subject:authentication-method";

    public static final String SUBJECT_REQUEST_TIME = "urn:oasis:names:tc:xacml:1.0:subject:request-time";

    public static final String SUBJECT_SESSION_START_TIME = "urn:oasis:names:tc:xacml:1.0:subject:session-start-time";

    public static final String SUBJECT_AUTHLOCAL_IPADDRESS = "urn:oasis:names:tc:xacml:1.0:subject:authn-locality:ip-address";

    public static final String SUBJECT_AUTHLOCAL_DNSNAME = "urn:oasis:names:tc:xacml:1.0:subject:authn-locality:dns-name";

    public static final String SUBJECT_USERPASSWORD = "http://www.ietf.org/rfc/rfc2256.txt#userPassword";

    /** The category of the subject */
    private URI subjectCategory = null;

    public SubjectAttributeDesignatorBuilder(){
        super(AttributeDesignator.SUBJECT_TARGET, attributeId);
    }

    public SubjectAttributeDesignatorBuilder(String attributeId){
        super(AttributeDesignator.SUBJECT_TARGET, attributeId);
    }
    /**
     * Createa a new SubjectAttributeDesignator instance.
     * 
     * @see EvaluatableBuilder#getInstance()
     */
    public Evaluatable getInstance() {
        AttributeDesignator result = (AttributeDesignator)super.getInstance();
        result.setSubjectCategory(subjectCategory);
        return result;
    }

    /**
     * Sets the subject category.
     * @param subjectCategory The subject category to set.
     */
    public void setSubjectCategory(URI subjectCategory) {
        this.subjectCategory = subjectCategory;
    }
}
