/*
 * $Id: TargetBuilder.java,v 1.3 2005/10/24 09:38:39 suhrbier Exp $
 * Created on 02.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: TargetBuilder.java,v $
 * Revision 1.3  2005/10/24 09:38:39  suhrbier
 * Assignment test ok
 *
 * Revision 1.2  2005/10/12 10:36:27  suhrbier
 * Target und TargetMatchBuilder completed.
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.1  2005/03/07 18:22:33  suhrbier
 * Initiale Version
 *
 * Revision 1.2  2005/02/03 17:30:09  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/02 17:20:10  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.util.ArrayList;
import java.util.List;

import nbi.util.CollectionUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.Target;
import com.sun.xacml.TargetMatch;

/**
 * Class to create rule instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */
public class TargetBuilder implements XACMLObjectBuilder {
    static Logger logger = Logger.getLogger(TargetBuilder.class);

    /**
     * List containing the subjects of the Target
     */
    List subjects = null;

    /** List containing the resources of the Target */
    List resources = null;

    /** List containing the actions of the Target */
    List actions = null;

    public Target getInstance() {
        return new Target(subjects, resources, actions);
    }

    /**
     * Add a {@link TargetMatch TargetMatch} to the corresponding list.
     * The adequate list is identified on the {@link TargetMatch#getType() type} of the target match. 
     * 
     * @param match the target match to add
     * @return <code>true</code>, if the match was succesfully added, false otherwise.
     */
    public boolean add(TargetMatch match) {
        boolean result = false;
        try {
            switch (match.getType()) {
            case TargetMatch.SUBJECT:
                subjects = (List) CollectionUtil.create(subjects,
                        ArrayList.class);
                break;
            case TargetMatch.RESOURCE:
                resources = (List) CollectionUtil.create(resources,
                        ArrayList.class);
                break;
            case TargetMatch.ACTION:
                actions = (List) CollectionUtil
                        .create(actions, ArrayList.class);
                break;
            default:
                throw new IllegalArgumentException(
                        "Parameter is not a target match");
            }
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }

        List targetMatches = new ArrayList(1);
        if (CollectionUtil.add(targetMatches, match)) {
            switch (match.getType()) {
            case TargetMatch.SUBJECT:
                result = CollectionUtil.add(subjects, targetMatches);
                break;
            case TargetMatch.RESOURCE:
                result = CollectionUtil.add(resources, targetMatches);
                break;
            case TargetMatch.ACTION:
                result = CollectionUtil.add(actions, targetMatches);
                break;
            }
        }
        return result;
    }

    /**
     * Add the instance of a {@link TargetMatchBuilder TargetMatchBuilder} to the corresponding list.
     * The adequate list is identified on the {@link TargetMatch#getType() type} of the target match. 
     * 
     * @param matchBuilder the target match (builder) to add
     * @return <code>true</code>, if the match (builder) was succesfully added, false otherwise.
     */
    public boolean add(TargetMatchBuilder matchBuilder) {
        return add(matchBuilder.getInstance());
    }
}
