/*
 * $Id: TargetMatchBuilder.java,v 1.4 2005/10/15 21:37:40 suhrbier Exp $
 * Created on 02.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: TargetMatchBuilder.java,v $
 * Revision 1.4  2005/10/15 21:37:40  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/12 10:36:27  suhrbier
 * Target und TargetMatchBuilder completed.
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.1  2005/03/07 18:22:33  suhrbier
 * Initiale Version
 *
 * Revision 1.2  2005/02/02 17:33:54  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/02 17:20:10  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;

import com.sun.xacml.TargetMatch;
import com.sun.xacml.UnknownIdentifierException;
import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.attr.AttributeSelector;
import com.sun.xacml.attr.AttributeValue;
import com.sun.xacml.cond.Evaluatable;
import com.sun.xacml.cond.Function;
import com.sun.xacml.cond.FunctionFactory;
import com.sun.xacml.cond.FunctionTypeException;

/**
 * Base Class to create target matches.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

abstract public class TargetMatchBuilder {
    /**
     * Indicating whether this class represents a SubjectMatch, ResourceMatch,
     * or ActionMatch.
     */
    protected int type = -1;

    /** The Function that represents the MatchId */
    protected Function function = null;

    /**
     * The AttributeDesignator or AttributeSelector to be used to select
     * attributes from the request context
     */
    protected Evaluatable eval = null;

    /** The attribute to compare against */
    protected Object attribute=null;

    /**
     * Creates an TargetMatchBuilder based on the type target match
     * (SubjectMatch, ResourceMatch, or ActionMatch) to build.
     * 
     * @param type
     *            type of the target match.
     */
    protected TargetMatchBuilder(int type) {
        this(type,null);
    }

    /**
     * Creates an TargetMatchBuilder based on the type target match
     * (SubjectMatch, ResourceMatch, or ActionMatch) to build.
     * 
     * @param type
     *            type of the target match.
     * @param attribute
     *            attribute of the target match.
     */
    protected TargetMatchBuilder(int type, Object attribute) {
        this.type = type;
        this.attribute = attribute;
    }

    /**
     * Creates a {@link TargetMatch TargetMatch} instance.
     * @return {@link TargetMatch TargetMatch} instance.
     */
    public TargetMatch getInstance() {
        return new TargetMatch(type, function, eval, createMatchValue());
    }

    /**
     * Sets the Function used to do the matching.
     * 
     * @param functionId
     *            The name of the function.
     * @throws UnknownIdentifierException
     * @see FunctionFactory#createFunction(java.lang.String)
     * @throws FunctionTypeException
     * @see FunctionFactory#createFunction(java.lang.String)
     */
    public void setMatchFunction(String functionId)
            throws UnknownIdentifierException, FunctionTypeException {
        this.function = FunctionFactory.getTargetInstance().createFunction(
                functionId);
    }

    /**
     * Sets the Function used to do the matching.
     * 
     * @param functionId
     *            The name of the function.
     * @throws UnknownIdentifierException
     * @see FunctionFactory#createFunction(java.lang.String)
     * @throws FunctionTypeException
     * @see FunctionFactory#createFunction(java.lang.String)
     */
    public void setMatchFunction(URI functionId)
            throws UnknownIdentifierException, FunctionTypeException {
        this.function = FunctionFactory.getTargetInstance().createFunction(
                functionId);
    }

    /**
     * Sets the AttributeDesignator to be used to select attributes from the request context.
     * 
     * @param eval
     *            The AttributeDesignator to be used
     */
    public void setMatchEvaluatable(AttributeDesignator eval) {
        this.eval = eval;
    }

    /**
     * Sets the AttributeSelector to be used to select attributes from the request context.
     * 
     * @param eval
     *            The AttributeSelector to be used
     */
    public void setMatchEvaluatable(AttributeSelector eval) {
        this.eval = eval;
    }

    /**
     * Creates the attribute match value from the attribute.
     * @return The created attribute value.
     */
    abstract public AttributeValue createMatchValue();
    /**
     * Sets the attribute.
     * @param attribute The attribute to set.
     */
    protected void setAttribute(Object attribute) {
        this.attribute = attribute;
    }
}
