/*
 * $Id: X500NameSubjectMatchBuilder.java,v 1.4 2005/10/15 21:37:40 suhrbier Exp $
 * Created on 13.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: X500NameSubjectMatchBuilder.java,v $
 * Revision 1.4  2005/10/15 21:37:40  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/13 16:53:18  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import javax.security.auth.x500.X500Principal;

import com.sun.xacml.UnknownIdentifierException;
import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.attr.AttributeValue;
import com.sun.xacml.attr.X500NameAttribute;
import com.sun.xacml.cond.EqualFunction;
import com.sun.xacml.cond.FunctionFactory;
import com.sun.xacml.cond.FunctionTypeException;
import com.sun.xacml.cond.MatchFunction;

/**
 * Implements an SubjectMatchBuilder for X500Name attributes.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

public class X500NameSubjectMatchBuilder extends SubjectMatchBuilder {

    public X500NameSubjectMatchBuilder()
    {
        this(null);
    }
    
    /**
     * Creates the instance.
     * 
     * @param subject
     *            The X500Name subject to match.
     * @param match
     *            <code>true</code>, selects the
     *            {@link MatchFunction#NAME_X500NAME_MATCH MatchFunction.NAME_X500NAME_MATCH}
     *            function, <code>false</code>
     *            {@link EqualFunction#NAME_X500NAME_EQUAL EqualFunction.NAME_X500NAME_EQUAL}.
     * @throws UnknownIdentifierException
     *             see
     *             {@link FunctionFactory#createFunction(java.lang.String) FunctionFactory.createFunction(String)}.
     * @throws FunctionTypeException
     *             see
     *             {@link FunctionFactory#createFunction(java.lang.String) FunctionFactory.createFunction(String)}.
     */
    public X500NameSubjectMatchBuilder(X500Principal subject, boolean match)
{
        super(match ? MatchFunction.NAME_X500NAME_MATCH
                : EqualFunction.NAME_X500NAME_EQUAL);
        setMatchEvaluatable((AttributeDesignator) new X500NameSubjectAttributeDesignatorBuilder()
                .getInstance());
        setAttribute(subject);
    }

    /**
     * Creates the instance and selects the
     * {@link EqualFunction#NAME_X500NAME_EQUAL EqualFunction.NAME_X500NAME_EQUAL}.
     * 
     * @param subject
     *            The X500Name subject to match.
     * @throws UnknownIdentifierException
     *             see
     *             {@link FunctionFactory#createFunction(java.lang.String) FunctionFactory.createFunction(String)}.
     * @throws FunctionTypeException
     *             see
     *             {@link FunctionFactory#createFunction(java.lang.String) FunctionFactory.createFunction(String)}.
     */
    public X500NameSubjectMatchBuilder(X500Principal subject)
{
        super(EqualFunction.NAME_X500NAME_EQUAL);
        setMatchEvaluatable((AttributeDesignator) new X500NameSubjectAttributeDesignatorBuilder()
                .getInstance());
        setAttribute(subject);
    }

    /**
     * Sets an X500Principal attribute.
     * @param subject The X500Principal attribute.
     */
    public void setAttribute(X500Principal subject)
    {
        super.setAttribute(subject);
    }
    
    /**
     * Creates the attribute match value from the attribute.
     * @return The created attribute value.
     */
    public AttributeValue createMatchValue()
    {
        return attribute!=null?new X500NameAttribute((X500Principal)attribute):null;
    }
}
