/*
 * $Id: TargetMatchParserFactory.java,v 1.2 2005/10/21 17:43:03 suhrbier Exp $
 * Created on 15.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: TargetMatchParserFactory.java,v $
 * Revision 1.2  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/18 18:15:54  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.2  2005/10/15 21:37:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.parser;

import java.net.URI;
import java.util.HashMap;

import nbi.util.MapUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.TargetMatch;
import com.sun.xacml.attr.AttributeValue;
/**
 * Implements a factory for TargetMatchParser. TargetMatchParsers have to be
 * registered under the key of the related attribute id of the match value
 * {@link AttributeValue AttributeValue}. The
 * {@link #getInstance(int, TargetMatch) getInstance(int,TargetMatch)}method returns the
 * matching {@link TargetMatchParser TargetMatchParser}instance, initialised
 * with the given target match.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */

public class TargetMatchParserFactory {
    static Logger logger = Logger.getLogger(TargetMatchParserFactory.class);

    /** Registry for subject match parsers */
    HashMap subjectMatchParser;

    /** Registry for resource match parsers */
    HashMap resourceMatchParser;

    /** Registry for action match parsers */
    HashMap actionMatchParser;

    /**
     * Creates an instance of a matching
     * {@link TargetMatchParser TargetMatchParser}of the given target match.
     * 
     * @param type
     *            {@link TargetMatchParser TargetMatchParser}type (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param targetMatch
     *            The target match to create a parser for.
     * @return {@link TargetMatchParser TargetMatchParser}instance or
     *         <code>null</code>.
     */
    public TargetMatchParser getInstance(int type, TargetMatch targetMatch) {
        TargetMatchParser result = null;
        switch (type) {
        case TargetMatch.SUBJECT:
            result = (TargetMatchParser) subjectMatchParser.get(targetMatch
                    .getMatchValue().getType());
            break;
        case TargetMatch.RESOURCE:
            result = (TargetMatchParser) resourceMatchParser.get(targetMatch
                    .getMatchValue().getType());
            break;
        case TargetMatch.ACTION:
            result = (TargetMatchParser) actionMatchParser.get(targetMatch
                    .getMatchValue().getType());
            break;
        }
        if (result != null) {
            result.setTargetMatch(targetMatch);
        }
        return result;
    }

    /**
     * Registers a {@link TargetMatchParser TargetMatchParser} for a given attribute identifier.
     * @param attributeId
     *            The attribute identifier of the parser to register.
     * @param parser
     *            {@link TargetMatchParser TargetMatchParser}related to the
     *            attribute id..
     */
    public void registerSubjectMatchParser(URI attributeId, TargetMatchParser parser) {
        registerTargetMatchParser(TargetMatch.SUBJECT, attributeId, parser);
    }
    /**
     * Registers a {@link TargetMatchParser TargetMatchParser} for a given attribute identifier.
     * @param attributeId
     *            The attribute identifier of the parser to register.
     * @param parser
     *            {@link TargetMatchParser TargetMatchParser}related to the
     *            attribute id..
     */
    public void registerResourceMatchParser(URI attributeId, TargetMatchParser parser) {
        registerTargetMatchParser(TargetMatch.RESOURCE, attributeId, parser);
    }
    /**
     * Registers a {@link TargetMatchParser TargetMatchParser} for a given attribute identifier.
     * @param attributeId
     *            The attribute identifier of the parser to register.
     * @param parser
     *            {@link TargetMatchParser TargetMatchParser}related to the
     *            attribute id..
     */
    public void registerActionMatchParser(URI attributeId, TargetMatchParser parser) {
        registerTargetMatchParser(TargetMatch.ACTION, attributeId, parser);
    }
    /**
     * Registers a {@link TargetMatchParser TargetMatchParser}for a given
     * attribute id.
     * 
     * @param type
     *            {@link TargetMatchParser TargetMatchParser}type (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param attributeId
     *            The attribute identifier of the parser to register.
     * @param parser
     *            {@link TargetMatchParser TargetMatchParser}related to the
     *            attribute id..
     */
    protected void registerTargetMatchParser(int type, URI attributeId,
            TargetMatchParser parser) {
        try {
        switch (type) {
        case TargetMatch.SUBJECT:
            subjectMatchParser = (HashMap) MapUtil.create(
                    subjectMatchParser, HashMap.class);
            MapUtil.put(subjectMatchParser, attributeId, parser);
            break;
        case TargetMatch.RESOURCE:
            resourceMatchParser = (HashMap) MapUtil.create(
                    resourceMatchParser, HashMap.class);
            MapUtil.put(resourceMatchParser, attributeId, parser);
            break;
        case TargetMatch.ACTION:
            actionMatchParser = (HashMap) MapUtil.create(
                    actionMatchParser, HashMap.class);
            MapUtil.put(actionMatchParser, attributeId, parser);
            break;
        }
    } catch (InstantiationException e) {
        logger.fatal("The exception " + e + "should never happen!");
    } catch (IllegalAccessException e) {
        logger.fatal("The exception " + e + "should never happen!");
    }
        
    }

    /**
     * Unregisters a {@link TargetMatchParser TargetMatchParser} for a given attribute identifier.
     * @param attributeId
     *            The attribute identifier of the parser to unregister.
     */
    public void unregisterSubjectMatchParser(URI attributeId) {
        unregisterTargetMatchBuilder(TargetMatch.SUBJECT, attributeId);
    }
    /**
     * Unregisters a {@link TargetMatchParser TargetMatchParser} for a given attribute identifier.
     * @param attributeId
     *            The attribute identifier of the parser to unregister.
     */
    public void unregisterResourceMatchParser(URI attributeId) {
        unregisterTargetMatchBuilder(TargetMatch.RESOURCE, attributeId);
    }
    /**
     * Unregisters a {@link TargetMatchParser TargetMatchParser} for a given attribute identifier.
     * @param attributeId
     *            The attribute identifier of the parser to unregister.
     */
    public void unregisterActionMatchParser(URI attributeId) {
        unregisterTargetMatchBuilder(TargetMatch.ACTION, attributeId);
    }
    /**
     * Unregisters a {@link TargetMatchParser TargetMatchParser}of an attribute
     * id.
     * 
     * @param type
     *            {@link TargetMatchParser TargetMatchParser}type (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param attributeId
     *            The attribute identifier of the parser to remove,
     */
    protected void unregisterTargetMatchBuilder(int type, URI attributeId) {
        switch (type) {
        case TargetMatch.SUBJECT:
            MapUtil.remove(subjectMatchParser, attributeId);
            break;
        case TargetMatch.RESOURCE:
            MapUtil.remove(resourceMatchParser, attributeId);
            break;
        case TargetMatch.ACTION:
            MapUtil.remove(actionMatchParser, attributeId);
            break;
        }
    }
}
