/*
 * $Id: Identifier.java,v 1.13 2005/11/02 20:10:33 suhrbier Exp $
 * Created on 24.03.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Identifier.java,v $
 * Revision 1.13  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.12  2005/11/01 11:50:45  suhrbier
 * RoleManager add, remove and list of users completed.
 *
 * Revision 1.11  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.10  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.9  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.8  2005/10/12 19:58:08  suhrbier
 * Identifier classes revised and JavaDoc.
 *
 * Revision 1.7  2005/10/12 19:48:59  suhrbier
 * Identifier classes revised and JavaDoc.
 *
 * Revision 1.6  2005/10/12 16:47:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/08/25 20:54:51  suhrbier
 * Implementierung des RoleParameters fr ShowCase
 *
 * Revision 1.4  2005/04/01 17:42:14  suhrbier
 * + RoleManager add/remove (role)permission(s) und set DefaultPerm
 *
 * Revision 1.3  2005/03/31 18:08:53  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/03/29 15:36:52  suhrbier
 * IdRefPolicyModule implementiert.
 * Fehler in RolePolicySet.getPolicySet behoben
 *
 * Revision 1.1  2005/03/24 20:01:18  suhrbier
 * Erzeugen von RBAC-Policies nach V2.0, Schreiben in Verzeichnisstruktur.
 * Es fehlt noch:
 * - checken des Einlesens
 * - PolicyFindetModule um die IdReferences einzulesen
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.io.File;
import java.net.URI;

import org.apache.log4j.Logger;

import com.sun.xacml.ParsingException;

/**
 * Implements the XACML-identifier for RBAC elements like Assignments,
 * Permissions etc. The identifier is composed of the strings for domain, type
 * and label, where the domain is optional. The identifier may be presented as
 * URI or String as follows <domain:>type:label (ex.
 * "biocase:permission:scan-request"). In a similar way, the identifier is
 * represented in the file system. The enableFile flag marks, if the identifier
 * is allowed to be represented as a File. Otherwise, the
 * {@link Identifier#toFile(String) toFile(String)}method returns null.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.13 $
 * @since
 */

public class Identifier {
    private static final Logger logger = Logger.getLogger(Identifier.class);

    private static int iDOMAIN = 0;

    private static int iTYPE = 1;

    private static int iLABEL = 2;

    /** Domain of the identifier */
    private String domain = null;

    /** Type of the identifier */
    private String type = null;

    /** Label of the identifier */
    private String label = null;

    /**
     * Flag that marks, whether the identifier may be represented in the file
     * system ({@link Identifier#toFile(String) toFile(String)}).
     */
    private boolean enableFile = false;

    /**
     * Creates an Identifier instance.
     * 
     * @param domain
     *            Identifier's domain.
     * @param type
     *            Identifier's type.
     * @param label
     *            Identifier's label.
     * @param enableFile
     *            <code>true</code>, if the identifier may be represented in
     *            the file system, <code>false</code> otherwise.
     */
    public Identifier(String domain, String type, String label,
            boolean enableFile) {
        super();
        this.domain = domain;
        this.type = type;
        this.label = label;
        this.enableFile = enableFile;
        checkParameters();
    }

    /**
     * Identifiers should be created using
     * {@link #getInstance(URI) getInstance(URI)}only.
     */
    protected Identifier() {
        super();
    }

    private void checkParameters() throws IllegalArgumentException {
        if ((type == null) || (this.label == null))
            throw new IllegalArgumentException(
                    "fields type and label must not be null");
    }

    /**
     * Creates an Identifier instance based on its URI representation.
     * 
     * @param id
     *            Identifier's URI representation.
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public static Identifier getInstance(URI id) throws ParsingException {
        Identifier result = null;
        String[] parseResult = parse(id);
        if (parseResult[iTYPE].equals(AssignmentIdentifier.TYPEID))
            result = new AssignmentIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else if (parseResult[iTYPE].equals(RolePolicySetIdentifier.TYPEID))
            result = new RolePolicySetIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else if (parseResult[iTYPE].equals(PermissionIdentifier.TYPEID))
            result = new PermissionIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else if (parseResult[iTYPE].equals(PermissionPolicyIdentifier.TYPEID))
            result = new PermissionPolicyIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else if (parseResult[iTYPE].equals(PermissionPolicySetIdentifier.TYPEID))
            result = new PermissionPolicySetIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else if (parseResult[iTYPE].equals(RoleValueIdentifier.TYPEID))
            result = new RoleValueIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else if (parseResult[iTYPE]
                .equals(RoleAssignmentPolicySetIdentifier.TYPEID))
            result = new RoleAssignmentPolicySetIdentifier(
                    parseResult[iDOMAIN], parseResult[iLABEL]);
        else if (parseResult[iTYPE]
                .equals(RoleAssignmentPolicyIdentifier.TYPEID))
            result = new RoleAssignmentPolicyIdentifier(parseResult[iDOMAIN],
                    parseResult[iLABEL]);
        else
            throw new ParsingException("Unknown identifier type "+parseResult[iTYPE]);
        return result;
    }

    /**
     * Creates an Identifier instance based on its string representation.
     * 
     * @param id
     *            Identifier's URI representation.
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public static Identifier getInstance(String id) throws ParsingException {
        return getInstance(URI.create(id));
    }

    /**
     * Parses domain, type and label from an Identifier string representation.
     * 
     * @param id
     *            Identifier string representation to parse.
     * @return Array with parsed domain(may be <code>null</code>), type and
     *         label.
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public static String[] parse(String id) throws ParsingException {
        String[] result = new String[3];

        int iLabel = id.lastIndexOf(':');
        result[iLABEL] = id.substring(iLabel + 1);
        int iType = id.lastIndexOf(':', iLabel - 1);
        result[iTYPE] = id.substring(iType + 1, iLabel);
        if (iType > 0)
            result[iDOMAIN] = id.substring(0, iType);
        else
            result[iDOMAIN] = null;
        return result;
    }

    /**
     * Parses domain, type and label from an Identifier string representation.
     * 
     * @param id
     *            Identifier string representation to parse.
     * @return Array with parsed domain(may be <code>null</code>), type and
     *         label.
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public static String[] parse(URI id) throws ParsingException {
        return parse(id.toString());
    }

    /**
     * Sets the domain.
     * 
     * @param domain
     *            The domain to set.
     */
    public void setDomain(String domain) {
        this.domain = domain;
    }

    /**
     * Gets the domain.
     * 
     * @return The domain.
     */
    public String getDomain() {
        return this.domain;
    }

    /**
     * Sets the type.
     * 
     * @param type
     *            The type to set.
     */
    protected void setType(String type) {
        this.type = type;
    }

    /**
     * Gets the type.
     * 
     * @return The type.
     */
    public String getType() {
        return this.type;
    }

    /**
     * Checks wether the identifier is file enabled.
     * 
     * @return <code>true</code>, identifier is file enabled,
     *         <code>false</code> otherwise.
     */
    public boolean isEnableFile() {
        return enableFile;
    }

    /**
     * Sets the label.
     * 
     * @param label
     *            The label to set.
     */
    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * Gets the label.
     * 
     * @return The label.
     */
    public String getLabel() {
        return this.label;
    }

    /**
     * Creates the representation of this identifier.
     * 
     * @return The identifier's representation.
     */
    public URI toURI() {
        return URI.create(toString());
    }

    /**
     * Creates the representation of this identifier.
     * 
     * @return The identifier's representation.
     */
    public String toString() {
        checkParameters();
        StringBuffer id = new StringBuffer();
        if (domain != null)
            id.append(this.domain + ":");
        if (type != null)
            id.append(type + ":");
        if (label != null)
            id.append(this.label);
        return id.toString();
    }

    /**
     * Creates a file system representation of this identifier. The
     * representation is only created, when the identifier is file enabled.
     * 
     * @param parent
     *            Parent directory for the file representation.
     * @return File representation if the identifier is file enabled,
     *         <code>null</code> otherwise.
     */
    public File toFile(String parent) {
        return toFile(new File(parent));
    }

    /**
     * Creates a file system representation of this identifier. The
     * representation is only created, when the identifier is file enabled.
     * 
     * @param parent
     *            Parent directory for the file representation.
     * @return File representation if the identifier is file enabled,
     *         <code>null</code> otherwise.
     */
    public File toFile(File parent) {
        File ret = null;
        if (enableFile) {
            checkParameters();
            StringBuffer file = new StringBuffer();
            if (domain != null)
                file.append(this.domain + File.separator);
            if (type != null)
                file.append(type + File.separator);
            if (label != null)
                file.append(this.label);
            file.append(".xml");
            ret = new File(parent, file.toString());
        }
        return ret;
    }

    /**
     * Returns true, if the String representations are identical.
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        boolean ret = false;
        if (obj instanceof Identifier)
            ret = toString().equals(((Identifier) obj).toString());
        return ret;
    }

    /**
     * Returns hash value of the string representation.
     * 
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return toString().hashCode();
    }

}
