/*
 * $Id: Permission.java,v 1.10 2006/05/30 19:05:35 suhrbier Exp $
 * Created on 31.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: Permission.java,v $
 * Revision 1.10  2006/05/30 19:05:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.9  2006/04/27 18:22:22  suhrbier
 * Condition im RoleManager eingebaut.
 *
 * Revision 1.8  2005/12/07 18:08:13  suhrbier
 * Fehler beim Parsen von PermitRuleAlgs in RBACPolicy behoben.
 *
 * Revision 1.7  2005/12/06 17:12:15  suhrbier
 * list Befehl implementiert und getestet.
 * Fehler bei Domains der auf den DirectoryFileFitler von commons.io zurckgeht.
 *
 * Revision 1.6  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.5  2005/11/01 18:45:19  suhrbier
 * Permission upgedated.
 *
 * Revision 1.4  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/03/30 14:51:32  suhrbier
 * RoleAssignmentPolicy fertiggestellt.
 *
 * Revision 1.2  2005/03/24 20:01:18  suhrbier
 * Erzeugen von RBAC-Policies nach V2.0, Schreiben in Verzeichnisstruktur.
 * Es fehlt noch:
 * - checken des Einlesens
 * - PolicyFindetModule um die IdReferences einzulesen
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.10  2005/03/01 12:43:55  suhrbier
 * Nach Urlaub
 *
 * Revision 1.9  2005/02/10 12:33:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.8  2005/02/07 17:37:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.7  2005/02/05 17:12:48  suhrbier
 * *** empty log message ***
 *
 * Revision 1.6  2005/02/03 18:42:09  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/02/03 17:30:10  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/02/02 17:39:23  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/02/02 17:33:54  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/02/02 17:20:10  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/01/31 18:01:35  suhrbier
 * RBAC Policy Writer
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;
import java.util.Set;

import nbi.util.StringUtil;
import nbi.xmlsec.xacml.builder.ConditionTypeBuilder;
import nbi.xmlsec.xacml.builder.RuleBuilder;
import nbi.xmlsec.xacml.profile.rbac.builder.PermissionTargetMatchBuilderFactory;
import nbi.xmlsec.xacml.profile.rbac.parser.PermissionTargetMatchParserFactory;

import org.apache.log4j.Logger;

import com.sun.xacml.ParsingException;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.cond.Apply;
import com.sun.xacml.ctx.Result;

/**
 * This class implements a permission rule according to the XACML RBAC Profile.
 * It assigns permissions to access resources for given actions. The permission
 * may be represented as a {@link Rule Rule}or parsed from a {@link Rule Rule}
 * representation.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.10 $
 * @since
 */

public class Permission extends RBACPolicyElement {
    static Logger logger = Logger.getLogger(Permission.class);

    /** The effect of the permission rule (permit or deny) */
    boolean permit = false;

    /** The condition of the permission rule or null */
    Apply condition = null;
    

    /**
     * Create Permission instance.
     * 
     * @param domain
     *            Domain of the permission identifier.
     * @param label
     *            Label of the permission identifier.
     * @param permit
     *            <code>true</code> sets the effect to
     *            {@link Result#DECISION_PERMIT DECISION_PERMIT},
     *            <code>false</code> to
     *            {@link Result#DECISION_DENY DECISION_DENY}.
     */
    public Permission(String domain, String label, boolean permit) {
        setIdentifier(new PermissionIdentifier(domain, label));
        setPermit(permit);
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));

    }

    /**
     * Create an Permission instance.
     * 
     * @param permId
     *            Assignment identifier (domain:assignment:label).
     * @param permit
     *            <code>true</code> sets the effect to
     *            {@link Result#DECISION_PERMIT DECISION_PERMIT},
     *            <code>false</code> to
     *            {@link Result#DECISION_DENY DECISION_DENY}.
     */
    public Permission(URI permId, boolean permit) throws ParsingException {
        setIdentifier(permId);
        setPermit(permit);
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
    }

    /**
     * Create a Permission instance from its {@link Rule Rule}representation.
     * 
     * @param rule
     *            {@link Rule Rule}representation.
     */
    public Permission(Rule rule) throws ParsingException {
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
        parse(rule);
    }

    /**
     * Adds an String resource to the resource set.
     * 
     * @param resource
     *            The resource to add.
     * @return <code>true</code> if the resource has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addResource(String resource) {
        return getTarget().addResource(resource);
    }

    /**
     * Removes an String resource from the resource set.
     * 
     * @param resource
     *            The resource to remove.
     * @return <code>true</code> if the resource has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean removeResource(String resource) {
        return getTarget().removeResource(resource);
    }

    /**
     * Adds an String action to the resource set.
     * 
     * @param action
     *            The action to add.
     * @return <code>true</code> if the action has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addAction(String action) {
        return getTarget().addAction(action);
    }

    /**
     * Removes an String action from the action set.
     * 
     * @param action
     *            The action to remove.
     * @return <code>true</code> if the action has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean removeAction(String action) {
        return getTarget().removeAction(action);
    }

    /**
     * Adds the resources and actions of another Permission from the resources
     * and actions of this permission respectively.
     * 
     * @param element
     *            The permission to add.
     * @return <code>true</code> if the any of the permission target objects
     *         has been added succesfully, <code>false</code> otherwise.
     * @see RBACPolicyElement#add(RBACPolicyElement)
     */
    public boolean add(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof Permission)
            result = getTarget().add(element.getTarget());
        return result;
    }

    /**
     * Removes the resources and actions of another Permission from the
     * resources and actions of this permission respectively.
     * 
     * @param element
     *            The permission to remove.
     * @return <code>true</code> if the any of the permission target objects
     *         has been removed succesfully, <code>false</code> otherwise.
     * @see RBACPolicyElement#remove(RBACPolicyElement)
     */
    public boolean remove(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof Permission)
            result = getTarget().remove(element.getTarget());
        return result;
    }

    /**
     * Returns the {@link Rule Rule}representation of the permission.
     * 
     * @return Permission's rule representation.
     */
    public PolicyTreeElement toPolicyTreeElement() {
        RuleBuilder ruleBuilder = new RuleBuilder(getIdentifier().toURI(),
                isPermit());
        ruleBuilder.setCondition(condition);
        ruleBuilder.setTarget(getTarget().toTarget());
        return (Rule) ruleBuilder.getInstance();
    }

    /**
     * Parses an permission from its {@link Rule Rule}representation.
     * 
     * @param element
     *            The rule to parse.
     * @throws ParsingException
     *             Indicates a malformed {@link Rule Rule}representation.
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        if (element == null)
            throw new ParsingException("Tree element is null");

        Rule rule = null;
        if (element instanceof Rule)
            rule = (Rule) element;
        setIdentifier(rule.getId());
        if (rule.getEffect() == Result.DECISION_PERMIT)
            permit = true;
        else
            permit = false;
        setCondition(rule.getCondition());
        getTarget().parse(rule.getTarget());
    }

    /**
     * Gets the permit effect.
     * 
     * @return <code>true</code> if the effect is set to
     *         {@link Result#DECISION_PERMIT DECISION_PERMIT},
     *         <code>false</code> if the effect is set to
     *         {@link Result#DECISION_DENY DECISION_DENY}.
     */
    public boolean isPermit() {
        return permit;
    }

    /**
     * Sets the permit effect.
     * 
     * @param permit
     *            <code>true</code> sets the effect to
     *            {@link Result#DECISION_PERMIT DECISION_PERMIT},
     *            <code>false</code> sets the effect to
     *            {@link Result#DECISION_DENY DECISION_DENY}.
     */
    public void setPermit(boolean permit) {
        this.permit = permit;
    }

    /**
     * Sets the condition.
     * 
     * @param condition The condition
     */
    public void setCondition(Apply condition) {
        this.condition = condition;
    }

    /**
     * Sets the condition.
     * 
     * @param conditionBuilder The condition
     */
    public void setCondition(ConditionTypeBuilder conditionBuilder) {
        setCondition((Apply)conditionBuilder.getInstance());
    }

    /**
     * Gets the condition.
     * 
     * @return The condition
     */
    public Apply getCondition() {
        return this.condition;
    }

    /**
     * Gets the assigned resources of this this permission.
     * 
     * @return The permission's resources.
     */
    public Set getResources() {
        return getTarget().getResources();
    }

    /**
     * Gets the assigned actions of this this permission.
     * 
     * @return The permission's actions.
     */
    public Set getActions() {
        return getTarget().getActions();
    }

    /**
     * Compares the specified object with this permission. Returns true if the
     * specified object is also an Permission and the underlying policy element
     * is also equal.
     * 
     * @param obj
     *            Object to be compared for equality with this permission.
     * @return <code>true</code> if the specified Object is equal to this
     *         permission.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj instanceof Permission) {
            result = super.equals(obj);
        }
        return result;
    }

    public String toString() {
        StringBuffer result = new StringBuffer();
        result.append("Permission ");
        result.append(getIdentifier().getLabel());
        result.append(" (");
        result.append(isPermit() ? "Permit" : "Deny");
        result.append(")");
        result.append(" has ");
        result.append(StringUtil.lineSeparator);
        result.append(getTarget().toString());
        return result.toString();
    }
}
