/*
 * $Id: PermissionPolicy.java,v 1.6 2005/12/07 18:08:13 suhrbier Exp $
 * Created on 28.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PermissionPolicy.java,v $
 * Revision 1.6  2005/12/07 18:08:13  suhrbier
 * Fehler beim Parsen von PermitRuleAlgs in RBACPolicy behoben.
 *
 * Revision 1.5  2005/12/06 17:12:15  suhrbier
 * list Befehl implementiert und getestet.
 * Fehler bei Domains der auf den DirectoryFileFitler von commons.io zurckgeht.
 *
 * Revision 1.4  2005/12/05 19:04:04  suhrbier
 * RoleManager tested, but list command.
 *
 * Revision 1.3  2005/11/04 17:52:36  suhrbier
 * Permission add mit Targets funzt.
 *
 * Revision 1.2  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.1  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.7  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.6  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.5  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.4  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/18 18:15:54  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.2  2005/10/15 18:00:50  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/04/01 22:01:37  suhrbier
 * + RoleManager Assignment Handling fertig.
 *
 * Revision 1.4  2005/04/01 18:22:21  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/03/31 18:08:53  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/03/30 14:51:32  suhrbier
 * RoleAssignmentPolicy fertiggestellt.
 *
 * Revision 1.1  2005/03/29 17:47:51  suhrbier
 * last edit: Assignment
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.5  2005/02/10 12:33:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/02/08 17:51:31  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/02/08 12:17:15  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/31 18:01:35  suhrbier
 * RBAC Policy Writer
 *
 * Revision 1.1  2005/01/28 15:51:19  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;
import java.util.Iterator;
import java.util.Set;

import nbi.util.StringUtil;
import nbi.xmlsec.xacml.profile.rbac.builder.PermissionTargetMatchBuilderFactory;
import nbi.xmlsec.xacml.profile.rbac.parser.PermissionTargetMatchParserFactory;
import nbi.xmlsec.xacml.util.TargetUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Target;
import com.sun.xacml.TargetMatch;

/**
 * This class implements a permission policy according to the XACML RBAC
 * Profile. It manages permissions, which may be represented as a
 * {@link Policy Policy}or parsed from a {@link Policy Policy}representation.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class PermissionPolicy extends RBACPolicy {

    static Logger logger = Logger.getLogger(PermissionPolicy.class);

    /**
     * Creates permission policy with its identifier.
     * 
     * @param domain
     *            Identifier's domain.
     * @param label
     *            Identifier's label.
     */
    public PermissionPolicy(String domain, String label) {
        super(new PermissionPolicyIdentifier(domain, label));
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
    }

    /**
     * Creates permission policy with its identifier.
     * 
     * @param id
     *            Identifier's URI representation.
     */
    public PermissionPolicy(URI id) throws ParsingException {
        super(Identifier.getInstance(id));
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
    }

    /**
     * Create an permission policy instance from its {@link Policy Policy}
     * representation.
     * 
     * @param policy
     *            {@link Policy Policy}representation.
     */
    public PermissionPolicy(Policy policy) throws ParsingException {
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
        parse(policy);
    }

    /**
     * Create an permission policy instance from its
     * {@link PolicyReference PolicyPolicyReference}representation.
     * 
     * @param policy
     *            {@link Policy Policy}representation.
     */
    public PermissionPolicy(PolicyReference policy) throws ParsingException {
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
        parse(policy);
    }

    /**
     * Parses the target of an PermissionPolicySet.
     * 
     * @see nbi.xmlsec.xacml.profile.rbac.RBACPolicySet#parse(com.sun.xacml.Target)
     */
    public void parse(Target target) throws ParsingException {
        if (TargetUtil.getTargetMatchSize(target, TargetMatch.SUBJECT) > 0)
            throw new ParsingException(
                    "Target subject of the PermissionPolicySet "
                            + getIdentifier().toString() + " must be empty.");
        getTarget().parse(target);
    }

    /**
     * Parses a role assignment policy from its {@link Policy Policy}
     * representation.
     * 
     * @param element
     *            The policy to parse.
     * @throws ParsingException
     *             Indicates a malformed {@link Policy Policy}representation.
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        super.parse(element, Permission.class);
        /*
         * if (element == null) throw new ParsingException("Tree element is
         * null");
         * 
         * if (element instanceof Policy || element instanceof PolicyReference) {
         * AbstractPolicy policy = (AbstractPolicy) element;
         * setIdentifier(policy.getId()); if
         * (!TargetUtil.isEmpty(policy.getTarget())) throw new ParsingException(
         * "Target of the RoleAssignmentPolicy " + getIdentifier().toString() + "
         * must be empty."); Iterator ruleIt = policy.getChildren().iterator();
         * while (ruleIt.hasNext()) { add(new Permission((Rule) ruleIt.next())); } }
         * else { throw new ParsingException("Unsupported tree element class:" +
         * element.getClass().getName()); }
         */
    }

    /**
     * Returns the {@link Policy Policy}representation of the role assignment
     * policy.
     * 
     * @return The {@link Policy Policy}representation.
     */
    public PolicyTreeElement toPolicyTreeElement() {
        return super.toPolicyTreeElement();
        /*
         * PolicyBuilder policyBuilder = new PolicyBuilder(
         * getIdentifier().toURI(), new PermitOverridesRuleAlg());
         * policyBuilder.setTarget(getTarget().toTarget()); if (elements !=
         * null) { Iterator it = elements.iterator(); while (it.hasNext())
         * policyBuilder.add((Rule) ((Permission) it.next())
         * .toPolicyTreeElement()); } return (Policy)
         * policyBuilder.getInstance();
         */
    }

    /**
     * Returns the {@link PolicyReference PolicyRefence}to this the role
     * assignment policy.
     * 
     * @return The {@link PolicyReference PolicyReference}.
     */
    public PolicyReference toPolicyReference() {
        return super.toPolicyRerence(PolicyReference.POLICY_REFERENCE);
    }

    /**
     * If the element is an {@link Permission Permission}, the element is added
     * to the set of elements.
     * 
     * @return <code>true</code>, if the permission was succesfully added,
     *         <code>false</code> otherwise.
     * 
     * @see RBACPolicy#add(RBACPolicyElement)
     */
    public boolean add(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof Permission)
            result = super.add(element);
        return result;
    }

    /**
     * If the element is an {@link Permission Permission}, the element is
     * removed from the set of elements.
     * 
     * @return <code>true</code>, if the permission was succesfully removed,
     *         <code>false</code> otherwise.
     * 
     * @see RBACPolicy#remove(RBACPolicyElement)
     */
    public boolean remove(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof Permission)
            result = elements.remove(element);
        return result;
    }

    /**
     * Searches a Permission within this PermissionPolicy. The domain of the
     * PermissionPolicy is used for searching.
     * 
     * @param permLabel
     *            Label of the Permission to search for
     * @return The Permission with the given label, or null.
     */
    public Permission searchPermission(String permLabel) {
        return (Permission) search(new PermissionIdentifier(getIdentifier()
                .getDomain(), permLabel));
    }

    /**
     * Gets the permissions of this role.
     * 
     * @return Set of permissions..
     */
    public Set getPermissions() {
        return elements;
    }

    public String toString() {
        StringBuffer result = new StringBuffer();
        result.append("PermissionPolicy ");
        result.append(getIdentifier().getLabel());
        result.append(" has ");
        
        if (getPermissions() == null)
        {
            result.append("no Permissions");
            result.append(" (");
            result.append(isPermitRule() ? "PermitOverrides" : "DenyOverrides");
            result.append(")");
            int len = result.length() +5;
            for (int i = 0; i < len; i++)
                result.append("=");
            result.append(StringUtil.lineSeparator);
        }
        else {
            result.append(getPermissions().size());
            result.append(" Permissions");
            result.append(" (");
            result.append(isPermitRule() ? "PermitOverrides" : "DenyOverrides");
            result.append(")");
            result.append(StringUtil.lineSeparator);
            int len = result.length() +5;
            for (int i = 0; i < len; i++)
                result.append("=");
            result.append(StringUtil.lineSeparator);
            Iterator it = getPermissions().iterator();
            while (it.hasNext()) {
                result.append(it.next().toString());
                result.append(StringUtil.lineSeparator);
            }
        }
        return result.toString();
    }
}
