/*
 * $Id: PermissionPolicySet.java,v 1.2 2005/11/03 21:15:03 suhrbier Exp $
 * Created on 05.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PermissionPolicySet.java,v $
 * Revision 1.2  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.1  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.11  2005/10/14 18:30:58  suhrbier
 * *** empty log message ***
 *
 * Revision 1.10  2005/08/20 18:52:04  suhrbier
 * Umstellung auf Target Filterung in PermissionPolicySet
 *
 * Revision 1.9  2005/07/11 10:35:07  suhrbier
 * ReponseFilter tests addeds
 *
 * Revision 1.8  2005/04/01 22:01:37  suhrbier
 * + RoleManager Assignment Handling fertig.
 *
 * Revision 1.7  2005/04/01 17:42:14  suhrbier
 * + RoleManager add/remove (role)permission(s) und set DefaultPerm
 *
 * Revision 1.6  2005/03/31 18:08:53  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/03/30 14:51:31  suhrbier
 * RoleAssignmentPolicy fertiggestellt.
 *
 * Revision 1.4  2005/03/29 17:47:51  suhrbier
 * last edit: Assignment
 *
 * Revision 1.3  2005/03/24 20:01:18  suhrbier
 * Erzeugen von RBAC-Policies nach V2.0, Schreiben in Verzeichnisstruktur.
 * Es fehlt noch:
 * - checken des Einlesens
 * - PolicyFindetModule um die IdReferences einzulesen
 *
 * Revision 1.2  2005/03/08 18:42:37  suhrbier
 * Default Permit korrigiert. Wird jetzt immer am Ende der Rules hinzugefgt.
 * Permit bzw. Deny-Overrides Algorithmus wird angepasst.
 *
 * Revision 1.1  2005/03/07 18:22:31  suhrbier
 * Initiale Version
 *
 * Revision 1.6  2005/03/01 12:43:55  suhrbier
 * Nach Urlaub
 *
 * Revision 1.5  2005/02/10 12:33:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/02/08 17:51:31  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/02/08 12:17:15  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/02/07 17:37:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/05 17:12:48  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;

import nbi.xmlsec.xacml.profile.rbac.builder.PermissionTargetMatchBuilderFactory;
import nbi.xmlsec.xacml.profile.rbac.parser.PermissionTargetMatchParserFactory;
import nbi.xmlsec.xacml.util.TargetUtil;

import com.sun.xacml.ParsingException;
import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicySet;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Target;
import com.sun.xacml.TargetMatch;
import com.sun.xacml.ctx.Result;

/**
 * This class implements a permission policy set according to the XACML RBAC
 * Profile. It administers permission policies, which may be represented as a
 * {@link PolicySet PolicySet}or parsed from a {@link PolicySet PolicySet}
 * representation.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */

public class PermissionPolicySet extends RBACPolicySet {
    static final int DEFAULT_PERMIT_NONE = -1;

    static final int DEFAULT_PERMIT_PERMIT = Result.DECISION_PERMIT;

    static final int DEFAULT_PERMIT_DENY = Result.DECISION_DENY;

    static final int DEFAULT_PERMIT_NOT_APPLICABLE = Result.DECISION_NOT_APPLICABLE;

    static final int DEFAULT_PERMIT_INDETERMINATE = Result.DECISION_INDETERMINATE;

    int defaultPermit = DEFAULT_PERMIT_NONE;

    /**
     * Creates a permission policy set from its identifier. The default policy
     * representation type is set to
     * {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICYREF}.
     * 
     * @param domain
     *            Identifier's domain.
     * @param label
     *            Identifier's label.
     */
    public PermissionPolicySet(String domain, String label) {
        super(new PermissionPolicySetIdentifier(domain, label),
                REPRESENT_POLICIES_AS_POLICYREF);
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
    }

    /**
     * Creates a permission policy set from its identifier. The default policy
     * representation type is set to
     * {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICYREF}.
     * 
     * @param id
     *            Identifier's URI representation.
     */
    public PermissionPolicySet(URI id) throws ParsingException {
        super(Identifier.getInstance(id), REPRESENT_POLICIES_AS_POLICYREF);
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
    }

    /**
     * Creates a permission policy set from its {@link PolicySet PolicySet}
     * representation. The first policy parsed determines the policy
     * representation type of the policy set.
     * 
     * @param policySet
     *            {@link PolicySet PolicySet}representation.
     */
    public PermissionPolicySet(PolicySet policySet) throws ParsingException {
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
        parse(policySet);
    }

    /**
     * Creates a permission policy set from its {@link PolicyReference PolicyReference}
     * representation. The first policy parsed determines the policy
     * representation type of the policy set.
     * 
     * @param policySet
     *            {@link PolicySet PolicySet}representation.
     */
    public PermissionPolicySet(PolicyReference policySet) throws ParsingException {
        setTarget(new RBACTarget(PermissionTargetMatchBuilderFactory
                .getInstance(), PermissionTargetMatchParserFactory
                .getInstance()));
        parse(policySet);
    }

    /**
     * If the element is an {@link PermissionPolicy PermissionPolicy}, the
     * element is added to the set of elements.
     * 
     * @return <code>true</code>, if the permission policy was succesfully
     *         added, <code>false</code> otherwise.
     * 
     * @see RBACPolicy#add(RBACPolicyElement)
     */
    public boolean add(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof PermissionPolicy)
            result = super.add(element);
        return result;
    }

    /**
     * If the element is an {@link PermissionPolicy PermissionPolicy}, the
     * element is removed from the set of elements.
     * 
     * @return <code>true</code>, if the permission policy was succesfully
     *         removed, <code>false</code> otherwise.
     * 
     * @see RBACPolicy#remove(RBACPolicyElement)
     */
    public boolean remove(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof PermissionPolicy)
            result = super.remove(element);
        return result;
    }

    /**
     * Parses the target of an PermissionPolicySet.
     * 
     * @see nbi.xmlsec.xacml.profile.rbac.RBACPolicySet#parse(com.sun.xacml.Target)
     */
    public void parse(Target target) throws ParsingException {
        if (TargetUtil.getTargetMatchSize(target, TargetMatch.SUBJECT) > 0)
            throw new ParsingException(
                    "Target subject of the PermissionPolicySet "
                            + getIdentifier().toString() + " must be empty.");
        getTarget().parse(target);
    }

    /**
     * Parses a permission policy set from its {@link PolicySet PolicySet}
     * representation. The first policy parsed determines the policy
     * representation type of the policy set.
     * 
     * @param element
     *            The policy set to parse.
     * @throws ParsingException
     *             Indicates errors during the parsing process.
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        super.parse(element, PermissionPolicy.class, PermissionPolicy.class,
                null);
    }
}
