/*
 * $Id: PolicyManager.java,v 1.22 2006/07/24 17:31:10 suhrbier Exp $
 * Created on 30.03.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PolicyManager.java,v $
 * Revision 1.22  2006/07/24 17:31:10  suhrbier
 * update JavaDoc
 *
 * Revision 1.21  2005/12/14 17:57:09  suhrbier
 * JavaDoc korriguert.
 *
 * Revision 1.20  2005/12/14 15:34:30  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.19  2005/12/09 11:01:28  suhrbier
 * 2 special PolicyFinder fr REA und PDP
 *
 * Revision 1.18  2005/12/06 17:12:15  suhrbier
 * list Befehl implementiert und getestet.
 * Fehler bei Domains der auf den DirectoryFileFitler von commons.io zurckgeht.
 *
 * Revision 1.17  2005/12/05 19:04:04  suhrbier
 * RoleManager tested, but list command.
 *
 * Revision 1.16  2005/11/07 20:21:36  suhrbier
 * *** empty log message ***
 *
 * Revision 1.15  2005/11/07 10:59:41  suhrbier
 * *** empty log message ***
 *
 * Revision 1.14  2005/11/02 20:10:32  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.13  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.12  2005/10/28 23:36:11  suhrbier
 * RoleManager begonnen mit add.
 *
 * Revision 1.11  2005/10/28 10:03:33  suhrbier
 * PolicyManager completed bis auf Comment.
 *
 * Revision 1.10  2005/10/27 18:26:05  suhrbier
 * RBACPolicyFinderModule completed.
 *
 * Revision 1.9  2005/10/26 15:20:32  suhrbier
 * *** empty log message ***
 *
 * Revision 1.8  2005/10/14 18:30:58  suhrbier
 * *** empty log message ***
 *
 * Revision 1.7  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 * Revision 1.6  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.5  2005/04/05 16:46:17  suhrbier
 * CVS-Verzeichnisse aus listDomains ausgefiltert
 *
 * Revision 1.4  2005/04/04 17:20:56  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/04/01 17:42:14  suhrbier
 * + RoleManager add/remove (role)permission(s) und set DefaultPerm
 *
 * Revision 1.2  2005/03/31 18:08:53  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/03/31 15:19:52  suhrbier
 * + PolicyManager list-Funktionen
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import nbi.util.FileUtil;
import nbi.util.ReflectionUtil;
import nbi.util.StringUtil;
import nbi.xmlsec.configuration.util.ConfigurationUtil;
import nbi.xmlsec.xacml.profile.rbac.finder.RBACPolicyFinderModule;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConversionException;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.AndFileFilter;
import org.apache.commons.io.filefilter.DirectoryFileFilter;
import org.apache.commons.io.filefilter.IOFileFilter;
import org.apache.commons.io.filefilter.NameFileFilter;
import org.apache.commons.io.filefilter.NotFileFilter;
import org.apache.commons.io.filefilter.SuffixFileFilter;
import org.apache.commons.io.filefilter.TrueFileFilter;
import org.apache.log4j.Logger;

import com.sun.xacml.AbstractPolicy;
import com.sun.xacml.Indenter;
import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicySet;
import com.sun.xacml.finder.PolicyFinder;
import com.sun.xacml.finder.PolicyFinderModule;
import com.sun.xacml.finder.PolicyFinderResult;

/**
 * This class is the main instance for policy management purposes. Starting on a
 * policy base dir, it searches, writes, removes and checks the existance of
 * policies using Identifiers, and retrieves Domains. The policy manager has to
 * be configured using a PolicyManager part of the configuration file.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.22 $
 * @since
 */

public class PolicyManager {
    static Logger logger = Logger.getLogger(PolicyManager.class);

    /** The PolicyManager's configuration */
    private Configuration configuration;

    /** The PolicyManager's policy finder */
    PolicyFinder policyFinder;

    /**
     * Inits the PolicyManager based on its configuration subset
     * (PolicyManager). Depending on the configuration this constructor sets up
     * the PolicyFinder using the following parts of the configuration
     * <li>PolicyFinderModules</li>,
     * <li>policy directories</li>
     * <li>policy file filters</li>.
     * 
     * @param config
     *            RoleManager's configuration.
     * @throws Exception
     */
    public PolicyManager(Configuration config) throws Exception {
        if (config == null)
            throw new NullPointerException("configuration may not be null");
        configuration = config;
        policyFinder = (PolicyFinder) ReflectionUtil
                .getInstance(config.getString("PolicyFinder[@class]",
                        PolicyFinder.class.getName()));
        List finderModuleConfig = ConfigurationUtil.getSubsets(config,
                "PolicyFinder.PolicyFinderModule");
        if (finderModuleConfig.isEmpty())
            throw new ConversionException(
                    "No policy finder modules configured!");
        Iterator fmIt = finderModuleConfig.iterator();

        HashSet finderModules = new HashSet();
        while (fmIt.hasNext()) {
            Configuration fmConfig = (Configuration) fmIt.next();
            if (!fmConfig.containsKey("[@policyBaseDir]"))
                fmConfig.addProperty("[@policyBaseDir]", getPolicyBaseDir());
            finderModules.add((PolicyFinderModule) ReflectionUtil.getInstance(
                    fmConfig.getString("[@class]"), Configuration.class,
                    fmConfig));
            logger.debug("PolicyFinderModule " + fmConfig.getString("[@class]")
                    + " addded.");
        }
        policyFinder.setModules(finderModules);
        // Done by the PDP init policyFinder.init();
    }

    /**
     * Write a policy to a file. The file location is derived from the policy
     * base directory and the path retrieved from the policy's identifier.
     * 
     * @param policy
     *            The policy to write.
     * @throws ParsingException
     *             Indicates a malformed policy identifier.
     * @throws IOException
     *             If an I/O-problem occurs.
     */
    public void write(AbstractPolicy policy) throws ParsingException,
            IOException {
        if (policy == null)
            throw new NullPointerException("policy must not be null");
        Identifier polId = Identifier.getInstance(policy.getId());
        File policyFile = polId.toFile(getPolicyBaseDir());
        new File(policyFile.getParent()).mkdirs();
        FileOutputStream fOut = new FileOutputStream(policyFile);
        policy.encode(fOut, new Indenter());
        fOut.flush();
        fOut.close();
    }

    /**
     * Write the policy related to the identifier into a file. The file location
     * is derived from the policy base directory and the path retrieved from the
     * identifier.
     * 
     * @param id
     *            The identifier whose policy shall be written.
     * @throws ParsingException
     *             Indicates a malformed policy identifier.
     * @throws IOException
     *             If an I/O-problem occurs.
     */
    public void write(Identifier id) throws ParsingException, IOException {
        if (id == null)
            throw new NullPointerException("identifier must not be null");
        write(findPolicy(id));
    }

    /**
     * Verifies the existence of an identifier in the file system.
     * 
     * @param id
     *            identifier to verify.
     * @return <code>true</code>, if the related policy file exists,
     *         <code>false</code> otherwise.
     */
    public boolean exists(Identifier id) {
        boolean result = false;
        if (id == null)
            throw new NullPointerException("identifier must not be null");
        return id.toFile(getPolicyBaseDir()).exists();
    }

    /**
     * Verifies the existence of the policy using its identifier in the file
     * system.
     * 
     * @param policy
     *            polciy to verify.
     * @return <code>true</code>, if the related policy file exists,
     *         <code>false</code> otherwise.
     */
    public boolean exists(AbstractPolicy policy) throws ParsingException {
        if (policy == null)
            throw new NullPointerException("policy must not be null");
        return exists(Identifier.getInstance(policy.getId()));
    }

    /**
     * Removes the file related to this policy. The file location is derived
     * from the policy base directory and the path retrieved from the policy's
     * identifier.
     * 
     * @param policy
     *            policy file to remove.
     * @return <code>true</code>, if the related policy file exists,
     *         <code>false</code> otherwise.
     */
    public boolean remove(AbstractPolicy policy) throws ParsingException {
        if (policy == null)
            throw new NullPointerException("policy must not be null");
        Identifier polId = Identifier.getInstance(policy.getId());
        return polId.toFile(getPolicyBaseDir()).delete();
    }

    /**
     * Removes the file related to this policy identifier. The file location is
     * derived from the policy base directory and the path retrieved from the
     * identifier.
     * 
     * @param id
     *            policy file to remove.
     * @return <code>true</code>, if the related policy file exists,
     *         <code>false</code> otherwise.
     */
    public boolean remove(Identifier id) throws ParsingException {
        if (id == null)
            throw new NullPointerException("identifier must not be null");
        return id.toFile(getPolicyBaseDir()).delete();
    }

    /**
     * Removes the whole domain directory from the file system.
     * 
     * @param domain
     *            Domain to remove.
     * @return true, if domain removed successfully, false otherwise.
     */
    public boolean removeDomain(String domain) {
        boolean result = true;
        try {
            FileUtils.deleteDirectory(new File(getPolicyBaseDir()
                    + File.separator + domain));
        } catch (IOException e) {
            result = false;
        }
        return result;

    }

    /**
     * Retrieves the directory path for policies of a given domain and type. The
     * path location is derived from the policy base directory, the domain label
     * and the policy type. If the domain is null or empty, then the path is
     * generated from the typeId.
     * 
     * @param domain
     *            The domain of the policy type, or null.
     * @param typeId
     *            The policy's typeId (e.g.
     *            {@link RolePolicySetIdentifier#TYPEID RolePolicySetIdentifier.TYPEID}).
     * @return The directory path to the policy type.
     */
    protected File getDirectoryPath(String domain, String typeId) {
        StringBuffer path = new StringBuffer(getPolicyBaseDir());
        if ((domain != null) && (domain.length() > 0))
            path.append(File.separator + domain);
        path.append(File.separator + typeId);
        return new File(path.toString());
    }

    /**
     * Gets a list of the domains in the current policy base directory. Domains
     * are assumed to be the top level directories within the policy base
     * directory.
     * 
     * @return List of domain names as {@link String String}instances.
     */
    public List getDomains() {
//      TODO: Fehler in commons.io.FileFilter- nimmt getName statt getPath und
//      findet so das Directorry nicht. Muss wohl anderen FileFilter verwenden.
//      DirectoryFilter msste die Methode accept(File,String) berschreiben)!!!
        
        NameFileFilter CVS = new NameFileFilter("CVS");
        NotFileFilter noCVS = new NotFileFilter(CVS);
        AndFileFilter domainFilter = new AndFileFilter(
                DirectoryFileFilter.INSTANCE, noCVS);
        File pbD = new File(getPolicyBaseDir());
        return Arrays.asList(pbD.list(domainFilter));
    }

    /**
     * Gets a list policies of the given type within the given domain. If the
     * domain is null or empty, then the are searched based on the typeId.
     * 
     * @return List of policies as {@link AbstractPolicy AbstractPolicy}
     *         instances.
     */
    public List getPolicies(String domain, String typeId) {
        File path = getDirectoryPath(domain, typeId);
        return RBACPolicyFinderModule.loadPolicies(path, getPolicyFileFilter(),
                getPolicyFinder());
    }

    /**
     * Gets a list of identifiers of the policies of the given type within the
     * given domain. If the domain is null or empty, then the are searched based
     * on the typeId.
     * 
     * @return List of policies as {@link AbstractPolicy AbstractPolicy}
     *         instances.
     */
    public List getIdentifiers(String domain, String typeId) {
        Iterator policyIt = getPolicies(domain, typeId).iterator();
        List identifiers = new ArrayList();
        while (policyIt.hasNext()) {
            try {
                identifiers
                        .add(Identifier.getInstance(((AbstractPolicy) policyIt
                                .next()).getId()));
            } catch (ParsingException e) {
                logger.error(e.getMessage());
            }
        }
        return identifiers;
    }

    /**
     * Retrieves the policy matching the given identifier. To find the policy,
     * the configured {@link PolicyFinder PolicyFinder}is used
     * 
     * @return The policy.
     * @throws ParsingException
     *             If the policy could not be loaded.
     *  
     */
    public AbstractPolicy findPolicy(Identifier id) throws ParsingException {
        initPolicyFinder();
        PolicyFinderResult finderResult = policyFinder
                .findPolicy(id.toURI(), 0);
        AbstractPolicy result = finderResult.getPolicy();
        if (result == null)
            throw new ParsingException("Unable to load " + id.toString() + "."
                    + StringUtil.lineSeparator + "Reason: "
                    + finderResult.getStatus().getMessage());
        return result;
    }

    /**
     * Retrieves the matching role policy set creating the given identifier. To
     * find the role policy set, the method
     * {@link #findPolicy(Identifier) findPolicy(Identifier)}is used.
     * 
     * @return The policy.
     * @throws ParsingException
     *             If the policy could not be loaded.
     */
    public RolePolicySet findRolePolicySet(String domain, String role)
            throws ParsingException {
        return new RolePolicySet(
                (PolicySet) findPolicy(new RolePolicySetIdentifier(domain, role)));
    }

    /**
     * Retrieves the matching RoleAssignmentPolicySet creating the given
     * identifier. To find the role policy set, the method
     * {@link #findPolicy(Identifier) findPolicy(Identifier)}is used.
     * 
     * @param domain
     *            The domain to search for.
     * @return The policy set.
     * @throws ParsingException
     *             If the policy could not be loaded.
     */
    public RoleAssignmentPolicySet findRoleAssignmentPolicySet(String domain)
            throws ParsingException {
        return new RoleAssignmentPolicySet(
                (PolicySet) findPolicy(new RoleAssignmentPolicySetIdentifier(
                        domain, domain)));
    }

    /**
     * Retrieves the matching RoleAssignmentPolicy creating the given
     * identifier. To find the role policy set, the method
     * {@link #findPolicy(Identifier) findPolicy(Identifier)}is used.
     * 
     * @param domain
     *            The domain to search for.
     * @param label
     *            The label of the RoleAssignmentPolicy to search for.
     * @return The policy.
     * @throws ParsingException
     *             If the policy could not be loaded.
     */
    public RoleAssignmentPolicy findRoleAssignmentPolicy(String domain,
            String label) throws ParsingException {
        return new RoleAssignmentPolicy(
                (Policy) findPolicy(new RoleAssignmentPolicyIdentifier(domain,
                        label)));
    }

    /**
     * Retrieves the matching permission policy set creating the given
     * identifier. To find the permission policy set, the method
     * {@link #findPolicy(Identifier) findPolicy(Identifier)}is used.
     * 
     * @param domain
     *            The domain to search for.
     * @param role
     *            The label of the Role of the PermissionPolicySet to search
     *            for.
     * @return The PermissionPolicySet.
     * @throws ParsingException
     *             If the policy could not be loaded.
     */
    public PermissionPolicySet findPermissionPolicySet(String domain,
            String role) throws ParsingException {
        return new PermissionPolicySet(
                (PolicySet) findPolicy(new RolePolicySetIdentifier(domain, role)));
    }

    /**
     * Retrieves the matching permission policy creating the given identifier.
     * To find the policy, the method
     * {@link #findPolicy(Identifier) findPolicy(Identifier)}is used.
     * 
     * @param domain
     *            The domain to search for.
     * @param permPolicy
     *            The label of the Permission Policy to search for.
     * @return The PermissionPolicy.
     * @throws ParsingException
     *             If the policy could not be loaded.
     */
    public PermissionPolicy findPermissionPolicy(String domain,
            String permPolicy) throws ParsingException {
        return new PermissionPolicy(
                (Policy) findPolicy(new PermissionPolicyIdentifier(domain,
                        permPolicy)));
    }

    /**
     * Inits the PolicyFinder. Must be called from non PDP-instances, which are
     * not initialising the policy finder itselfs, like the RoleManager.
     */
    public void initPolicyFinder() {
        getPolicyFinder().init();
    }

    /**
     * Gets the current policy finder
     * 
     * @return The current policy finder.
     */
    public PolicyFinder getPolicyFinder() {
        return policyFinder;
    }

    /**
     * Sets a new policy finder
     * 
     * @param policyFinder
     *            The new policy finder.
     */
    public void setPolicyFinder(PolicyFinder policyFinder) {
        this.policyFinder = policyFinder;
    }

    /**
     * Sets the policy base directory in the current configuration.
     * 
     * @return The base directory for all policies.
     */
    public void setPolicyBaseDir(String policyBaseDir) {
        configuration.addProperty("[@policyBaseDir]", policyBaseDir);
    }

    /**
     * Retrieves the policy base directory from the current configuration.
     * 
     * @return The base directory for all policies.
     */
    public String getPolicyBaseDir() {
        return configuration.getString("[@policyBaseDir]",
                FileUtil.currentDirectory);
    }

    /**
     * Retrieves a FileFilter bases on the file suffixes of the configuration.
     * 
     * @return {@link SuffixFileFilter SuffixFileFilter}, if suffixes are
     *         configured, otherwise {@link TrueFileFilter TrueFileFilter}.
     */
    public IOFileFilter getPolicyFileFilter() {
        return RBACPolicyFinderModule.getFileFilter(this.configuration,
                TrueFileFilter.INSTANCE);
    }
}
