/*
 * $Id: RBACPolicy.java,v 1.12 2005/12/07 18:08:13 suhrbier Exp $
 * Created on 15.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACPolicy.java,v $
 * Revision 1.12  2005/12/07 18:08:13  suhrbier
 * Fehler beim Parsen von PermitRuleAlgs in RBACPolicy behoben.
 *
 * Revision 1.11  2005/11/07 21:02:36  suhrbier
 * removeUserToRole test completed.
 *
 * Revision 1.10  2005/11/07 20:21:36  suhrbier
 * *** empty log message ***
 *
 * Revision 1.9  2005/11/04 17:52:36  suhrbier
 * Permission add mit Targets funzt.
 *
 * Revision 1.8  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.7  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.6  2005/11/01 11:50:45  suhrbier
 * RoleManager add, remove and list of users completed.
 *
 * Revision 1.5  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.3  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.2  2005/10/18 18:15:54  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.1  2005/10/15 18:00:50  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URI;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nbi.util.CollectionUtil;
import nbi.util.ReflectionUtil;
import nbi.xmlsec.xacml.builder.PolicyBuilder;
import nbi.xmlsec.xacml.builder.PolicyReferenceBuilder;

import org.apache.log4j.Logger;

import com.sun.xacml.AbstractPolicy;
import com.sun.xacml.Indenter;
import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.Target;
import com.sun.xacml.combine.CombiningAlgorithm;
import com.sun.xacml.combine.DenyOverridesRuleAlg;
import com.sun.xacml.combine.PermitOverridesRuleAlg;
import com.sun.xacml.combine.RuleCombiningAlgorithm;

/**
 * Base class for all RBACPolicies
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.12 $
 * @since
 */

public class RBACPolicy extends RBACPolicyElement {

    static Logger logger = Logger.getLogger(RBACPolicy.class);

    /**
     * Set of assigments administered in this role assignment policy
     */
    protected HashSet elements = null;

    /**
     * <code>true</code> sets the combining algorithm to
     * {@link PermitOverridesRuleAlg PermitOverridesRuleAlg},
     * <code>false</code> to {@link DenyOverridesRuleAlg DenyOverridesRuleAlg}.
     */
    protected boolean permitRule = true;

    protected RBACPolicy() {
    }

    /**
     * Create an RBACPolicy.
     * 
     * @param id
     *            Identifier of this element (domain:assignment:label).
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public RBACPolicy(Identifier id) {
        super(id);
    }

    /**
     * Create an RBACPolicy.
     * 
     * @param id
     *            Identifier of this element (domain:assignment:label).
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public RBACPolicy(URI id) throws ParsingException {
        super(id);
    }

    /**
     * Default method to parse policies.
     * <li>Verifies whether the parsed element is an policy or policy reference
     * </li>
     * <li>Sets the identifier of the policy</li>
     * <li>Calls {@link #parse(Target) parse(Target)}.</li>
     * <li>Adds each child element of the policy according to the class type
     * {@link Rule Rule}.</li>
     * According to the class type of the child element, an instance of the
     * related class (<code>ruleClass</code> is created with the child
     * element as single parameter. This instance is added to the set of policy
     * elements. If the related class parameter is <code>null</code>, then a
     * {@link ParsingException ParsingException}is thrown when the related
     * child element type is parsed.
     * 
     * @param element
     *            The policy (reference) to parse.
     * @param ruleClass
     *            Class to be instantiated for child elements instances of the
     *            (@link Policy Policy} class.
     * @throws ParsingException
     *             If some parsing error occurs.
     */
    protected void parse(PolicyTreeElement element, Class ruleClass)
            throws ParsingException {
        if (element == null)
            throw new ParsingException("Tree element is null");

        if (element instanceof Policy || element instanceof PolicyReference) {
            AbstractPolicy policy = (AbstractPolicy) element;
            setIdentifier(policy.getId());
            setPermitRule(PermitOverridesRuleAlg.algId.equals(policy
                    .getCombiningAlg().getIdentifier().toString()) ? true : false);
            parse(policy.getTarget());

            Iterator ruleIt = policy.getChildren().iterator();
            while (ruleIt.hasNext()) {
                Rule ruleObj = (Rule) ruleIt.next();
                if (ruleClass != null) {
                    try {
                        add((RBACPolicyElement) ReflectionUtil.getInstance(
                                ruleClass.getName(), Rule.class, ruleObj));
                    } catch (Exception e) {
                        logger.error(e.toString());
                    }
                } else
                    throw new ParsingException(
                            "Unexpected policy element of class type"
                                    + ruleObj.getClass().getName() + ".");
            }
        } else {
            throw new ParsingException("Unsupported tree element class:"
                    + element.getClass().getName());
        }
    }

    /**
     * Not implemented. Must be overwritten in subclasses.
     *  
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        parse(element, null);
    }

    /**
     * Returns the {@link Policy Policy}representation of this
     * {@link RBACPolicy RBACPolicy}. It generates a policy consisting of the
     * rule representation of all elements of this policy and the given
     * {@link RuleCombiningAlgorithm RuleCombiningAlgorithm}.
     * 
     * @param combAlg
     *            The {@link RuleCombiningAlgorithm RuleCombiningAlgorithm}of
     *            this policy.
     * @return The {@link Policy Policy}representation of this
     *         {@link RBACPolicy RBACPolicy}.
     * @see RBACPolicyElement#toPolicyTreeElement()
     */
    protected PolicyTreeElement toPolicyTreeElement(CombiningAlgorithm combAlg) {
        PolicyBuilder policyBuilder = new PolicyBuilder(
                getIdentifier().toURI(), (RuleCombiningAlgorithm) combAlg);
        policyBuilder.setTarget(getTarget().toTarget());
        if (elements != null) {
            Iterator it = elements.iterator();
            while (it.hasNext())
                policyBuilder.add((Rule) ((RBACPolicyElement) it.next())
                        .toPolicyTreeElement());
        }
        return (Policy) policyBuilder.getInstance();
    }

    /**
     * Default implementation for RBACPolicies. It generates a policy consisting
     * of the rule representation of all elements of this policy and according
     * to the combining rule depending on
     * {@link #isPermitRule() isDefaultPermit()}.
     * 
     * @return The {@link Policy Policy}representation of this
     *         {@link RBACPolicy RBACPolicy}.
     * @see RBACPolicyElement#toPolicyTreeElement()
     */
    public PolicyTreeElement toPolicyTreeElement() {
        if (isPermitRule())
            return toPolicyTreeElement(new PermitOverridesRuleAlg());
        else
            return toPolicyTreeElement(new DenyOverridesRuleAlg());
    }

    /**
     * Returns the {@link PolicyReference PolicyReference}representation of
     * this policy element.
     * 
     * @param type
     *            type of the policy reference (
     *            {@link PolicyReference#POLICYSET_REFERENCE PolicyReference.POLICYSET_REFERENCE}
     *            or
     *            {@link PolicyReference#POLICY_REFERENCE PolicyReference.POLICY_REFERENCE}).
     * @return The policy element's {@link PolicyReference PolicyReference}
     *         representation.
     */
    protected PolicyReference toPolicyRerence(int type) {
        PolicyReferenceBuilder policyReferenceBuilder = new PolicyReferenceBuilder(
                getIdentifier().toURI(), type);
        return (PolicyReference) policyReferenceBuilder.getInstance();
    }

    /**
     * Returns the {@link PolicyReference PolicyReference}representation of
     * this policy element. The default type of the policy reference is set to
     * {@link PolicyReference#POLICY_REFERENCE PolicyReference.POLICY_REFERENCE}).
     * Overwrite this method to set another reference type.
     * 
     * @return The policy element's {@link PolicyReference PolicyReference}
     *         representation with type set to
     *         {@link PolicyReference#POLICY_REFERENCE PolicyReference.POLICY_REFERENCE}).
     */
    public PolicyReference toPolicyRerence() {
        return toPolicyRerence(PolicyReference.POLICY_REFERENCE);
    }

    /**
     * Encodes this Policy into its XML representation and writes this encoding
     * to the file path using the <code>path</code> parameter as parent
     * directory and the identifier's file representation as file name. Creates
     * directories if missing.
     * 
     * @param path
     *            Base directory path to the policies.
     */
    public void encode(File path) throws IOException {
        File file = getIdentifier().toFile(path.getPath());
        File parent = new File(file.getParent());
        parent.mkdirs();
        encode(new FileOutputStream(file));
    }

    /**
     * Encodes this Policy into its XML representation and writes this encoding
     * to the given OutputStream with no indentation. *
     * 
     * @param out
     *            a stream into which the XML-encoded data is written.
     * @throws IOException
     */
    public void encode(OutputStream out) throws IOException {
        toPolicyTreeElement().encode(out, new Indenter());
        out.flush();
        out.close();
    }

    /**
     * Retrieves an element from this policy.
     * 
     * @param element
     *            The element to search for.
     * @return The element from the policy or <code>null</code>.
     * @see CollectionUtil#search(Collection, Object)
     */
    public RBACPolicyElement search(RBACPolicyElement element) {
        RBACPolicyElement result = null;
        if (elements != null)
            result = (RBACPolicyElement) CollectionUtil.search(elements,
                    element);
        return result;
    }

    /**
     * Retrieves an element from this policy.
     * 
     * @param id
     *            The identifier of the element to search for.
     * @return The element from this policy or <code>null</code>.
     */
    public RBACPolicyElement search(Identifier id) {
        RBACPolicyElement result = null;
        if (elements != null) {
            Iterator it = elements.iterator();
            while (it.hasNext() && (result == null)) {
                result = (RBACPolicyElement) it.next();
                if (!id.equals(result.getIdentifier()))
                    result = null;
            }
        }
        return result;
    }

    /**
     * Adds an policy element to this policy. If this policy element already
     * exists in the policy, this policy element may be updated by the
     * {@link RBACPolicyElement#add(RBACPolicyElement) RBACPolicyElement.add(RBACPolicyElement)}
     * method of the current element.
     * 
     * @param element
     *            The element to be added.
     * @return <code>true</code>, if the element was succesfully added or
     *         successfully updates an existing element, <code>false</code>
     *         otherwise.
     */
    public boolean add(RBACPolicyElement element) {
        boolean result = false;
        RBACPolicyElement polElement = search(element);
        if (polElement != null) {
            result = polElement.add(element);
        } else {
            try {
                elements = (HashSet) CollectionUtil.create(elements,
                        HashSet.class);
            } catch (InstantiationException e) {
                logger.fatal("The exception " + e + "should never happen!");
            } catch (IllegalAccessException e) {
                logger.fatal("The exception " + e + "should never happen!");
            }
            result = CollectionUtil.add(elements, element);
        }
        return result;
    }

    /**
     * Removes an policy element from this policy.
     * 
     * @param element
     *            The element to remove.
     * @return <code>true</code>, if the element was succesfully removed ,
     *         <code>false</code> otherwise.
     */
    public boolean remove(RBACPolicyElement element) {
        return CollectionUtil.remove(elements, element);
    }

    /**
     * Compares the specified object with this RBAC policy. Returns true if the
     * specified object is also an RBACPolicy and the included policy tree
     * elements is also equal.
     * 
     * @param obj
     *            Object to be compared for equality with this RBACPolicy.
     * @return <code>true</code> if the specified Object is equal to this
     *         RBACPolicy.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj instanceof RBACPolicy) {
            result = super.equals(obj);
            result &= elements != null ? elements
                    .equals(((RBACPolicy) obj).elements)
                    : ((RBACPolicy) obj).elements == null;
        }
        return result;
    }

    /**
     * Gets the set of elements of this policy.
     * 
     * @return The set of elements.
     */
    public Set getElements() {
        return elements;
    }

    /**
     * Gets the default CombiningAlgorithm of this policy used within
     * {@link #toPolicyTreeElement() #toPolicyTreeElement()}.
     * 
     * @return <code>true</code> the combining algorithm is set to
     *         {@link PermitOverridesRuleAlg PermitOverridesRuleAlg}, if
     *         <code>false</code> it is set to
     *         {@link DenyOverridesRuleAlg DenyOverridesRuleAlg}..
     */
    public boolean isPermitRule() {
        return permitRule;
    }

    /**
     * Sets the default CombiningAlgorithm of this policy used within
     * {@link #toPolicyTreeElement() #toPolicyTreeElement()}.
     * 
     * @param defaultPermit
     *            <code>true</code> sets the combining algorithm to
     *            {@link PermitOverridesRuleAlg PermitOverridesRuleAlg},
     *            <code>false</code> to
     *            {@link DenyOverridesRuleAlg DenyOverridesRuleAlg}.
     *  
     */
    public void setPermitRule(boolean defaultPermit) {
        this.permitRule = defaultPermit;
    }
}
