/*
 * $Id: RBACTarget.java,v 1.11 2005/12/06 17:12:15 suhrbier Exp $
 * Created on 15.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACTarget.java,v $
 * Revision 1.11  2005/12/06 17:12:15  suhrbier
 * list Befehl implementiert und getestet.
 * Fehler bei Domains der auf den DirectoryFileFitler von commons.io zurckgeht.
 *
 * Revision 1.10  2005/11/01 18:45:19  suhrbier
 * Permission upgedated.
 *
 * Revision 1.9  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.8  2005/10/24 09:38:39  suhrbier
 * Assignment test ok
 *
 * Revision 1.7  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.6  2005/10/18 18:15:55  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.5  2005/10/17 21:07:20  suhrbier
 * Assignment auf RBACTarget umgestellt.
 *
 * Revision 1.4  2005/10/15 21:37:40  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/15 18:00:50  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import nbi.util.CollectionUtil;
import nbi.util.StringUtil;
import nbi.xmlsec.xacml.builder.TargetBuilder;
import nbi.xmlsec.xacml.builder.TargetMatchBuilderFactory;
import nbi.xmlsec.xacml.parser.TargetMatchParserFactory;
import nbi.xmlsec.xacml.util.TargetUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.ParsingException;
import com.sun.xacml.Target;
import com.sun.xacml.TargetMatch;

/**
 * This class RBACTarget administers target sets in the RBAC context. It stores
 * any kind of object in the subject, resource or action set and provides
 * methods to handle these sets. Furthermore a Target representation can be
 * generated.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.11 $
 * @since
 */

public class RBACTarget {

    private static final Logger logger = Logger.getLogger(RBACTarget.class);

    /**
     * Set of subject elements of this target.
     */
    private HashSet subjects = null;

    /**
     * Set of resource elements of this target.
     */
    private HashSet resources = null;

    /**
     * Set of action elements of this target.
     */
    private HashSet actions = null;

    /**
     * Factory to generate TargetMatchBuilders for the target match elements.
     */
    private TargetMatchBuilderFactory builderFactory = null;

    /**
     * Factory to generate TargetMatches for the target match elements.
     */
    private TargetMatchParserFactory parserFactory = null;

    protected RBACTarget() {
    }

    /**
     * Creates an instance.
     * 
     * @param builderFactory
     *            Factory to generate TargetMatchBuilders for the target match
     *            elements
     * @param parserFactory
     *            Factory to generate TargetMatchParsers for the target match
     *            elements
     */
    protected RBACTarget(TargetMatchBuilderFactory builderFactory,
            TargetMatchParserFactory parserFactory) {
        super();
        this.builderFactory = builderFactory;
        this.parserFactory = parserFactory;
    }

    /**
     * Adds any object to the target's set of subjects, actions or resources.
     * 
     * @param obj
     *            The object to add.
     * @param setId
     *            The id of the set to add the object to (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return <code>true</code> if the object has been added succesfully,
     *         <code>false</code> otherwise.
     */
    protected boolean add(Object obj, int setId) {
        boolean result = true;
        try {
            switch (setId) {
            case TargetMatch.SUBJECT:
                subjects = (HashSet) CollectionUtil.create(subjects,
                        HashSet.class);
                break;
            case TargetMatch.RESOURCE:
                resources = (HashSet) CollectionUtil.create(resources,
                        HashSet.class);
                break;
            case TargetMatch.ACTION:
                actions = (HashSet) CollectionUtil.create(actions,
                        HashSet.class);
                break;
            default:
                result = false;
                break;
            }
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        if (result) {
            switch (setId) {
            case TargetMatch.SUBJECT:
                result = CollectionUtil.add(subjects, obj);
                break;
            case TargetMatch.RESOURCE:
                result = CollectionUtil.add(resources, obj);
                break;
            case TargetMatch.ACTION:
                result = CollectionUtil.add(actions, obj);
                break;
            default:
                result = false;
                break;
            }
        }
        return result;
    }

    /**
     * Adds any set of objects to the target's set of subjects, actions or
     * resources.
     * 
     * @param objSet
     *            The object to add.
     * @param setId
     *            The id of the set to add the object to (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return <code>true</code> if the object set has been added succesfully,
     *         <code>false</code> otherwise.
     */
    protected boolean add(Set objSet, int setId) {
        boolean result = true;
        try {
            switch (setId) {
            case TargetMatch.SUBJECT:
                subjects = (HashSet) CollectionUtil.create(subjects,
                        HashSet.class);
                break;
            case TargetMatch.RESOURCE:
                resources = (HashSet) CollectionUtil.create(resources,
                        HashSet.class);
                break;
            case TargetMatch.ACTION:
                actions = (HashSet) CollectionUtil.create(actions,
                        HashSet.class);
                break;
            default:
                result = false;
                break;
            }
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        if (result) {
            switch (setId) {
            case TargetMatch.SUBJECT:
                result = CollectionUtil.addAll(subjects, objSet);
                break;
            case TargetMatch.RESOURCE:
                result = CollectionUtil.addAll(resources, objSet);
                break;
            case TargetMatch.ACTION:
                result = CollectionUtil.addAll(actions, objSet);
                break;
            default:
                result = false;
                break;
            }
        }
        return result;
    }

    /**
     * Adds any subject, resource and action to the target's set of subjects,
     * resources and actions.
     * 
     * @param target
     *            The target to add.
     * @return <code>true</code> if any of the target objects has been added
     *         succesfully, <code>false</code> otherwise.
     */
    public boolean add(RBACTarget target) {
        boolean result = false;
        result = addSubject(target.getSubjects());
        result |= addResource(target.getResources());
        result |= addAction(target.getActions());
        return result;
    }

    /**
     * Removes any subject, resource and action to the target's set of subjects,
     * resources and actions.
     * 
     * @param target
     *            The target to remove.
     * @return <code>true</code> if any of the target objects has been removed
     *         succesfully, <code>false</code> otherwise.
     */
    public boolean remove(RBACTarget target) {
        boolean result = false;
        result = removeSubject(target.getSubjects());
        result |= removeResource(target.getResources());
        result |= removeAction(target.getActions());
        return result;
    }

    /**
     * Adds any object to the target's set of subject.
     * 
     * @param obj
     *            The object to add.
     * @return <code>true</code> if the object has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addSubject(Object obj) {
        return add(obj, TargetMatch.SUBJECT);
    }

    /**
     * Adds any object to the target's set of resources.
     * 
     * @param obj
     *            The object to add.
     * @return <code>true</code> if the object has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addResource(Object obj) {
        return add(obj, TargetMatch.RESOURCE);
    }

    /**
     * Adds any object to the target's set of actions.
     * 
     * @param obj
     *            The object to add.
     * @return <code>true</code> if the object has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addAction(Object obj) {
        return add(obj, TargetMatch.ACTION);
    }

    /**
     * Adds any object set to the target's set of subjects.
     * 
     * @param objSet
     *            The object set to add.
     * @return <code>true</code> if the object set has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addSubject(Set objSet) {
        return add(objSet, TargetMatch.SUBJECT);
    }

    /**
     * Adds any object set to the target's set of resources.
     * 
     * @param objSet
     *            The object set to add.
     * @return <code>true</code> if the object set has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addResource(Set objSet) {
        return add(objSet, TargetMatch.RESOURCE);
    }

    /**
     * Adds any object set to the target's set of actions.
     * 
     * @param objSet
     *            The object set to add.
     * @return <code>true</code> if the object set has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean addAction(Set objSet) {
        return add(objSet, TargetMatch.ACTION);
    }

    /**
     * Removes any object from the target's set of subjects.
     * 
     * @param obj
     *            The object to remove.
     * @return <code>true</code> if the object has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean removeSubject(Object obj) {
        return remove(obj, TargetMatch.SUBJECT);
    }

    /**
     * Removes any object from the target's set of subjects.
     * 
     * @param obj
     *            The object to remove.
     * @return <code>true</code> if the object has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean removeResource(Object obj) {
        return remove(obj, TargetMatch.RESOURCE);
    }

    /**
     * Removes any object from the target's set of subjects.
     * 
     * @param obj
     *            The object to remove.
     * @return <code>true</code> if the object has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean removeAction(Object obj) {
        return remove(obj, TargetMatch.ACTION);
    }

    /**
     * Adds any object set to the target's set of subjects.
     * 
     * @param objSet
     *            The object set to remove.
     * @return <code>true</code> if the object set has been removed
     *         succesfully, <code>false</code> otherwise.
     */
    public boolean removeSubject(Set objSet) {
        return remove(objSet, TargetMatch.SUBJECT);
    }

    /**
     * Removes any object set from the target's set of resources.
     * 
     * @param objSet
     *            The object set to remove.
     * @return <code>true</code> if the object set has been removed
     *         succesfully, <code>false</code> otherwise.
     */
    public boolean removeResource(Set objSet) {
        return remove(objSet, TargetMatch.RESOURCE);
    }

    /**
     * Removes any object set from the target's set of actions.
     * 
     * @param objSet
     *            The object set to remove.
     * @return <code>true</code> if the object set has been removed
     *         succesfully, <code>false</code> otherwise.
     */
    public boolean removeAction(Set objSet) {
        return remove(objSet, TargetMatch.ACTION);
    }

    /**
     * Removes any object from the target's set of subjects, actions or
     * resources.
     * 
     * @param obj
     *            The object to remove.
     * @param setId
     *            The id of the set to remove the object from(
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return <code>true</code> if the object has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    protected boolean remove(Object obj, int setId) {
        boolean result = true;
        switch (setId) {
        case TargetMatch.SUBJECT:
            result = CollectionUtil.remove(subjects, obj);
            break;
        case TargetMatch.RESOURCE:
            result = CollectionUtil.remove(resources, obj);
            break;
        case TargetMatch.ACTION:
            result = CollectionUtil.remove(actions, obj);
            break;
        default:
            result = false;
            break;
        }
        return result;
    }

    /**
     * Removes any object set from the target's set of subjects, actions or
     * resources.
     * 
     * @param objSet
     *            The set of objects to remove.
     * @param setId
     *            The id of the set to remove the object from(
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return <code>true</code> if the object has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    protected boolean remove(Set objSet, int setId) {
        boolean result = true;
        switch (setId) {
        case TargetMatch.SUBJECT:
            result = CollectionUtil.removeAll(subjects, objSet);
            break;
        case TargetMatch.RESOURCE:
            result = CollectionUtil.removeAll(resources, objSet);
            break;
        case TargetMatch.ACTION:
            result = CollectionUtil.removeAll(actions, objSet);
            break;
        default:
            result = false;
            break;
        }
        return result;
    }

    /**
     * Removes all elements from the subjects.
     */
    public void clearSubjects() {
        CollectionUtil.clear(subjects);
    }

    /**
     * Removes all elements from the subjects.
     */
    public void clearResources() {
        CollectionUtil.clear(resources);
    }

    /**
     * Removes all elements from the subjects.
     */
    public void clearActions() {
        CollectionUtil.clear(actions);
    }

    /**
     * Removes all elements from all sets.
     */
    public void clearAll() {
        clearSubjects();
        clearResources();
        clearActions();
    }

    /**
     * Returns an iterator over the elements of the given set.
     * 
     * @param setId
     *            The id of the set to return the iterator from(
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return The iterator or <code>null</code>.
     */
    protected Iterator iterator(int setId) {
        Iterator result = null;
        switch (setId) {
        case TargetMatch.SUBJECT:
            result = subjects != null ? subjects.iterator() : null;
            break;
        case TargetMatch.RESOURCE:
            result = (resources != null ? resources.iterator() : null);
            break;
        case TargetMatch.ACTION:
            result = actions != null ? actions.iterator() : null;
            break;
        }
        return result;
    }

    /**
     * Returns an iterator over the subject elements.
     * 
     * @return The iterator or <code>null</code>.
     */
    protected Iterator subjectIterator() {
        return iterator(TargetMatch.SUBJECT);
    }

    /**
     * Returns an iterator over the resource elements.
     * 
     * @return The iterator or <code>null</code>.
     */
    protected Iterator resourceIterator() {
        return iterator(TargetMatch.RESOURCE);
    }

    /**
     * Returns an iterator over the action elements.
     * 
     * @return The iterator or <code>null</code>.
     */
    protected Iterator actionIterator() {
        return iterator(TargetMatch.ACTION);
    }

    /**
     * Returns the {@link Target Target}representation of the RBACTarget.
     * 
     * @return The {@link Target Target}representation.
     */
    public Target toTarget() {
        TargetBuilder targetBuilder = new TargetBuilder();
        Iterator it = subjectIterator();
        while ((it != null) && it.hasNext())
            targetBuilder.add(builderFactory.getInstance(TargetMatch.SUBJECT,
                    it.next()));
        it = resourceIterator();
        while ((it != null) && it.hasNext())
            targetBuilder.add(builderFactory.getInstance(TargetMatch.RESOURCE,
                    it.next()));
        it = actionIterator();
        while ((it != null) && it.hasNext())
            targetBuilder.add(builderFactory.getInstance(TargetMatch.ACTION, it
                    .next()));

        return targetBuilder.getInstance();
    }

    /**
     * Parses an RBACTarget from its {@link Target Target}representation.
     * 
     * @param target
     *            The target to parse.
     * @throws ParsingException
     */
    public void parse(Target target) throws ParsingException {
        List matchList = TargetUtil.getTargetMatchList(target,
                TargetMatch.SUBJECT);
        if (matchList != null) {
            Iterator matchIt = matchList.iterator();
            while (matchIt.hasNext())
                addSubject(parserFactory.getInstance(TargetMatch.SUBJECT,
                        (TargetMatch) matchIt.next()).parseMatchValue());
        }
        matchList = TargetUtil.getTargetMatchList(target, TargetMatch.RESOURCE);
        if (matchList != null) {
            Iterator matchIt = matchList.iterator();
            while (matchIt.hasNext())
                addResource(parserFactory.getInstance(TargetMatch.RESOURCE,
                        (TargetMatch) matchIt.next()).parseMatchValue());
        }
        matchList = TargetUtil.getTargetMatchList(target, TargetMatch.ACTION);
        if (matchList != null) {
            Iterator matchIt = matchList.iterator();
            while (matchIt.hasNext())
                addAction(parserFactory.getInstance(TargetMatch.ACTION,
                        (TargetMatch) matchIt.next()).parseMatchValue());
        }
    }

    /**
     * Gets the actions of this target.
     * 
     * @return Returns the actions.
     */
    public HashSet getActions() {
        return actions;
    }

    /**
     * Gets the resources of this target.
     * 
     * @return Returns the resources.
     */
    public HashSet getResources() {
        return resources;
    }

    /**
     * Gets the subjects of this target.
     * 
     * @return Returns the subjects.
     */
    public HashSet getSubjects() {
        return subjects;
    }

    /**
     * Compares the specified object with the subject set of this target.
     * 
     * @param obj
     *            Object to be compared for equality with the subject set.
     * @return <code>true</code> if the specified Object is equal to the
     *         subject set.
     */
    public boolean equalsSubjects(Object obj) {
        return subjects != null ? subjects.equals(obj) : obj == null;
    }

    /**
     * Compares the specified object with the resource set of this target.
     * 
     * @param obj
     *            Object to be compared for equality with the resource set.
     * @return <code>true</code> if the specified Object is equal to the
     *         resource set.
     */
    public boolean equalsResources(Object obj) {
        return resources != null ? resources.equals(obj) : obj == null;
    }

    /**
     * Compares the specified object with the action set of this target.
     * 
     * @param obj
     *            Object to be compared for equality with the action set.
     * @return <code>true</code> if the specified Object is equal to the
     *         action set.
     */
    public boolean equalsActions(Object obj) {
        return actions != null ? actions.equals(obj) : obj == null;
    }

    /**
     * Compares the specified object with this target. Returns true if the
     * specified object is also a RBACTarget, and if the subject, resource and
     * action set are equal.
     * 
     * @param obj
     *            Object to be compared for equality with this target.
     * @return <code>true</code> if the specified Object is equal to this
     *         target.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj instanceof RBACTarget) {
            result = equalsSubjects(((RBACTarget) obj).getSubjects());
            result &= equalsResources(((RBACTarget) obj).getResources());
            result &= equalsActions(((RBACTarget) obj).getActions());
        }
        return result;
    }

    protected StringBuffer targetSetToStringBuffer(int setId) {
        StringBuffer result = new StringBuffer();
        String targetSetType = null;
        HashSet set = null;
        Iterator setIt = null;
        switch (setId) {
        case TargetMatch.SUBJECT:
            targetSetType = "subject";
            set = getSubjects();
            setIt = subjectIterator();
            break;
        case TargetMatch.RESOURCE:
            targetSetType = "resource";
            set = getResources();
            setIt = resourceIterator();
            break;
        case TargetMatch.ACTION:
            targetSetType = "action";
            set = getActions();
            setIt = actionIterator();
            break;
        }
        if (set == null) {
            result.append("no " + targetSetType + "s");
            result.append(StringUtil.lineSeparator);
        } else {
            result.append(set.size());
            result.append(" ");
            result.append(targetSetType + "s ");
            result.append(StringUtil.lineSeparator);
            for (int i = 0; i < (targetSetType.length() + 5); i++)
                result.append("=");
            result.append(StringUtil.lineSeparator);
            while (setIt.hasNext()) {
                result.append(setIt.next().toString());
                result.append(StringUtil.lineSeparator);
            }
        }
        return result;
    }

    public String toString() {
        StringBuffer result = new StringBuffer();
        result.append(targetSetToStringBuffer(TargetMatch.SUBJECT));
        result.append(targetSetToStringBuffer(TargetMatch.RESOURCE));
        result.append(targetSetToStringBuffer(TargetMatch.ACTION));
        return result.toString();
    }
}
