/*
 * $Id: RoleAssignmentPolicy.java,v 1.8 2005/11/03 21:15:03 suhrbier Exp $
 * Created on 28.01.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RoleAssignmentPolicy.java,v $
 * Revision 1.8  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.7  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.6  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.5  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.4  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/18 18:15:54  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.2  2005/10/15 18:00:50  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/14 18:30:59  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/04/01 22:01:37  suhrbier
 * + RoleManager Assignment Handling fertig.
 *
 * Revision 1.4  2005/04/01 18:22:21  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/03/31 18:08:53  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/03/30 14:51:32  suhrbier
 * RoleAssignmentPolicy fertiggestellt.
 *
 * Revision 1.1  2005/03/29 17:47:51  suhrbier
 * last edit: Assignment
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.5  2005/02/10 12:33:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.4  2005/02/08 17:51:31  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/02/08 12:17:15  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/01/31 18:01:35  suhrbier
 * RBAC Policy Writer
 *
 * Revision 1.1  2005/01/28 15:51:19  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;
import java.util.Iterator;
import java.util.Set;

import nbi.xmlsec.xacml.builder.PolicyBuilder;
import nbi.xmlsec.xacml.util.TargetUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.AbstractPolicy;
import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.combine.PermitOverridesRuleAlg;

/**
 * This class implements a role assignment policy according to the XACML RBAC
 * Profile. It administers subject-to-role assigments, which may be represented
 * as a {@link Policy Policy}or parsed from a {@link Policy Policy}
 * representation.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.8 $
 * @since
 */

public class RoleAssignmentPolicy extends RBACPolicy {

    static Logger logger = Logger.getLogger(RoleAssignmentPolicy.class);

    /**
     * Creates role assigment policy with its identifier.
     * 
     * @param domain
     *            Identifier's domain.
     * @param label
     *            Identifier's label.
     */
    public RoleAssignmentPolicy(String domain, String label) {
        super(new RoleAssignmentPolicyIdentifier(domain, label));
        setTarget(new RBACTarget());
    }

    /**
     * Creates role assigment policy with its identifier.
     * 
     * @param id
     *            Identifier's URI representation.
     */
    public RoleAssignmentPolicy(URI id) throws ParsingException {
        super(Identifier.getInstance(id));
        setTarget(new RBACTarget());
    }

    /**
     * Create an Assigment instance from its {@link Policy Policy}
     * representation.
     * 
     * @param policy
     *            {@link Policy Policy}representation.
     */
    public RoleAssignmentPolicy(Policy policy) throws ParsingException {
        setTarget(new RBACTarget());
        parse(policy);
    }

    /**
     * Create an Assigment instance from its {@link Policy Policy}
     * representation.
     * 
     * @param policy
     *            {@link Policy Policy}representation.
     */
    public RoleAssignmentPolicy(PolicyReference policy) throws ParsingException {
        setTarget(new RBACTarget());
        parse(policy);
    }

    /**
     * Parses a role assignment policy from its {@link Policy Policy}
     * representation.
     * 
     * @param element
     *            The policy to parse.
     * @throws ParsingException
     *             Indicates a malformed {@link Policy Policy}representation.
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        if (element == null)
            throw new ParsingException("Tree element is null");
        
        if (element instanceof Policy || element instanceof PolicyReference) {
            AbstractPolicy policy = (AbstractPolicy) element;
            setIdentifier(policy.getId());
            if (!TargetUtil.isEmpty(policy.getTarget()))
                throw new ParsingException(
                        "Target of the RoleAssignmentPolicy "
                                + getIdentifier().toString()
                                + " must be empty.");
            Iterator ruleIt = policy.getChildren().iterator();
            while (ruleIt.hasNext()) {
                add(new Assignment((Rule) ruleIt.next()));
            }
        } else {
            throw new ParsingException("Unsupported tree element class:"+element.getClass().getName());
        }
    }

    /**
     * Returns the {@link Policy Policy} representation of the role assignment
     * policy. 
     * 
     * @return The {@link Policy Policy} representation.
     */
    public PolicyTreeElement toPolicyTreeElement() {
        PolicyBuilder policyBuilder = new PolicyBuilder(
                getIdentifier().toURI(), new PermitOverridesRuleAlg());
        if (elements != null) {
            Iterator it = elements.iterator();
            while (it.hasNext())
                policyBuilder.add((Rule) ((Assignment) it.next())
                        .toPolicyTreeElement());
        }
        return (Policy) policyBuilder.getInstance();
    }

    /**
     * Returns the {@link PolicyReference PolicyRefence} to this the role
     * assignment policy.
     * 
     * @return The {@link PolicyReference PolicyReference}.
     */
    public PolicyReference toPolicyReference() {
        return super.toPolicyRerence(PolicyReference.POLICY_REFERENCE);
    }

    /**
     * If the element is an {@link Assignment Assignment}, the element is added to the set of elements.
     * @return <code>true</code>, if an assigment was succesfully added, <code>false</code>
     *         otherwise.
     * 
     * @see RBACPolicyElement#add(RBACPolicyElement)
     */
    public boolean add(RBACPolicyElement element) {
        boolean result=false;
        if(element instanceof Assignment)
            result=super.add(element);
        return result;
    }

    /**
     * If the element is an {@link Assignment Assignment}, the element is removed from the set of elements.
     * @return <code>true</code>, if an assigment was succesfully removed, <code>false</code>
     *         otherwise.
     * 
     * @see RBACPolicyElement#remove(RBACPolicyElement)
     */
    public boolean remove(RBACPolicyElement element) {
        boolean result=false;
        if(element instanceof Assignment)
            result=elements.remove(element);
        return result;
    }
    
    /**
     * Gets the assignments of this role.
     * 
     * @return Set of role assignments.
     */
    public Set getAssignments()
    {
        return elements;
    }
}
