/*
 * $Id: RoleAssignmentPolicySet.java,v 1.12 2006/07/24 17:31:10 suhrbier Exp $
 * Created on 08.02.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RoleAssignmentPolicySet.java,v $
 * Revision 1.12  2006/07/24 17:31:10  suhrbier
 * update JavaDoc
 *
 * Revision 1.11  2005/12/14 15:34:31  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.10  2005/12/06 17:12:15  suhrbier
 * list Befehl implementiert und getestet.
 * Fehler bei Domains der auf den DirectoryFileFitler von commons.io zurckgeht.
 *
 * Revision 1.9  2005/11/07 20:21:36  suhrbier
 * *** empty log message ***
 *
 * Revision 1.8  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.7  2005/11/02 20:10:32  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.6  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/10/25 18:18:24  suhrbier
 * RoleAssignmentPolicyTest running.
 *
 * Revision 1.4  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/18 18:15:55  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.2  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/14 18:30:58  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/04/01 22:01:38  suhrbier
 * + RoleManager Assignment Handling fertig.
 *
 * Revision 1.4  2005/04/01 18:22:21  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/03/30 14:51:32  suhrbier
 * RoleAssignmentPolicy fertiggestellt.
 *
 * Revision 1.2  2005/03/29 17:47:52  suhrbier
 * last edit: Assignment
 *
 * Revision 1.1  2005/03/07 18:22:32  suhrbier
 * Initiale Version
 *
 * Revision 1.4  2005/02/10 22:05:11  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/02/10 12:33:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/02/08 17:51:31  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/02/08 12:17:15  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nbi.xmlsec.xacml.util.TargetUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicySet;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Target;
import com.sun.xacml.finder.PolicyFinder;

/**
 * This class implements a role assignment policy set according to the XACML
 * RBAC Profile. It administers role assigment policies, which may be
 * represented as a {@link PolicySet PolicySet}or parsed from a
 * {@link PolicySet PolicySet}representation.
 * 
 * NOTE: If a policyFinder is set, this method adds the policies of this set as
 * {@link PolicyReference PolicyReference}objects to this policy set.
 * Otherwise, policies are added as {@link Policy Policy}objects.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.12 $
 * @since
 */

public class RoleAssignmentPolicySet extends RBACPolicySet {
    static Logger logger = Logger.getLogger(RoleAssignmentPolicySet.class);

    PolicyFinder policyFinder = null;

    /**
     * Creates role assigment policy set from its identifier. The default policy
     * representation type is set to
     * {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICYREF}.
     * 
     * @param domain
     *            Identifier's domain.
     * @param label
     *            Identifier's label.
     */
    public RoleAssignmentPolicySet(String domain, String label) {
        super(new RoleAssignmentPolicySetIdentifier(domain, label),
                REPRESENT_POLICIES_AS_POLICYREF);
        setTarget(new RBACTarget());
    }

    /**
     * Creates role assigment policy set from its identifier. The default policy
     * representation type is set to
     * {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICYREF}.
     * 
     * @param id
     *            Identifier's URI representation.
     */
    public RoleAssignmentPolicySet(URI id) throws ParsingException {
        super(Identifier.getInstance(id), REPRESENT_POLICIES_AS_POLICYREF);
        setTarget(new RBACTarget());
    }

    /**
     * Create an role assigment policy set from its {@link PolicySet PolicySet}
     * representation. The first policy parsed determines the policy
     * representation type of the policy set.
     * 
     * @param policySet
     *            {@link PolicySet PolicySet}representation.
     */
    public RoleAssignmentPolicySet(PolicySet policySet) throws ParsingException {
        setTarget(new RBACTarget());
        parse(policySet);
    }

    /**
     * Create an role assigment policy set from its {@link PolicySet PolicySet}
     * representation. The first policy parsed determines the policy
     * representation type of the policy set.
     * 
     * @param policySet
     *            {@link PolicySet PolicySet}representation.
     */
    public RoleAssignmentPolicySet(PolicySet policySet, PolicyFinder finder)
            throws ParsingException {
        setTarget(new RBACTarget());
        setPolicyFinder(finder);
        parse(policySet);
    }

    /**
     * Parses the target of an PermissionPolicySet.
     * 
     * @see nbi.xmlsec.xacml.profile.rbac.RBACPolicySet#parse(com.sun.xacml.Target)
     */
    public void parse(Target target) throws ParsingException {
        if (!TargetUtil.isEmpty(target))
            throw new ParsingException("Target of the RoleAssignmentPolicySet "
                    + getIdentifier().toString() + " must be empty.");
    }

    /**
     * Parses a roleassignment policy set from its {@link PolicySet PolicySet}
     * representation. The first policy parsed determines the policy
     * representation type of the policy set.
     * 
     * @param element
     *            The policy set to parse.
     * @throws ParsingException
     *             Indicates errors during the parsing process.
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        super.parse(element, RoleAssignmentPolicy.class,
                RoleAssignmentPolicy.class, null);
    }

    /**
     * Retrieves an assigment from an role assignment policy of this role
     * assignment policy set.
     * 
     * @param label
     *            The name of the assignment to search for.
     * @return The assignemtn from the role assignment policy or
     *         <code>null</code>.
     * @see RBACPolicy#search(Identifier).
     */
    public Assignment searchAssignment(String label) {
        Assignment result = null;
        RoleAssignmentPolicy policy = (RoleAssignmentPolicy) search(new RoleAssignmentPolicyIdentifier(
                getIdentifier().getDomain(), label));
        if (policy != null)
            result = (Assignment) policy.search(new AssignmentIdentifier(
                    getIdentifier().getDomain(), label));
        return result;
    }

    /**
     * If the element is an {@link RoleAssignmentPolicy RoleAssignmentPolicy},
     * the element is added to the set of elements.
     * 
     * @return <code>true</code>, if an role assignment Policy was
     *         succesfully added, <code>false</code> otherwise.
     * 
     * @see RBACPolicyElement#add(RBACPolicyElement)
     */
    public boolean add(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof RoleAssignmentPolicy)
            result = super.add(element);
        return result;
    }

    /**
     * Adds the assignment to the element's related role assignment policy of
     * this role assignment policy set. If the RoleAssignmentPolicy does not
     * exist, it will be created and added to this RoleAssignmentPolicySet.
     * 
     * @param element
     *            The assignment to add.
     * @return <code>true</code>, if the element was succesfully added or
     *         successfully updates an existing element, <code>false</code>
     *         otherwise.
     * @see RBACPolicy#remove(RBACPolicyElement).
     */
    public boolean add(Assignment element) {
        boolean result = false;
        RoleAssignmentPolicy policy = (RoleAssignmentPolicy) search(new RoleAssignmentPolicyIdentifier(
                getIdentifier().getDomain(), element.getRoleIdentifier()
                        .getLabel()));
        if (policy == null)
            policy = new RoleAssignmentPolicy(getIdentifier().getDomain(),
                    element.getRoleIdentifier().getLabel());
        if (policy.add(element))
            result = add(policy);
        return result;
    }

    /**
     * Removes the assignment from the role assignment policy of this role role
     * assignment policy set.
     * 
     * @param element
     *            The assignment to remove.
     * @return <code>true</code>, if the element was succesfully removed or
     *         successfully updates an existing element, <code>false</code>
     *         otherwise.
     * @see RBACPolicy#remove(RBACPolicyElement).
     */
    public boolean remove(Assignment element) {
        boolean result = false;
        RoleAssignmentPolicy policy = (RoleAssignmentPolicy) search(new RoleAssignmentPolicyIdentifier(
                getIdentifier().getDomain(), getIdentifier().getLabel()));
        if (policy != null)
            result = policy.remove(element);
        return result;
    }

    /**
     * If the element is an {@link RoleAssignmentPolicy RoleAssignmentPolicy},
     * the element is removed from the set of elements.
     * 
     * @return <code>true</code>, if an role assigment policy was succesfully
     *         removed, <code>false</code> otherwise.
     * 
     * @see RBACPolicyElement#remove(RBACPolicyElement)
     */
    public boolean remove(RBACPolicyElement element) {
        boolean result = false;
        if (element instanceof RoleAssignmentPolicy)
            result = super.remove(element);
        return result;
    }

    /**
     * Sets policyFinder
     * 
     * @param policyFinder
     *            The policyFinder to set.
     */
    public void setPolicyFinder(PolicyFinder policyFinder) {
        this.policyFinder = policyFinder;
    }

    /**
     * Gets the Assignments of the RoleAssignmentPolicies of this
     * RoleAssignmentPolicySet.
     * 
     * @return Set of Assignments or null, if no RoleAssignmentPolicy exists.
     */
    public Set getAssignments() {
        HashSet result = null;
        Set roleAssignPolicies = getRoleAssignmentPolicies();
        if (roleAssignPolicies != null) {
            Iterator roleAssignPolicyIt = roleAssignPolicies.iterator();
            result = new HashSet();
            while (roleAssignPolicyIt.hasNext())
                result
                        .addAll(((RoleAssignmentPolicy) roleAssignPolicyIt
                                .next()).getElements());
        }
        return result;
    }

    /**
     * Gets the RoleAssignmentPolicies of this RoleAssignmentPolicySet.
     * 
     * @return Set of RoleAssignmentPolicies or null, if no elements available.
     */
    public Set getRoleAssignmentPolicies() {
        Set result = null;
        if ((getElements() != null) && (!getElements().isEmpty()))
            result = getElements();
        return result;
    }
}
