/*
 * $Id: RoleEnablementAuthority.java,v 1.14 2005/12/19 19:11:05 suhrbier Exp $
 * Created on 02.04.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RoleEnablementAuthority.java,v $
 * Revision 1.14  2005/12/19 19:11:05  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.13  2005/12/14 17:27:09  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.12  2005/12/14 15:34:31  suhrbier
 * Neue Policies scheinen jetzt zu laufen...
 *
 * Revision 1.11  2005/12/09 11:01:29  suhrbier
 * 2 special PolicyFinder fr REA und PDP
 *
 * Revision 1.10  2005/12/07 19:34:54  suhrbier
 * Mehrere Configuration Fehler beim Start von PEP behoben !
 *
 * Revision 1.9  2005/11/04 17:52:36  suhrbier
 * Permission add mit Targets funzt.
 *
 * Revision 1.8  2005/11/02 20:10:32  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.7  2005/10/05 14:38:00  suhrbier
 * Klasse ReflectionUtil eingefhrt.
 *
 * Revision 1.6  2005/10/04 18:21:30  suhrbier
 * Umgestellt auf commons-Configuration
 *
 * Revision 1.5  2005/09/23 19:11:12  suhrbier
 * Jetty Anbindung bis auf Parameter fertig.
 *
 * Revision 1.4  2005/04/05 16:31:34  suhrbier
 * Setup fr Prsentation beim BGBM
 *
 * Revision 1.3  2005/04/04 21:23:23  suhrbier
 * Erste lauffhige Version mit RoleEnableAuth und RBACPDP
 *
 * Revision 1.2  2005/04/04 17:20:57  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/04/04 09:38:58  suhrbier
 * + RoleEnableAuth und RBACPDP integriert
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import nbi.util.ReflectionUtil;
import nbi.xmlsec.xacml.builder.ActionAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.builder.ResourceAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.builder.SubjectAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.profile.rbac.builder.EnableRoleActionAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.profile.rbac.builder.RoleResourceAttributeDesignatorBuilder;

import org.apache.commons.configuration.Configuration;
import org.apache.log4j.Logger;

import com.sun.xacml.PDP;
import com.sun.xacml.PDPConfig;
import com.sun.xacml.ParsingException;
import com.sun.xacml.attr.AnyURIAttribute;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.attr.X500NameAttribute;
import com.sun.xacml.ctx.Attribute;
import com.sun.xacml.ctx.RequestCtx;
import com.sun.xacml.ctx.Result;
import com.sun.xacml.ctx.Subject;
import com.sun.xacml.finder.AttributeFinder;
import com.sun.xacml.finder.impl.CurrentEnvModule;
import com.sun.xacml.finder.impl.SelectorModule;

/**
 * This class realises the functionality of the RoleEnablementAuthority as
 * described in the XACML RBAC profile v2.0. The class acts like an PDP, which
 * works on RoleAssignmentPolicies only. The main method of this class is
 * {@link #getRoles(String, X509Certificate) getRoles(String, X509Certificate)},
 * which evaluates all roles for a given entity authenticated by its X.509
 * certificate. The configuration is done within the XML-tag
 * <code>RoleEnablementAuthority</code>. It mainly consist configuring the
 * underlying {@link PolicyManager PolicyManager}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.14 $
 * @since
 */

public class RoleEnablementAuthority {
    static Logger logger = Logger.getLogger(RoleEnablementAuthority.class);

    /**
     * Configuration instance of this class.
     */
    Configuration config;

    /**
     * PolicyManager instance
     */
    PolicyManager polMgr;

    /**
     * The PDP instance.
     */
    PDP pdp;

    /**
     * Creates a RoleEnablementAuthority based on its configuration.
     * 
     * @param config
     *            The configuration.
     * @throws ClassNotFoundException
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    public RoleEnablementAuthority(Configuration config)
            throws ClassNotFoundException, NoSuchMethodException,
            InvocationTargetException, InstantiationException,
            IllegalAccessException {
        this.config = config;
        reset();
    }

    /**
     * Resets the REA reinitialising the PolicyManager instance, reloading the
     * assigned policies needed to create a new PDP instance.
     * 
     * @throws IllegalArgumentException
     * @throws SecurityException
     * @throws ClassNotFoundException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    protected void reset() throws IllegalArgumentException, SecurityException,
            ClassNotFoundException, InstantiationException,
            IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        polMgr = (PolicyManager) ReflectionUtil.getInstance(config
                .getString("PolicyManager[@class]"), Configuration.class,
                config.subset("PolicyManager"));

        CurrentEnvModule envAttributeModule = new CurrentEnvModule();
        SelectorModule selectorAttributeModule = new SelectorModule();
        List attributeModules = new ArrayList();
        attributeModules.add(envAttributeModule);
        attributeModules.add(selectorAttributeModule);
        AttributeFinder attributeFinder = new AttributeFinder();
        attributeFinder.setModules(attributeModules);

        pdp = new PDP(new PDPConfig(attributeFinder, polMgr.getPolicyFinder(),
                null));

    }

    /**
     * Evaluates all roles for a given X.509-certificate.
     * 
     * @param domain
     *            The domain to be evaluated.
     * @param subjectCert
     *            The certificate.
     * @return A set of RoleAssignmentPolicyIdentifiers containing the evaluated
     *         roles for the subject.
     * @throws ParsingException
     * @throws URISyntaxException
     * @throws ClassNotFoundException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    public Set getRoles(String domain, X509Certificate subjectCert)
            throws ParsingException, URISyntaxException,
            ClassNotFoundException, InstantiationException,
            IllegalAccessException, InvocationTargetException,
            NoSuchMethodException, InvocationTargetException,
            NoSuchMethodException {
        //        reset();
        HashSet result = null;
        HashSet roles = new HashSet(polMgr.getIdentifiers(domain,
                RoleAssignmentPolicyIdentifier.TYPEID));

        Iterator roleIt = roles.iterator();
        while (roleIt.hasNext()) {
            RoleAssignmentPolicyIdentifier roleId = (RoleAssignmentPolicyIdentifier) roleIt
                    .next();
            RequestCtx enableRoleRequest = createEnableRoleRequest(domain,
                    subjectCert, roleId.getLabel());
            Result requestResult = (Result) pdp.evaluate(enableRoleRequest)
                    .getResults().toArray()[0];
            if (requestResult.getDecision() == Result.DECISION_PERMIT) {
                logger.info("Subject " + subjectCert.getSubjectDN().getName()
                        + " is member of role " + roleId.toString()
                        + " in domain " + domain);
                if (result == null)
                    result = new HashSet(1);
                result.add(roleId);
            } else
                logger.info("Subject " + subjectCert.getSubjectDN().getName()
                        + " is not member of role " + roleId.toString()
                        + " in domain " + domain);
        }
        return result;
    }

    /**
     * Creates an enableRole-Request to evaluate the PDP, if the subject has the
     * given role enabled.
     * 
     * @param domain
     *            The domain.
     * @param subject
     *            The subject.
     * @param role
     *            The role label.
     * @return The enableRole request.
     * @throws URISyntaxException
     */
    public RequestCtx createEnableRoleRequest(String domain,
            X509Certificate subject, String role) throws URISyntaxException {
        HashSet subjects = new HashSet();
        HashSet resources = new HashSet();
        HashSet actions = new HashSet();
        HashSet environments = new HashSet();

        HashSet subjectAttrs = new HashSet();
        URI subjectId = new URI(SubjectAttributeDesignatorBuilder.attributeId);
        X500NameAttribute subjectValue = new X500NameAttribute(subject
                .getSubjectX500Principal());
        subjectAttrs.add(new Attribute(subjectId, null, null, subjectValue));
        subjects.add(new Subject(subjectAttrs));

        URI resourceId = new URI(
                RoleResourceAttributeDesignatorBuilder.attributeId);
        AnyURIAttribute resourceValue = new AnyURIAttribute(
                new RoleValueIdentifier(domain, role).toURI());
        resources.add(new Attribute(resourceId, null, null, resourceValue));

        URI actionId = new URI(
                EnableRoleActionAttributeDesignatorBuilder.attributeId);
        AnyURIAttribute actionValue = new AnyURIAttribute(
                Assignment.enableRoleActionId);
        actions.add(new Attribute(actionId, null, null, actionValue));
        RequestCtx request = new RequestCtx(subjects, resources, actions,
                environments);
        return request;
    }

    /**
     * Creates an enableRole-Request to evaluate the PDP, if the subject has the
     * given role enabled. The subject is expected to be a X500Name-String.
     * 
     * @param domain
     *            The domain.
     * @param subject
     *            The subject as X500Name String..
     * @param role
     *            The role label.
     * @return The enableRole request.
     * @throws URISyntaxException
     */
    public RequestCtx createEnableRoleRequest(String domain, String subject,
            String role) throws URISyntaxException {
        HashSet subjects = new HashSet();
        HashSet resources = new HashSet();
        HashSet actions = new HashSet();
        HashSet environments = new HashSet();

        HashSet subjectAttrs = new HashSet();
        URI subjectId = new URI(SubjectAttributeDesignatorBuilder.attributeId);
        StringAttribute subjectValue = new StringAttribute(subject);
        subjectAttrs.add(new Attribute(subjectId, null, null, subjectValue));
        subjects.add(new Subject(subjectAttrs));

        //        URI resourceId = new URI(RolePolicySet.ATTRIBUTE_ID);
        URI resourceId = new URI(ResourceAttributeDesignatorBuilder.attributeId);
        AnyURIAttribute resourceValue = new AnyURIAttribute(
                new RoleValueIdentifier(domain, role).toURI());
        resources.add(new Attribute(resourceId, null, null, resourceValue));

        URI actionId = new URI(ActionAttributeDesignatorBuilder.attributeId);
        AnyURIAttribute actionValue = new AnyURIAttribute(
                Assignment.enableRoleActionId);
        actions.add(new Attribute(actionId, null, null, actionValue));
        RequestCtx request = new RequestCtx(subjects, resources, actions,
                environments);
        return request;
    }

}
