/*
 * $Id: RBACPolicyFinderModule.java,v 1.7 2006/07/24 17:31:09 suhrbier Exp $
 * Created on 04.04.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACPolicyFinderModule.java,v $
 * Revision 1.7  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.6  2005/12/19 19:11:04  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.5  2005/12/09 11:01:28  suhrbier
 * 2 special PolicyFinder fr REA und PDP
 *
 * Revision 1.4  2005/11/01 11:50:45  suhrbier
 * RoleManager add, remove and list of users completed.
 *
 * Revision 1.3  2005/10/28 10:03:33  suhrbier
 * PolicyManager completed bis auf Comment.
 *
 * Revision 1.2  2005/10/27 18:26:05  suhrbier
 * RBACPolicyFinderModule completed.
 *
 * Revision 1.1  2005/10/27 10:21:04  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/07/25 15:30:07  suhrbier
 * HttpInputStream updated.
 *
 * Revision 1.1  2005/04/04 21:23:23  suhrbier
 * Erste lauffhige Version mit RoleEnableAuth und RBACPDP
 *
 * Revision 1.1  2005/04/04 17:20:56  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac.finder;

import java.io.File;
import java.io.FileFilter;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import nbi.util.FileUtil;
import nbi.xmlsec.xacml.profile.rbac.Identifier;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.io.filefilter.IOFileFilter;
import org.apache.commons.io.filefilter.SuffixFileFilter;
import org.apache.commons.io.filefilter.TrueFileFilter;
import org.apache.log4j.Logger;

import com.sun.xacml.AbstractPolicy;
import com.sun.xacml.EvaluationCtx;
import com.sun.xacml.MatchResult;
import com.sun.xacml.ParsingException;
import com.sun.xacml.ctx.Status;
import com.sun.xacml.finder.PolicyFinder;
import com.sun.xacml.finder.PolicyFinderModule;
import com.sun.xacml.finder.PolicyFinderResult;
import com.sun.xacml.finder.impl.FilePolicyModule;

/**
 * Implementation of the standard policy finder module for the RBAC profile.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */

public class RBACPolicyFinderModule extends PolicyFinderModule {

    private static final Logger logger = Logger
            .getLogger(RBACPolicyFinderModule.class);

    /** PolicyFinder initialised with this module */
    protected PolicyFinder policyFinder = null;

    /** Configuration part <code>PolicyFinderModule</code> for this module */
    protected Configuration configuration = null;

    /** The policy store of this policy Finder */
    protected Set policies = null;

    /**
     * @param config
     *            Configuration of the type <code>PolicyFinderModule</code>
     *            for this module.
     */
    public RBACPolicyFinderModule(Configuration config) {
        this.configuration = config;
        this.policies = new HashSet();
    }

    /**
     * Loads policies from a given directory. This method misses out policies,
     * where
     * {@link FilePolicyModule#loadPolicy(String, PolicyFinder) loadPolicy(String, PolicyFinder)}
     * returns <code>null</code>.
     * 
     * 
     * @param policyDir
     *            The directory to search for policies.
     * @param filter
     *            The {@link IOFileFilter IOFileFilter}to select policy files.
     * @param finder
     *            The {@link PolicyFinder PolicyFinder}to find policies.
     * @return Set with all policies retrieved, which may be empty.
     */
    public static List loadPolicies(File policyDir, IOFileFilter filter,
            PolicyFinder finder) {
        logger.debug("loadPolicies from " + policyDir + ".");
        List policies = new ArrayList();
        File[] policyFiles = null;
        if (policyDir.isDirectory()) {
            policyFiles = policyDir.listFiles((FileFilter) filter);
            for (int i = 0; i < policyFiles.length; i++) {
                AbstractPolicy policy = FilePolicyModule.loadPolicy(
                        policyFiles[i].toString(), finder);
                if (policy == null)
                    logger.debug("Error loading policy file "
                            + policyFiles[i].toString());
                else {
                    logger.debug("Policy file " + policyFiles[i].toString()
                            + " loaded.");
                    policies.add(policy);
                }
            }
        }
        return policies;
    }

    /**
     * Retrieves a FileFilter bases on the file suffixes in the configuration.
     * TODO: Add further FileFilter types, if needed.
     * 
     * @return {@link SuffixFileFilter SuffixFileFilter}, if suffixes are
     *         configured, otherwise {@link TrueFileFilter TrueFileFilter}.
     */
    public static IOFileFilter getFileFilter(Configuration config,
            IOFileFilter defaultFilter) {
        IOFileFilter result = defaultFilter;
        List fileSuffixes = config.getList("SuffixFileFilter");
        if (!fileSuffixes.isEmpty())
            result = new SuffixFileFilter(fileSuffixes);
        return result;
    }

    /**
     * Inits this PolicyFinder loading all policy files with the given file
     * suffixes from all subdirectories of the policy base dir.
     * 
     * @see PolicyFinderModule#init(PolicyFinder)
     */
    public void init(PolicyFinder finder) {
        this.policyFinder = finder;
        Iterator policyDirIt = FileUtil.findSubDirectories(
                new File(getPolicyBaseDir())).iterator();
        while (policyDirIt.hasNext())
            this.policies.addAll(loadPolicies((File) policyDirIt.next(),
                    getPolicyFileFilter(), policyFinder));
    }

    /**
     * Simply copied from
     * {@link FilePolicyModule#findPolicy(EvaluationCtx) FilePolicyModule.findPolicy(EvaluationCtx)}.
     * 
     * @see FilePolicyModule#findPolicy(EvaluationCtx)
     */
    public PolicyFinderResult findPolicy(EvaluationCtx context) {
        AbstractPolicy selectedPolicy = null;
        Iterator it = policies.iterator();

        while (it.hasNext()) {
            AbstractPolicy policy = (AbstractPolicy) (it.next());

            // see if we match
            MatchResult match = policy.match(context);
            int result = match.getResult();

            // if there was an error, we stop right away
            if (result == MatchResult.INDETERMINATE)
                return new PolicyFinderResult(match.getStatus());

            if (result == MatchResult.MATCH) {
                // if we matched before, this is an error...
                if (selectedPolicy != null) {
                    ArrayList code = new ArrayList();
                    code.add(Status.STATUS_PROCESSING_ERROR);
                    Status status = new Status(code, "too many applicable top-"
                            + "level policies");
                    return new PolicyFinderResult(status);
                }

                // ...otherwise remember this policy
                selectedPolicy = policy;
            }
        }

        // if we found a policy, return it, otherwise we're N/A
        if (selectedPolicy != null)
            return new PolicyFinderResult(selectedPolicy);
        else
            return new PolicyFinderResult();
    }

    /**
     * Signals that this policy finder module supports request.
     * 
     * @see PolicyFinderModule#isIdReferenceSupported()
     */
    public boolean isRequestSupported() {
        return true;
    }

    /**
     * Creates a {@link PolicyFinderResult PolicyFinderResult}instance with a
     * single code and message.
     * 
     * @param code
     *            The status code string (see
     *            {@link Status#STATUS_PROCESSING_ERROR Status}.
     * @param message
     *            The error message.
     * @return {@link PolicyFinderResult PolicyFinderResult}with
     *         {@link Status Status}.
     */
    protected PolicyFinderResult createErrorResult(String code, String message) {
        List statusCode = new ArrayList(1);
        statusCode.add(code);
        return new PolicyFinderResult(new Status(statusCode, message));
    }

    /**
     * Tries to find a policy based on the {@link Identifier Identifier}built
     * from the <code>idReference</code> parameter. Errors are documentated in
     * the Status of the {@link PolicyFinderResult PolicyFinderResult}return
     * value.
     * 
     * @see PolicyFinderModule#findPolicy(URI, int)
     */
    public PolicyFinderResult findPolicy(URI idReference, int type) {
        PolicyFinderResult result = new PolicyFinderResult();
        try {
            Identifier refId = Identifier.getInstance(idReference);
            AbstractPolicy policy = FilePolicyModule.loadPolicy(refId.toFile(
                    getPolicyBaseDir()).toString(), this.policyFinder);
            if (policy != null)
                result = new PolicyFinderResult(policy);
            else
                result = createErrorResult(Status.STATUS_PROCESSING_ERROR,
                        "Policy " + idReference.toString() + "not found at "
                                + refId.toFile(getPolicyBaseDir()).toString());
        } catch (ParsingException e) {
            result = createErrorResult(Status.STATUS_PROCESSING_ERROR,
                    "Unknown policy identifier: " + idReference.toString());
        }
        return result;
    }

    /**
     * Signals that this policy finder module supports policy references.
     * 
     * @see PolicyFinderModule#isIdReferenceSupported()
     */
    public boolean isIdReferenceSupported() {
        return true;
    }

    /**
     * Retrieves the policy base directory from the current configuration.
     * 
     * @return The base directory for all policies.
     */
    public String getPolicyBaseDir() {
        return configuration.getString("[@policyBaseDir]",
                FileUtil.currentDirectory);
    }

    /**
     * Sets the policy base directory in the current configuration.
     * 
     * @param policyBaseDir
     *            The base directory for all policies.
     */
    public void setPolicyBaseDir(String policyBaseDir) {
        configuration.addProperty("[@policyBaseDir]", policyBaseDir);
    }

    /**
     * Retrieves the domain from the current configuration.
     * 
     * @return The domain label for all policies to find.
     */
    public String getDomain() {
        return configuration.getString("[@domain]", FileUtil.currentDirectory);
    }

    /**
     * Sets the domain in the current configuration.
     * 
     * @param domain
     *            The domain label for all policies to find.
     */
    public void setDomain(String domain) {
        configuration.addProperty("[@domain]", domain);
    }

    /**
     * Retrieves a FileFilter bases on the file suffixes of the configuration.
     * 
     * @return {@link SuffixFileFilter SuffixFileFilter}, if suffixes are
     *         configured, otherwise {@link TrueFileFilter TrueFileFilter}.
     */
    public IOFileFilter getPolicyFileFilter() {
        return getFileFilter(this.configuration, TrueFileFilter.INSTANCE);
    }
}
