/*
 * $Id: PermissionMatchParser.java,v 1.2 2005/12/19 19:11:05 suhrbier Exp $
 * Created on 04.11.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PermissionMatchParser.java,v $
 * Revision 1.2  2005/12/19 19:11:05  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.1  2005/11/04 17:52:36  suhrbier
 * Permission add mit Targets funzt.
 *
 */
package nbi.xmlsec.xacml.profile.rbac.parser;

import java.net.URI;

import nbi.xmlsec.xacml.parser.TargetMatchParser;

import com.sun.xacml.ParsingException;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.cond.EqualFunction;
import com.sun.xacml.cond.MatchFunction;

/**
 * This class serves to parse specialised TargetMatches for Permissions from
 * match function identifiers into target scheme
 * descriptions(datatype-matchfunc[value]). The following functions are
 * supported:
 * <li>string-equal</li>
 * <li>anyuri-equal</li>
 * <li>x500Name-equal</li>
 * <li>string-match</li>
 * <li>x500Name-match</li>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */

public class PermissionMatchParser extends TargetMatchParser {

    /**
     * Retrieves target scheme descriptions(datatype-matchfunc[value]) from
     * function type URI's. The following functions are supported:
     * <li>string-equal</li>
     * <li>anyuri-equal</li>
     * <li>x500Name-equal</li>
     * <li>string-match</li>
     * <li>x500Name-match</li>
     * 
     * @param typeId The URI of the function to retrieve.
     * @return The corresponding target scheme description or null.
     */
    public String getDataTypeFunc(URI typeId) {
        String result = null;
        String funcId = typeId.toString();
        result = EqualFunction.NAME_STRING_EQUAL;
        if (EqualFunction.NAME_ANYURI_EQUAL.equals(funcId))
            result = "anyuri-equal";
        else if (EqualFunction.NAME_STRING_EQUAL.equals(funcId))
            result = "string-equal";
        else if (EqualFunction.NAME_X500NAME_EQUAL.equals(funcId))
            result = "x500Name-equal";
        else if (MatchFunction.NAME_REGEXP_STRING_MATCH.equals(funcId))
            result = "string-match";
        else if (MatchFunction.NAME_X500NAME_MATCH.equals(funcId))
            result = "x500Name-match";
        return result;
    }

    /*
     * Parses the target scheme descriptions(datatype-matchfunc[value]) match value as {@link String String} from the Target.
     * 
     * @see nbi.xmlsec.xacml.parser.TargetMatchParser#parseMatchValue()
     */
    public Object parseMatchValue() throws ParsingException {
        StringBuffer result = new StringBuffer();
        result.append(getDataTypeFunc(getMatchFunction().getIdentifier()));
        result.append("[");
        result.append(((StringAttribute) getMatchValue()).getValue());
        result.append("]");
        return result.toString();
    }
}
