/*
 * $Id: SyntaxScheme.java,v 1.6 2006/07/24 19:05:47 suhrbier Exp $
 * Created on 24.04.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: SyntaxScheme.java,v $
 * Revision 1.6  2006/07/24 19:05:47  suhrbier
 * Im Umbau von signedElements.
 *
 * Revision 1.5  2006/07/24 18:15:01  suhrbier
 * update JavaDoc
 *
 * Revision 1.4  2006/04/26 10:52:30  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2006/04/26 08:54:45  suhrbier
 * SyntaxScheme ready.
 *
 * Revision 1.2  2006/04/25 17:17:08  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2006/04/24 19:50:19  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.util;

import java.util.ArrayList;
import java.util.List;

/**
 * A parser for the syntax scheme used from the
 * {@link nbi.xmlsec.xacml.profile.rbac.RoleManager RoleManager} to parse
 * conditions from the command line. The syntax scheme to define resources,
 * actions or conditions for permissions is defined as follows:
 * <b>datatype-matchfunc[value] </b>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class SyntaxScheme {
    /** The match function name. */
    String function = null;

    /** A function, which is a value of the function. */
    String functionValue = null;

    /** The value of a function. */
    String value = null;

    /** The list of arguments of a function. */
    List args = new ArrayList();

    /**
     * Creates a syntax scheme by parsing the expression argument.
     * @param expression The expression to be parsed as a syntax scheme.
     */
    public SyntaxScheme(String expression) {
        parse(expression);
    }

    /**
     * Parses the expression into the syntax scheme.
     * 
     * @param expression The expression to be parsed as a syntax scheme.
     */
    private void parse(String expression) {
        parseFunction(expression, 0);
        if (function == null)
            value = expression;
        else {
            if (functionValue != null)
                parseArgs(functionValue, 0);
        }
    }

    /**
     * Parses the next function name until the opening bracket is reached
     * 
     * @param expression The expression to be parsed.
     * @param startInd The start index within expression.
     * @return The index after the closing bracket of the function, or -1.
     */
    private int parseFunction(String expression, int startInd) {
        int result = -1;
        int startArgs = expression.indexOf("[", startInd);
        int sep = expression.indexOf(",", startInd);
        if (sep < 0) {
            if (startArgs >= startInd) {
                function = expression.substring(0, startArgs);
                int iClosingBracket = indexOfClosingBracket(expression,
                        startArgs);
                if (iClosingBracket > (startArgs + 1))
                    functionValue = expression.substring(startArgs + 1,
                            iClosingBracket);
                result = iClosingBracket + 1;
            } else
                result = startInd;
        } else {
            if (sep > startArgs) {
                int iClosingBracket = indexOfClosingBracket(expression,
                        startArgs);
                if (sep < iClosingBracket) {
                    function = expression.substring(0, startArgs);
                    iClosingBracket = indexOfClosingBracket(expression,
                            startArgs);
                    if (iClosingBracket > (startArgs + 1))
                        functionValue = expression.substring(startArgs + 1,
                                indexOfClosingBracket(expression, startArgs));
                    result = startArgs;
                } else
                    result = startInd;
            } else
                result = startInd;
        }
        return result;
    }

    /**
     * Parses the arguments of a function into the list of {@link #args args}.
     * 
     * @param expression The expression to be parsed.
     * @param iStart The start index within expression.
     */
    private void parseArgs(String expression, int iStart) {
        int iStartArg = iStart;
        int iEndArg = indexOfNextArg(expression, iStartArg);

        if (iEndArg == -1)
            args.add(new SyntaxScheme(expression));
        else {
            while (iEndArg != -1) {
                SyntaxScheme scheme = new SyntaxScheme(expression.substring(
                        iStartArg, iEndArg));
                args.add(scheme);
                iStartArg = iEndArg + 1;
                iEndArg = indexOfNextArg(expression, iStartArg);
            }
        }

    }

    /**
     * Determines the index of the next argument within the given expression.
     * 
     * @param expression The expression to be parsed.
     * @param iStart The start index within expression.
     * @return The index after the next komma within the arguments of a function, or -1.
     */
    private int indexOfNextArg(String expression, int iStart) {
        int result = -1;
        int iSep = expression.indexOf(",", iStart);
        int iClosedBracket = indexOfClosingBracket(expression, iStart);
        int iOpenBracket = indexOfOpenBracket(expression, iStart);
        if ((iClosedBracket == -1) && (iSep == -1)) {
            if (expression.length() > iStart)
                result = expression.length();
        } else if (iClosedBracket == -1)
            result = iSep;
        else if (iSep == -1) {
            result = iClosedBracket + 1;
        } else {
            if ((iOpenBracket < iSep) && (iSep < iClosedBracket))
                result = iClosedBracket + 1;
            else if (iSep < iClosedBracket)
                result = iSep;
            else
                result = iClosedBracket + 1;
        }
        return result;
    }

    /**
     * Determines the index of the next opening or closing bracket.
     * 
     * @param expression The expression to be parsed.
     * @param iStartBracket The start index within expression.
     * @return The index of the next opening or closing bracket, or -1.
     */
    private int indexOfNextBracket(String expression, int iStartBracket) {
        int result = -1;
        int iOpenBracket = expression.indexOf("[", iStartBracket);
        int iClosedBracket = expression.indexOf("]", iStartBracket);
        if (iOpenBracket == -1)
            result = iClosedBracket;
        else if (iClosedBracket == -1)
            result = iOpenBracket;
        else
            result = Math.min(iOpenBracket, iClosedBracket);
        return result;
    }

    /**
     * Determines the index of the next opening bracket.
     * 
     * @param expression The expression to be parsed.
     * @param iStart The start index within expression.
     * @return The index of the next opening bracket, or -1.
     */
    private int indexOfOpenBracket(String expression, int iStart) {
        return expression.indexOf("[", iStart);
    }

    /**
     * Determines the index of the next closing bracket.
     * 
     * @param expression The expression to be parsed.
     * @param iStartBracket The start index within expression.
     * @return The index of the next closing bracket, or -1.
     */
    private int indexOfClosingBracket(String expression, int iStartBracket) {
        int result = -1;
        int iMax = expression.length();
        int nOpenBrackets = 0;
        int nClosedBrackets = 0;
        int iBracket = indexOfNextBracket(expression, iStartBracket);

        if (iBracket != -1) {
            if (expression.charAt(iBracket) == '[')
                nOpenBrackets++;
            else if (expression.charAt(iBracket) == ']')
                nClosedBrackets++;

            while ((nOpenBrackets > nClosedBrackets) && (iBracket < iMax)) {
                iBracket = indexOfNextBracket(expression, iBracket + 1);

                if (expression.charAt(iBracket) == '[')
                    nOpenBrackets++;
                else if (expression.charAt(iBracket) == ']')
                    nClosedBrackets++;

            }
            if (iBracket < iMax)
                result = iBracket;
        }
        return result;
    }

    /**
     * Gets the function name.
     * 
     * @return Returns the function name.
     */
    public String getFunction() {
        return function;
    }

    /**
     * Gets the datatype of the function.
     * 
     * @return Returns the datatype of the function.
     */
    public String getFunctionType() {
        String result = null;
        if (function != null) {
            int iSep = function.indexOf("-");
            if (iSep != -1)
                result = function.substring(0, iSep);
        }
        return result;
    }

    /**
     * Gets the functionValue.
     * 
     * @return Returns the functionValue.
     */
    public String getFunctionValue() {
        return functionValue;
    }

    /**
     * Gets the argument list of the function.
     * 
     * @return Returns the argument list.
     */
    public List getArgs() {
        return args;
    }

    /**
     * Gets the value of the function.
     * 
     * @return Returns the value of the function.
     */
    public String getValue() {
        return value;
    }
}
