/*
 * $Id: TargetUtil.java,v 1.4 2005/10/24 20:48:59 suhrbier Exp $
 * Created on 14.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: TargetUtil.java,v $
 * Revision 1.4  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.3  2005/10/24 09:38:39  suhrbier
 * Assignment test ok
 *
 * Revision 1.2  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/14 18:31:00  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.sun.xacml.Target;
import com.sun.xacml.TargetMatch;
import com.sun.xacml.attr.AttributeValue;
import com.sun.xacml.cond.Evaluatable;
import com.sun.xacml.cond.Function;

/**
 * Class to support parsing of {@link Target Target}instances.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

public class TargetUtil {

    /**
     * Returns the TargetMatch from the given target. If the retrieved target
     * match does not match the given type, <code>null</code> is returned.
     * 
     * @param target
     *            The target to be searches.
     * @param type
     *            The type of the target match (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param index
     *            The index of the target match in the list.
     * @return The target match requested, or <code>null</code>.
     */
    static public TargetMatch getTargetMatch(Target target, int type, int index) {
        TargetMatch result = null;
        List matches = getTargetMatchList(target, type);
        switch (type) {
        case TargetMatch.SUBJECT:
            matches = target.getSubjects();
            break;
        case TargetMatch.RESOURCE:
            matches = target.getResources();
            break;
        case TargetMatch.ACTION:
            matches = target.getActions();
            break;
        }
        if (matches != null)
            result = type == ((TargetMatch) matches.get(index)).getType() ? (TargetMatch) matches
                    .get(index)
                    : null;
        return result;
    }

    /**
     * Returns the list of target matches of the given type from the given
     * target.
     * 
     * @param target
     *            The target..
     * @param type
     *            The type of the target match (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return The list of target matches requested, or <code>null</code>.
     */
    static public List getTargetMatchList(Target target, int type) {
        List result = null;
        List matchList = null;
        switch (type) {
        case TargetMatch.SUBJECT:
            matchList = target.getSubjects();
            break;
        case TargetMatch.RESOURCE:
            matchList = target.getResources();
            break;
        case TargetMatch.ACTION:
            matchList = target.getActions();
            break;
        }
        if (matchList != null) {
            result = new ArrayList();
            Iterator matchIt = matchList.iterator();
            while (matchIt.hasNext()) {
                List targetMatchList = ((List) matchIt.next());
                result.addAll(targetMatchList
                        .subList(0, targetMatchList.size()));
            }
        }
        return result;
    }

    /**
     * Returns the number of target matches of the given type in the given
     * target.
     * 
     * @param target
     *            The target.
     * @param type
     *            The type of the target match (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @return The number of target matches.
     */
    static public int getTargetMatchSize(Target target, int type) {
        int result = 0;
        if (getTargetMatchList(target, type) != null)
            result = getTargetMatchList(target, type).size();
        return result;
    }

    /**
     * Returns the attribute value if the TargetMatch from the given target.
     * 
     * @param target
     *            The target to be searched.
     * @param type
     *            The type of the target match (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param index
     *            The index of the target match in the list.
     * @return The attribute value requested, or <code>null</code>.
     */
    static public AttributeValue getTargetMatchValue(Target target, int type,
            int index) {
        return getTargetMatch(target, type, index).getMatchValue();
    }

    /**
     * Returns the Evaluatable if the TargetMatch from the given target.
     * 
     * @param target
     *            The target to be searched.
     * @param type
     *            The type of the target match (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param index
     *            The index of the target match in the list.
     * @return The evaluatable requested, or <code>null</code>.
     */
    static public Evaluatable getTargetMatchEvaluatable(Target target,
            int type, int index) {
        return getTargetMatch(target, type, index).getMatchEvaluatable();
    }

    /**
     * Returns the match function if the TargetMatch from the given target.
     * 
     * @param target
     *            The target to be searched.
     * @param type
     *            The type of the target match (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param index
     *            The index of the target match in the list.
     * @return The evaluatable requested, or <code>null</code>.
     */
    static public Function getTargetMatchFunction(Target target, int type,
            int index) {
        return getTargetMatch(target, type, index).getMatchFunction();
    }

    /**
     * Verifies if the target is empty.
     * 
     * @param target
     *            The target to verify.
     * @return <code>true</code>, if the target is empty, <code>false</code>
     *         otherwise.
     */
    static public boolean isEmpty(Target target) {
        boolean result = true;
        if (target != null) {
            if ((target.getSubjects() != null)
                    || (target.getResources() != null)
                    || (target.getActions() != null))
                result = false;
        }
        return result;
    }

}
